% -----rbMIT Software Copyright MIT 2006-07----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% REDUCED BASIS APPROXIMATION AND A POSTERIORI ERROR ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% FOR PARAMETRIZED PARTIAL DIFFERENTIAL EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Anthony T. PATERA - Gianluigi ROZZA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% (C) M.I.T. Massachusetts Institute of Technology 2006-07 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
% function [du_DEF, du_OFFLINE, du_ONLINE] = Greedy_dual_coer_noncompliant(du_DEF, pr_DEF, du_OFFLINE, pr_OFFLINE, du_ONLINE, InsideOutsideD, Get_Theta_q)
%
% Part II, Coercive and Compliant Affine Linear Elliptic Problems
% 
%
% Optimization Algorithm for reduced basis assembling procedure and selection based
% on a posteriori error bounds and dual norm of residual.
% 
% INPUTS FOR THE PROBLEM TO BE SET AND/OR PROVIDED: 
%   - DEF.Q_affine:  
%      DEF.Q_affine.a: Q_a  Number of \theta_a^q's
%      DEF.Q_affine.f: Q_f  Number of \theta_f^q's
%      DEF.Q_affine.l: Q_l(optional, in this compliant case Q_l=Q_f)
%   - DEF.P: No.of mu's
%   - DEF.mu_min: minimum value of admissible parameters 
%   - DEF.mu_max: maximum value of admissible parameters  
%
% REDUCED BASIS AUXILIARY DATA STRUCTURE (DATA FOR GREEDY ASSEMBLING ALGORITHM)
% 
%
% - OFFLINE.sample.mu_samples : vectors of (quantities depending on) parameters: N_max x P
%             used to compute  coefficients of bilinear forms and rhs
%             (Q_a: No. of \theta_a^q, Q_f: No. of \theta_f^q, N_max:  max number of samples ) 
%             [if pure affine: \theta_a^q=\mu^q and \theta_f^q=\mu_f^q,
%             otherwise samples are calculated as \theta_a^q and \theta_f^q
%             using an intermediate handle function in the library "samples=Get_theta_q(DEF, mu_samples)" ]
%
% - OFFLINE.space.tol : tolerance on the max error bound to stop the iterative assembling procedure
%
% - OFFLINE.space.Nbarmax:  max number of basis functions
%
% - OFFLINE.FEM:  FINITE ELEMENT DATA STRUCTURE (OFFLINE TO PROVIDE)
%     FEM.matrix: 
%
%     FEM.matrix.Aq: {Q_a x \calN x \calN}
%     FEM.matrix.Fq: {Q_f x \calN x 1}
%     FEM.matrix.Lq: {Q_l x \calN x 1} (Lq=Fq))
%     FEM.matrix.Xnorm: [\calNx\calN]
%
%  OUTPUTS COMPUTED:
%       - REDUCED BASIS Auxiliary DATA STRUCTURE (Produced OFF-LINE  in greedy assembling algorithm and to be used ONLINE)
%              ONLINE.space.Nmax: number of basis fucntion selected and
%              stored 
%              ONLINE.space.eps_out_star: [1 x N_max] vector containg all the
%                                        max error bound computed in the samples at each iteration of
%                                        basis assembling algorithm.
%
%              OFFLINE.space.Mus: [NxP double] contains N vectors of P parameters of selected basis functions 
%              OFFLINE.space.theta_f: [NxQ_f double] contains N vectors of Q_f coeffients corresponding to rhs coefficients of selected basis functions 
%              OFFLINE.space.theta_a: [NxQ_a double] contains N vectors of Q_a coeffients corresponding to bilinear forms coefficients of selected basis functions 
%              OFFLINE.space.basisInds: [Nx2 double] contains indexes of selected basis
%                                functions among the samples provided and the number of the set of samples.
%
%              OFFLINE.space.Z: [\calN x N double] basis functions selected (\calN: nodof of FEM solution)
%                
%              OFFLINE.space.matrix: [1x1 struct]
%                    (pre-assembled offline)
%                    ->Zqf: {Q_f x \calN x 1}   Xnorm \ Fq{q} 
%                    ->Zqa: {Q_a x \cal N x  N} Xnorm \ (Aq{q}*Z) 
%
%          - ONLINE.RB  REDUCED BASIS DATA STRUCTURE (Produced OFF-LINE  in greedy assembling algorithm and to be used ONLINE)
%              ONLINE.RB.matrix
%                    (to be used online for error bounds)
%                    ->Zqfprime_Fq: {Q_f x 1 x 1}  Zqf'*Fq{q}
%                    ->Zqfprime_Aq_Z: {Q_a x 1 x N} Zqf'*Aq{q}*Z
%                    ->Zqa'_Aq_Z: {Q_a x N x N}   Zqa'*(Aq{q}*Z)
%                    (to be used online for RB solutions)
%                    ->Zqa'_Aq_Z: {N x N}   Zqa'*(Aq{q}*Z)
%                    ->Aqn: {Q_a x N x N} Z'*Aq{q}*Z
%                    ->FN: [Nx1]  Z'*Fq
%                    ->Y_norm_rb: [N x N] RB matrix for norm
%                                 calculation Z'*Y_norm*Z

function [du_DEF, du_OFFLINE, du_ONLINE, pr_ONLINE] = Greedy_dual_coer_noncompliant(du_DEF, pr_DEF, du_OFFLINE, pr_OFFLINE, du_ONLINE, pr_ONLINE, InsideOutsideD, Get_Theta_q, output_Get_Theta_q)

if (du_OFFLINE.space.restartflg~=1) 
du_OFFLINE.space.N=0;     
du_ONLINE.space.Nmax=0;    
end

% cuong-6/29
if strcmp(pr_DEF.type,'parabolic')    
    RB_Offline=str2func(du_OFFLINE.GREEDY.Offline);
    RB_Online=str2func(du_OFFLINE.GREEDY.Online);
    du_ONLINE.RB.timestep.nt=du_OFFLINE.FEM.timestep.nt;
    du_ONLINE.RB.timestep.dt=du_OFFLINE.FEM.timestep.dt;
end        
% cuong-6/29

% for q=1:pr_DEF.Q_affine.a 
%   Aq{q} = pr_OFFLINE.FEM.matrix.Aq{q}';
% end

 % thetamubar=feval(Get_Theta_q, DEF, DEF.mu_bar);
 % DEF.aux.thetamubar=thetamubar;
% if (OFFLINE.space.restartflg~=1) 
%   % fill-in, re-ordering and Cholesky factorization by:
%   Xnorm = Aq{1}*thetamubar(1);
%   for q=2:DEF.Q_affine.a
%   Xnorm = Xnorm + Aq{q}*thetamubar(q);
%   end
%   OFFLINE.FEM.matrix.Xnorm=Xnorm;
%   OFFLINE.FEM.matrix.HT=eye(size(Xnorm));
%  if (OFFLINE.space.restartflg==0) 
%   %fill-in and Cholesky factorization
%   m=symamd(Xnorm);
%   H=chol(Xnorm(m,m));
%   OFFLINE.FEM.matrix.Xnorm=H;
  
 % OFFLINE.FEM.matrix.HT=pr_OFFLINE.FEM.matrix.Xnorm';
  
  %same re-ordering for other FEM matrices and rhs
%   for q=1:DEF.Q_affine.a
%     Aq{q}=(Aq{q}(m,m));
%     OFFLINE.FEM.matrix.Aq{q}=Aq{q};
%   end
% if (du_OFFLINE.space.restartflg~=1) 
%    m=pr_OFFLINE.FEM.matrix.m;
%    for q=1:du_DEF.Q_affine.l
%    du_OFFLINE.FEM.matrix.Lq{q} =full(pr_OFFLINE.FEM.matrix.Fq{q}(m)); 
%    end
% end 
  %end
  %end
  %%%%%%%%%%%
du_ONLINE.space.absrelflg = du_OFFLINE.space.absrelflg;

if du_OFFLINE.space.sample.newflg==1.0
du_OFFLINE.space.sample.mu_samples=Samples_generation(du_OFFLINE.space.sample.size, InsideOutsideD, pr_DEF, du_OFFLINE.space.sample.densityflg, du_OFFLINE.space.sample.newflg); %possible generalization with log sample
du_ONLINE.coerlb=pr_ONLINE.coerlb;
du_ONLINE.coerlb.failedpoints=[ ];

%du_OFFLINE.space.sample.mu_samples=pr_OFFLINE.space.sample.mu_samples;
[alphaLB, alphaUB, rat, du_ONLINE]=alphaLB_coer(du_OFFLINE.space.sample.mu_samples, pr_DEF, du_ONLINE, Get_Theta_q);
du_ONLINE.coerlb.alphaLB=alphaLB;
pr_ONLINE.coerlb.failedpoints=[pr_ONLINE.coerlb.failedpoints; du_ONLINE.coerlb.failedpoints];
du_ONLINE.coerlb.failedpoints=[ ];
end
  
  du_OFFLINE.space.startPoint=1;
  du_OFFLINE.space.endPoint=du_OFFLINE.space.sample.size;     
  du_OFFLINE.space.sampleSetNo=1;
  tol=du_OFFLINE.space.tol;
  mu_samples=du_OFFLINE.space.sample.mu_samples;
  endPoint=du_OFFLINE.space.endPoint;
  startPoint=du_OFFLINE.space.startPoint;
  sampleSetNo=du_OFFLINE.space.sampleSetNo;
  du_OFFLINE.space.sample_in_basis = zeros(1,endPoint); 
  %generating coefficient of linear and bilinear form
  samples=feval(Get_Theta_q,pr_DEF, mu_samples);
  samples_l=feval(output_Get_Theta_q,du_DEF, mu_samples);
  %Checking 
  %[DEF]=Check_parcoer(samples,OFFLINE,DEF);
  
   cur_tol = 2*tol;
  
  
    if du_OFFLINE.space.N == 0
       %fist sample selection
    first_index =round(startPoint + rand*(endPoint-startPoint));
    if first_index < startPoint | first_index > endPoint
      first_index = ceil(0.5*(startPoint+endPoint));
    end
      % storing first sample as first basis function
    du_ONLINE.space.Mus = mu_samples(first_index,1:end);
    du_ONLINE.space.theta_a=samples(first_index,1:pr_DEF.Q_affine.a);
    du_ONLINE.space.theta_l=samples_l(first_index,1: du_DEF.Q_affine.l);
      
    du_ONLINE.space.basisInds = [sampleSetNo first_index];
    du_OFFLINE.space.sample_in_basis(first_index) = 1;
    
    % cuong 6/29
    flagHaveBases = 0;
    if strcmp(pr_DEF.type,'parabolic')
        [du_ONLINE, pr_ONLINE, du_OFFLINE]=feval(RB_Offline,first_index,samples,samples_l,du_OFFLINE,pr_OFFLINE,du_ONLINE,pr_ONLINE,du_DEF,pr_DEF,flagHaveBases);
    else
        [du_ONLINE, pr_ONLINE, du_OFFLINE]=coer_noncompliant_offline(first_index,samples,samples_l,du_OFFLINE, pr_OFFLINE, du_ONLINE, pr_ONLINE, du_DEF, pr_DEF,flagHaveBases);
    end
    % cuong 6/29
        
    du_ONLINE.space.eps_out_star=[];
    
   % [ONLINE.coerlb.alphaLB, alphaUB, rat, ONLINE]=alphaLB_coer(mu_samples, DEF, ONLINE, Get_Theta_q);
  end
  
    % iterative algorithm for basis assembling
     disp(['Greedy algorithm for dual problem']); 
  while (cur_tol > tol)
       du_ONLINE.space.Nmax=du_OFFLINE.space.N;
       if (du_ONLINE.space.Nmax >= du_OFFLINE.space.Nbarmax)
           du_ONLINE.space.Nmax=du_OFFLINE.space.N;

       disp(['WARNING : increase Nbar (max) or reduce the tolerance']);
       
       return;
           break;
       end
    all_err_bounds = zeros(1,endPoint);
    du_ONLINE.RB.matrix.X_norm_rb=du_OFFLINE.space.Z'*(pr_OFFLINE.FEM.matrix.HT*pr_OFFLINE.FEM.matrix.Xnorm)*du_OFFLINE.space.Z;
    for i_s = startPoint:endPoint
      if du_OFFLINE.space.sample_in_basis(i_s) ~= 1 
         if strcmp(pr_DEF.type,'parabolic')
            mul=mu_samples(i_s,:);
            Ndu=du_OFFLINE.space.N;
            alpha_LB=du_ONLINE.coerlb.alphaLB(i_s);
            sigma_LB=1;
            nt=pr_OFFLINE.FEM.timestep.nt;
            dt=pr_OFFLINE.FEM.timestep.dt;             
            [bN,normRdu]=feval(RB_Online, mul, Ndu, pr_DEF, du_DEF, du_ONLINE,...
                InsideOutsideD, Get_Theta_q, output_Get_Theta_q,1,2,0,pr_OFFLINE, du_OFFLINE);
            all_err_bounds(i_s)=sqrt((dt/alpha_LB)*sum(abs(normRdu(nt:-1:1))));
         else
            [all_err_bounds(i_s),du_ONLINE] = ...
              calc_res_du_coer_dual_norm(i_s,mu_samples,du_ONLINE, pr_ONLINE, du_DEF, pr_DEF, Get_Theta_q, output_Get_Theta_q);
         end       
           % all_true_err(i_s)=calc_bar_epsilon(i_s,samples,ONLINE,OFFLINE,PD);
           % eta(ONLINE.aux.N,i_s)=all_err_bounds(i_s)/all_true_err(i_s);
           % deltasN(ONLINE.aux.N,i_s)= all_err_bounds(i_s);
           % deltasNtrue(ONLINE.aux.N,i_s)=all_true_err(i_s)
      end
    end
    
    [max_err_bound,max_err_bound_index] =  max(all_err_bounds);
    cur_tol = max_err_bound;
   if  isempty(du_ONLINE.space.eps_out_star)
       du_ONLINE.space.eps_out_star = [cur_tol]; 
   else
    du_ONLINE.space.eps_out_star = [du_ONLINE.space.eps_out_star cur_tol]; 
   end
    
    disp(' ');
    disp([' N = ',num2str(du_OFFLINE.space.N),...
          ' Max err bound = ',num2str(max_err_bound),...
          ' Next mu = ',num2str(mu_samples(max_err_bound_index,:))]);
    disp(' ');
    
    % now look for the next basis function
   %if cur_tol > tol
      du_ONLINE.space.Mus(end+1,:) = mu_samples(max_err_bound_index,1:end);  
      du_ONLINE.space.theta_a(end+1,:) = samples(max_err_bound_index,1:pr_DEF.Q_affine.a);
      du_ONLINE.space.theta_l(end+1,:) = samples_l(max_err_bound_index, 1: du_DEF.Q_affine.l);
      du_ONLINE.space.basisInds(end+1,:) = [sampleSetNo max_err_bound_index];
      du_OFFLINE.space.sample_in_basis(max_err_bound_index) = 1; 
      % cuong 6/29
      flagHaveBases = 1;
      if strcmp(pr_DEF.type,'parabolic')
        [du_ONLINE, pr_ONLINE, du_OFFLINE]=feval(RB_Offline,max_err_bound_index,samples,samples_l,du_OFFLINE,pr_OFFLINE,du_ONLINE,pr_ONLINE,du_DEF,pr_DEF,flagHaveBases);      
      else
        [du_ONLINE, pr_ONLINE, du_OFFLINE]=coer_noncompliant_offline(max_err_bound_index,samples,samples_l,du_OFFLINE, pr_OFFLINE, du_ONLINE, pr_ONLINE, du_DEF, pr_DEF,flagHaveBases);
      end
      % cuong 6/29

      %ONLINE.space.Nmax=OFFLINE.space.N;
      save OFFLINE_recovery_du du_OFFLINE
      save ONLINE_recovery_du  du_ONLINE
      save ONLINE_recovery_pr pr_ONLINE
      %end    
    
    du_ONLINE.RB.matrix.X_norm_rb=du_OFFLINE.space.Z'*(pr_OFFLINE.FEM.matrix.HT*pr_OFFLINE.FEM.matrix.Xnorm)*du_OFFLINE.space.Z;
    
   end
   delete OFFLINE_recovery_du.mat
   delete ONLINE_recovery_du.mat
   delete ONLINE_recovery_pr.mat   
   
  return
  

  
  
  
  
  
  
