% -----rbMIT Software Copyright MIT 2006-07-----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% REDUCED BASIS APPROXIMATION AND A POSTERIORI ERROR ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% FOR PARAMETRIZED PARTIAL DIFFERENTIAL EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Anthony T. PATERA - Gianluigi ROZZA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% (C) M.I.T. Massachusetts Institute of Technology 2006-07 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function Online_RB_rcoer_compliant_many_queries(param_index, mu_min, mu_max, mu, N, epsdes, DEF, ONLINE, InsideOutsideD, Get_Theta_q)
%
% ONLINE
% 
% Inputs :-
%          param_index: vector of indexes of varying parameters, for ex: [2 3] for second and third parameter.
%          mu_min: vector containing the minimum value of each varying
%                  parameter [mu_min_1,...,mu_min_p]
%          mu_max: vector containing the maximum value of each varying
%                  parameter [mu_max_1,...,mu_max_p]
%          mu: all parameters (for the varying ones a random value is OK)
%          N : Size of Reduced Basis Space 
%
% Outputs :-
%           visualize the error bound and the output value as function of
%           one selected varying parameter (opt).
%
% Ranges :-
%          mu: <= mu_max and >= mu_min
%          N : <= N_max
% PartII coercive problems
function Online_RB_coer_compliant_many_queries(param_index, mu_min, mu_max, mu, N, epsdes, DEF, ONLINE, InsideOutsideD, Get_Theta_q)
  
 for q=1:size(mu_min,2) 
  
   if mu_min(q) < DEF.mu_min | mu_max(q) > DEF.mu_max
    error(['WARNING : Range of mu : ', num2str(DEF.mu_min), '-' ,num2str(DEF.mu_max)]);
   end
  
   if mu_max(q) < DEF.mu_min | mu_min(q) > DEF.mu_max
    error(['WARNING : Range of mu: ', num2str(DEF.mu_min), '-' ,num2str(DEF.mu_max)]);
   end
  
  if mu_min(q) > mu_max(q)
    error([' WARNING : mu_min > mu_max']);
  end
end
  
[InsideBoolean] = feval(InsideOutsideD, DEF, mu);

if InsideBoolean==0
   error(['WARNING : parameter value is out of range']);
end


  
  if N > ONLINE.space.Nmax
     error(['WARNING : N should be smaller than ', num2str(ONLINE.space.Nmax)]);
  end
 
 if  (N==0)
     if (min(ONLINE.space.eps_out_star(1:end-1))<epsdes)
     N=min(find(ONLINE.space.eps_out_star<epsdes))
     else
     disp(['epsdes is too small for given approximation']);
     disp(['output and error bound are given for Nmax!!!']);
          N=ONLINE.space.Nmax;
     end
 end
  
for q=1:size(mu_min,2)
    if (mu_min(q)==0.0)
        mu_min(q)=10^(-12);
    end
    if(mu_min(q)<0)
   mu_val{q} = linspace((mu_min(q)),(mu_max(q)),20);
   lpl=1.0;
    else
   mu_val{q} = 10.^linspace(log10(mu_min(q)),log10(mu_max(q)),20);
   lpl=0.0;
    end
  mu_value=mu_val{q};
end
 
  allSN = [];allE = [];
  for i_val = 1: size(mu_value,2)
   sol_bound=101.0;
  if epsdes==0.0
     epsdesloc=100.0;
 else 
     epsdesloc=epsdes;
  end
  iter=1.0;
while (sol_bound>epsdesloc)
  if iter>1
      N=N+3
      if N>ONLINE.space.Nmax
          disp(['Reduce the accuracy or enrich the basis, epsdes is too small and Nmax not enough']);
          disp(['output and error bound are given for Nmax!!!']);
          N=ONLINE.space.Nmax;
      end
  end    
      
    for q=1:size(mu_min,2)
        mu(param_index)=mu_val{q}(i_val);
    sample = feval(Get_Theta_q, DEF, mu);
    theta = sample(1:DEF.Q_affine.a);
    theta_f = sample(1+DEF.Q_affine.a:end);
    end
    
    
     AN = theta(1)*ONLINE.RB.matrix.Aqn{1}(1:N,1:N);
     for q=2:DEF.Q_affine.a
       AN = AN + theta(q)*ONLINE.RB.matrix.Aqn{q}(1:N,1:N);
     end
     
    FN = ONLINE.RB.matrix.FN{1}(1:N)*theta_f(1);
  for q=2:DEF.Q_affine.f
    FN = FN + ONLINE.RB.matrix.FN{q}(1:N)*theta_f(q);
  end
  
  uN = AN\FN;
allSN(i_val) = uN' *FN;

     
 %\alpha lower bound
 
 
 [alpha, alphaUB, rat, ONLINE]=alphaLB_coer(mu, DEF, ONLINE, Get_Theta_q);  
  %res_ff = RB_pr.res_ff;
   res_ff = 0;
    for q=1:DEF.Q_affine.f
      for qprime=1:DEF.Q_affine.f
        res_ff = res_ff + theta_f(q)*theta_f(qprime)*ONLINE.RB.matrix.Zqfprime_Fq{q,qprime};
      end
    end
    

  res_af = 0;
  for q=1:DEF.Q_affine.a
    for qprime=1:DEF.Q_affine.f
      res_af = res_af + theta(q)*theta_f(qprime)*...
               ONLINE.RB.matrix.Zqfprime_Aq_Z{q,qprime}(1,1:N)*uN;
    end
  end
  
  res_aa = 0;
  for q = 1:DEF.Q_affine.a
    for qprime = 1:DEF.Q_affine.a
      res_aa = res_aa + theta(q)*theta(qprime)*...
               (uN'*ONLINE.RB.matrix.Zqaprime_Aq_Z{q,qprime}(1:N,1:N)*uN);
    end
  end  
  
  res_dual_norm = real(res_ff -2*res_af + res_aa);
  
     
     allE(i_val) = res_dual_norm / alpha;
      
  if ONLINE.space.absrelflg==0
  sol_bound = allE(i_val);
  else
  sol_bound = allE(i_val)./(allSN(i_val)); 
  end
     iter=iter+1;
     if (N==ONLINE.space.Nmax)
      break;
     end
 end
  end
  %select a range of parameters to plot
  if (lpl==0)
  errorbar(log10(mu_value), allSN, allE);
   xlabel(['log_{10}(mu_{',num2str(param_index),'})']);
  else
  errorbar((mu_value), allSN, allE);
   xlabel(['mu_{',num2str(param_index),'}']);
  end
  axis tight;
  set(gca,'FontSize',14);
  set(gca,'FontWeight','bold');    
  %xlabel(['log_{10}(mu_{',num2str(param_index),'})']);
  ylabel('s_{N}');
  if ONLINE.space.absrelflg==0
   title('Output Error')
  else
   title('Output Error')
  end

  
  if (0)
    %display the right ticks
    set(gca,'FontSize',12);
    f = get(gca,'XTick');
    f=10.^f;f=round(f*1000)/1000;lab = num2str(f');
    set(gca,'XTickLabel',lab);
  end
  return;