% -----rbMIT Software Copyright MIT 2006-07----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% REDUCED BASIS APPROXIMATION AND A POSTERIORI ERROR ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% FOR PARAMETRIZED PARTIAL DIFFERENTIAL EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Anthony T. PATERA - Gianluigi ROZZA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% (C) M.I.T. Massachusetts Institute of Technology 2006-07 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [sN, DeltaN]=Online_RB_coer_noncompliant(mu, N, epsdes, DEF, ONLINE, InsideOutsideD, Get_Theta_q, output_Get_Theta_q) 
%  
% ONLINE
% Inputs :-
%          mu: parameters (P)
%          N : Size of Reduced Basis Space 
%
% Outputs :-
%          Print on the screen:
%          the Associated Error Bound  
%          to indicate accuracy of 
%          reduced basis approximation and norm of the solution and also
%          cputime of calculation
%
% Ranges :-
%          mu: <= mu_max and >= mu_min
%          N : <= N_max
% PartII coercive problems 
  
function [sN, sol_bound]=Online_RB_coer_noncompliant(mu, N_pr, N_du, pr_DEF, du_DEF, pr_ONLINE, du_ONLINE, InsideOutsideD, Get_Theta_q, output_Get_Theta_q) 

  

 
  
%  paramter range check
%   if (min(mu) < DEF.mu_min)
%     error(['WARNING : all parameters have to be >= ', num2str(DEF.mu_min)]);
%   end
%   
% %
%   if (max(mu) > DEF.mu_max)
%     error(['WARNING : all parameters have to be <= ', num2str(DEF.mu_max)]);
%   end
%   

[InsideBoolean] = feval(InsideOutsideD, pr_DEF, mu);

if InsideBoolean==0
   error(['WARNING : parameter value is out of range']);
end
  if (N_pr > pr_ONLINE.space.Nmax)
     error(['Size of reduced basis space (primal) should be smaller than Nmax_pr']);
 end
 
 if (N_du > du_ONLINE.space.Nmax)
     error(['Size of reduced basis space (dual) should be smaller than Nmax_du']);
 end
 
%  if  (N==0)
%      if (min(ONLINE.space.eps_out_star(1:end-1))<epsdes)
%      N=min(find(ONLINE.space.eps_out_star<epsdes))
%      else
%      disp(['epsdes is too small for given approximation']);
%      disp(['output and error bound are given for Nmax!!!']);
%      N=ONLINE.space.Nmax;
%      end
%  end
  sample=feval(Get_Theta_q, pr_DEF, mu);
  sample_l=feval(output_Get_Theta_q, du_DEF, mu);
  theta_a = sample(1:pr_DEF.Q_affine.a);
  theta_f = sample(1+pr_DEF.Q_affine.a: pr_DEF.Q_affine.f+pr_DEF.Q_affine.a);
  theta_l = sample_l(1: du_DEF.Q_affine.l);
  
  t_rb_sol = cputime;
  sol_bound=101.0;
%   if epsdes==0.0
%    epsdes=100.0;
%   end
%   iter=1.0;
%while (sol_bound>epsdes)
%   if iter>1
%       N=N+3;
%       if N>ONLINE.space.Nmax
%           disp(['Reduce the accuracy or enrich the basis, epsdes is too small and Nmax not enough']);
%           disp(['output and error bound are given for Nmax!!!']);
%           N=ONLINE.space.Nmax;
%       end
%   end
  %%%%  solve RB primal system %%%%%%%%%
  AN = theta_a(1)*pr_ONLINE.RB.matrix.Aqn{1}(1:N_pr,1:N_pr);
  for q=2:pr_DEF.Q_affine.a
    AN = AN + theta_a(q)*pr_ONLINE.RB.matrix.Aqn{q}(1:N_pr,1:N_pr);
  end
  
  FN = pr_ONLINE.RB.matrix.FN{1}(1:N_pr)*theta_f(1);
  for q=2:pr_DEF.Q_affine.f
    FN = FN + pr_ONLINE.RB.matrix.FN{q}(1:N_pr)*theta_f(q);
  end
  
  LNs = pr_ONLINE.RB.matrix.LN{1}(1:N_pr)*theta_l(1);
   for q=2:du_DEF.Q_affine.l
    LNs = LNs + pr_ONLINE.RB.matrix.LN{q}(1:N_pr)*theta_l(q);
  end
  
  uN = AN\FN;

  sN_pr = -uN' * LNs;
  
  %%%%  solve RB dual system %%%%%%%%%
  
  AN = theta_a(1)*du_ONLINE.RB.matrix.Aqn{1}(1:N_du,1:N_du);
  for q=2:pr_DEF.Q_affine.a
    AN = AN + theta_a(q)*du_ONLINE.RB.matrix.Aqn{q}(1:N_du,1:N_du);
  end
  
  LN = du_ONLINE.RB.matrix.LN{1}(1:N_du)*theta_l(1);
  for q=2:du_DEF.Q_affine.l
    LN = LN + du_ONLINE.RB.matrix.LN{q}(1:N_du)*theta_l(q);
  end
  
  FNs = du_ONLINE.RB.matrix.FN{1}(1:N_du)*theta_f(1);
   for q=2:pr_DEF.Q_affine.f
    FNs = FNs + du_ONLINE.RB.matrix.FN{q}(1:N_du)*theta_f(q);
  end
  
  psiN = AN\LN;

  sN_du = -psiN' * FNs;
  
  %%%%%%output correction
  
  
  ANs = theta_a(1)*du_ONLINE.RB.matrix.Aqns{1}(1:N_pr,1:N_du);
  for q=2:pr_DEF.Q_affine.a
    ANs = ANs + theta_a(q)*du_ONLINE.RB.matrix.Aqns{q}(1:N_pr,1:N_du);
  end
  sN_corr=uN'*ANs*psiN;
  sN=sN_pr+sN_du+sN_corr;
  
  %%%%%%  end solve RB system %%%%%%%%%
  
  t_rb_sol = cputime - t_rb_sol;

  % alpha lower bound
  
  time=cputime;
  [alpha, alphaUB, rat, pr_ONLINE]=alphaLB_coer(mu, pr_DEF, pr_ONLINE, Get_Theta_q);
  tot=cputime-time;
  %% error estimation %%%
  t_rb_err = cputime;
  
  %res_ff = RB_pr.res_ff;
  
  res_ff = 0;
    for q=1:pr_DEF.Q_affine.f
      for qprime=1:pr_DEF.Q_affine.f
        res_ff = res_ff + theta_f(q)*theta_f(qprime)*pr_ONLINE.RB.matrix.Zqfprime_Fq{q,qprime};
      end
    end

  res_af = 0;
  for q=1:pr_DEF.Q_affine.a
    for qprime=1:pr_DEF.Q_affine.f
      res_af = res_af + theta_a(q)*theta_f(qprime)*...
               pr_ONLINE.RB.matrix.Zqfprime_Aq_Z{q,qprime}(1,1:N_pr)*uN;
    end
  end
  
  res_aa = 0;
  for q = 1:pr_DEF.Q_affine.a
    for qprime = 1:pr_DEF.Q_affine.a
      res_aa = res_aa + theta_a(q)*theta_a(qprime)*...
               (uN'*pr_ONLINE.RB.matrix.Zqaprime_Aq_Z{q,qprime}(1:N_pr,1:N_pr)*uN);
    end
  end  
  
  pr_res_dual_norm = sqrt(real(res_ff -2*res_af + res_aa));
  
   res_ll = 0;
    for q=1:du_DEF.Q_affine.l
      for qprime=1:du_DEF.Q_affine.l
        res_ll = res_ll + theta_l(q)*theta_l(qprime)*du_ONLINE.RB.matrix.Zqlprime_Lq{q,qprime};
      end
    end

  res_al = 0;
  for q=1:pr_DEF.Q_affine.a
    for qprime=1:du_DEF.Q_affine.l
      res_al = res_al + theta_a(q)*theta_l(qprime)*...
               du_ONLINE.RB.matrix.Zqlprime_Aq_Z{q,qprime}(1,1:N_du)*psiN;
    end
  end
  
  res_aa = 0;
  for q = 1:pr_DEF.Q_affine.a
    for qprime = 1:pr_DEF.Q_affine.a
      res_aa = res_aa + theta_a(q)*theta_a(qprime)*...
               (psiN'*du_ONLINE.RB.matrix.Zqaprime_Aq_Z{q,qprime}(1:N_du,1:N_du)*psiN);
    end
  end  
  
  du_res_dual_norm = sqrt(real(res_ll -2*res_al + res_aa));
  
  
  
  
  
  delta_s = pr_res_dual_norm*du_res_dual_norm/alpha;
  
  t_rb_err = cputime - t_rb_err;

  
  %u_RB = OFFLINE.space.Z(:,1:N)*uN;
  
  rb_norm = sqrt(uN'*pr_ONLINE.RB.matrix.X_norm_rb(1:N_pr,1:N_pr)*uN);
  Delta_u = real(sqrt(pr_res_dual_norm)/alpha);
  Delta_u_2 = Delta_u/(rb_norm - Delta_u);
%   if pr_ONLINE.space.absrelflg==0
  sol_bound = delta_s;
%   else
%   sol_bound = delta_s/(sN); 
%   end

  %iter=iter+1;
%   if (N==ONLINE.space.Nmax)
%       return;
%   end
  %end
  %disp(['N = ',num2str(N_pr),' rb_norm = ',num2str(rb_norm),' Delta_u = ',num2str(Delta_u), ' Delta_s = ', num2str(delta_s), ' res_dual_norm = ', num2str(pr_res_dual_norm)]);
  return