% -----rbMIT Software Copyright MIT 2006-07----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% REDUCED BASIS APPROXIMATION AND A POSTERIORI ERROR ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% FOR PARAMETRIZED PARTIAL DIFFERENTIAL EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Anthony T. PATERA - Gianluigi ROZZA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% (C) M.I.T. Massachusetts Institute of Technology 2006-07%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
% function [DEF, OFFLINE, ONLINE] = SCM_coer(DEF, OFFLINE, ONLINE, InsideOutsideD, Get_Theta_q)
%
% Part II, Coercive and Compliant Affine Linear Elliptic Problems
% 
%
% SCM -Successive Constraints Method for the approximation of the
% coercivity constant for the a posteriori error bounds.

function [DEF, OFFLINE, ONLINE]=SCM_coer(DEF, OFFLINE, ONLINE, InsideOutsideD, Get_Theta_q)


 Aq = OFFLINE.FEM.matrix.Aq;
  thetamubar=feval(Get_Theta_q, DEF, DEF.mu_bar);
   for q=1:DEF.Q_affine.a
 Bq{q}= 0.5*(OFFLINE.FEM.matrix.Aq{q}+OFFLINE.FEM.matrix.Aq{q}');
end
  Xnorm = Bq{1}*thetamubar(1);
  for q=2:DEF.Q_affine.a
  Xnorm = Xnorm + Bq{q}*thetamubar(q);
  end
  %lambda L2
OPTS.tol = 1e-4;
OPTS.issym = 1;
OPTS.isreal = 1;
OPTS.disp = 0;
    
lambda = eigs(Xnorm,OFFLINE.FEM.matrix.M_ref, 1,'SM',OPTS);

  XLnorm= Xnorm+lambda*OFFLINE.FEM.matrix.M_ref;
  m=symamd(XLnorm);
 % m=[1:1:size(XLnorm,1)];
  Xnorm=Xnorm(m,m);
  XLnorm=XLnorm(m,m);
  HL=chol(XLnorm);
  OFFLINE.FEM.matrix.Xnorm=HL;
  
  %same re-ordering for other FEM matrices and rhs
  for q=1:DEF.Q_affine.a
    Bq{q}=Bq{q}(m,m);
    Aq{q}=(Aq{q}(m,m));
  end
 for q=1:DEF.Q_affine.f
   Fq{q} =full(OFFLINE.FEM.matrix.Fq{q}(m)); 
 end
  OFFLINE.FEM.matrix.m=m;
 
  %\Xi_train^\alpha

if OFFLINE.coerlb.sample.newflg==1.0
OFFLINE.coerlb.sample.mu_samples=Samples_generation(OFFLINE.coerlb.sample.size, InsideOutsideD, DEF, OFFLINE.coerlb.sample.densityflg, OFFLINE.coerlb.sample.newflg); %possible generalization with log sample
end
 disp(['SCM greedy algorithm for \alpha lower bounds']); 

%funcS = inline('K\(Y*(transpose(K)\x))','x','K','Y');
%funcL = inline('K*(Y\(transpose(K)*x))','x','K','Y');

OPTS.tol = 1e-4;
OPTS.issym = 1;
OPTS.isreal = 1;
OPTS.disp = 0;
OPTS.maxit =400;
for q = 1:DEF.Q_affine.a
    D = eigs(Bq{q},XLnorm, 2,'BE',OPTS);
   % tic
%     [DD, DDlb, VV, eigiter] = lanczos_alpha(Bq{q},Xnorm,1,OPTS.maxit,OPTS.tol,0);
%     DD
%     [MM]=eigpower(Bq{q}*inv(XLnorm),OPTS.tol,OPTS.maxit,ones(size(Aq{q},1),1));
%     MM
     ONLINE.coerlb.sigmaLB(q) = min(D);
     ONLINE.coerlb.sigmaUB(q) = max(D);
 %    ONLINE.coerlb.sigmaLB(q) = DD;
 %   ONLINE.coerlb.sigmaUB(q) = MM;
%     if (ONLINE.coerlb.sigmaLB(q)<=-10^(-9))
%         error(['The problem is NOT coercive, check mathematical formulation and matrices'])
%    end
end
%return
% generate train sample set
sample=OFFLINE.coerlb.sample.mu_samples;
THETA=feval(Get_Theta_q, DEF, sample);
THETA_Q=THETA(:,1:DEF.Q_affine.a);

maxit = OFFLINE.coerlb.Kbarmax;
if (OFFLINE.coerlb.restartflg==0)
OFFLINE.coerlb.new_mu_bar_index = floor(length(sample)/2);
ONLINE.coerlb.mu_bar=[ ];
%invect=[ ];
init=1;
else
init= size(ONLINE.coerlb.mu_bar,1)+1;  
end
if (OFFLINE.coerlb.Kloc==0)
    OFFLINE.coerlb.Kloc=2^(DEF.P);
end
ONLINE.coerlb.alpha_tol=OFFLINE.coerlb.alpha_tol;
ONLINE.coerlb.Kloc=OFFLINE.coerlb.Kloc;
ONLINE.coerlb.Jloc=OFFLINE.coerlb.Jloc; 
ONLINE.coerlb.mu_samples=OFFLINE.coerlb.sample.mu_samples; 
Kloc = OFFLINE.coerlb.Kloc; % M_alpha
Jloc = OFFLINE.coerlb.Jloc; %M_+


failed_sample=OFFLINE.coerlb.failedpoints;
sample=[sample; failed_sample];
OFFLINE.coerlb.sample.mu_samples=sample;
OFFLINE.coerlb.sample.size=size(sample);
OFFLINE.coerlb.failedpoints=[];
OPTS.tol = 1e-2;
OPTS.issym = 1;
OPTS.isreal = 1;
OPTS.disp = 0;
OPTS.maxit = 400;
for iter = init:maxit
    % compute new mu_bar data and add failed points
    tic;
%      if (size(failed_sample,1)==0)
%       ONLINE.coerlb.mu_bar=[ONLINE.coerlb.mu_bar; sample(OFFLINE.coerlb.new_mu_bar_index,1:end)];
%       OFFLINE.coerlb.failedpoints=[];
%      else
%       ONLINE.coerlb.mu_bar=[ONLINE.coerlb.mu_bar; failed_sample(1,:)] ;
%       temp=failed_sample;
%       failed_sample=[];
%        if (size(temp,1)>1)
%        failed_sample=temp(2:end,:)
%        OFFLINE.coerlb.failedpoints=failed_sample;
%        end
%        if (size(temp,1)==1)
%        OFFLINE.coerlb.failedpoints=[];
%        end
%      end 
    ONLINE.coerlb.mu_bar=[ONLINE.coerlb.mu_bar; sample(OFFLINE.coerlb.new_mu_bar_index,1:end)];  
    
    THETA_bar_new=feval(Get_Theta_q, DEF, ONLINE.coerlb.mu_bar(iter,1:end));
    THETA_Q_bar_new = THETA_bar_new(:,1:DEF.Q_affine.a);
    K_bar = 0*Bq{1};
    for q = 1:DEF.Q_affine.a
        K_bar = K_bar + THETA_Q_bar_new(q)*Bq{q};
    end
  
    
%     % using MATLAB eigs
%     [VV, DD] = eigs(K_bar,Y, 2,'SM',OPTS);
    [DD, DDlb, VV, eigiter] = lanczos_alpha(K_bar,HL,1,OPTS.maxit,OPTS.tol,0);
    ONLINE.coerlb.alpha_bar(iter) = DD(end,end);
    ONLINE.coerlb.xi_bar(:,iter) = VV(:,end);
    if (ONLINE.coerlb.alpha_bar(iter)<-10^(-12))
           disp(['Warning: Coercivity constant \alpha is zero or negative: check the mathematical formulation'])
        end
    % compute zval for gammaUB calculation
    for j = 1:DEF.Q_affine.a
        xval = ONLINE.coerlb.xi_bar(:,iter);
        ONLINE.coerlb.zeta_bar(iter,j) =xval'*(Bq{j}'*xval);
    end
    zval=ONLINE.coerlb.zeta_bar;
    eigtime = toc;
    % compute LB
    THETA=feval(Get_Theta_q, DEF, sample);
    THETA_Q = THETA(:,1:DEF.Q_affine.a);
    THETA=feval(Get_Theta_q, DEF, ONLINE.coerlb.mu_bar);
    THETA_Q_bar = THETA(:,1:DEF.Q_affine.a);
    tic;
    
prefailedpoints=[ ];
    for i = 1:size(sample,1)
        num_mu_bar = min(size(ONLINE.coerlb.mu_bar,1),Kloc);
         % get the "active" mu_bar list
        d = zeros(size(ONLINE.coerlb.mu_bar,1),1);
        for j = 1:size(ONLINE.coerlb.mu_bar,1)
            d(j) = norm(sample(i,:)-ONLINE.coerlb.mu_bar(j,:),2);
        end
        [temp, index] = sort(d);
        
        % num_mu_near is the number of 0 constraint points around a sample point
        num_mu_near = Jloc;
        % get the "active" mu_bar list
        d = zeros(size(sample,1),1);
        for j = 1:size(sample,1)
            d(j) = norm(sample(i,:)-sample(j,:),2);
        end
        [temp, mu_near_index] = sort(d);
        mu_near = sample(mu_near_index(2:num_mu_near+1),:);
        
        if num_mu_near > 0
             THETA=feval(Get_Theta_q, DEF, mu_near);
            THETA_Q_near =THETA(:,1:DEF.Q_affine.a);
        end
        
        f = THETA_Q(i,:)';
        A = zeros(num_mu_bar+num_mu_near+DEF.Q_affine.a*2,DEF.Q_affine.a);
        b = zeros(1,num_mu_bar+num_mu_near+DEF.Q_affine.a*2);
        for j = 1:num_mu_bar
            A(j,:) = -THETA_Q_bar(index(j),:);
            b(j) = -ONLINE.coerlb.alpha_bar(index(j));
        end
        % mu_near constraints
        for j = 1:num_mu_near
            A(num_mu_bar+j,:) = -THETA_Q_near(j,:);
            b(num_mu_bar+j) = 0;
        end
        for j = 1:DEF.Q_affine.a
            A(num_mu_bar+num_mu_near+(j-1)*2+1,j) = 1;
            A(num_mu_bar+num_mu_near+(j-1)*2+2,j) = -1;
            b(num_mu_bar+num_mu_near+(j-1)*2+1) = ONLINE.coerlb.sigmaUB(j);
            b(num_mu_bar+num_mu_near+(j-1)*2+2) = -ONLINE.coerlb.sigmaLB(j);
        end
        options = optimset('display','off');
        [zopt,fval] = linprog(f,A,b,[],[],[],[],[],options);
        alphaLB(i) = fval;
        % calculate sigmaUB
        alphaUB(i) = 1e5;
        for j = 1:size(ONLINE.coerlb.mu_bar,1)
            temp = zval(j,:)*THETA_Q(i,:)';
            if alphaUB(i) > temp
               alphaUB(i) = temp;
            end
        end
        rat(i) = (alphaUB(i)-alphaLB(i))/alphaUB(i);
        
        if (rat(i)>1+OPTS.tol)
           prefailedpoints=[prefailedpoints; sample(i,:)];
        end
        
        
    end
    % avg time for one LP solver
    avgtime = toc/(size(sample,1));
    % get new mu_bar
    [temp, OFFLINE.coerlb.new_mu_bar_index] = max(rat);
    if DEF.plotdemo==1
        close all
        figure()
    hold off;
    plot(alphaLB,'-b');
    hold on;
    plot(alphaUB,'-g');    
    if exist('alpha_true')
        plot(alpha_true,'-r');
    end
    drawnow;
    fprintf('iter = %d, [max_rat, eigtime, eigiter, avgtime] = [%f, %f, %d, %f]\n', iter, temp, eigtime, eigiter, avgtime);
    disp(['plotted UB and LB for \alpha at iteration n=',num2str(iter)])
    disp(['Press any key']);
    
    pause
    close all
    
   else
    
    fprintf('iter = %d, [max_rat, eigtime, eigiter, avgtime] = [%f, %f, %d, %f]\n', iter, temp, eigtime, eigiter, avgtime);
    
    end
    %fprintf('iter = %d, [max_rat, eigtime, eigiter, avgtime] = [%f, %f, %d, %f]\n', iter, temp, eigtime, eigiter, avgtime);
    save OFFLINE_recovery OFFLINE;
     save ONLINE_recovery  ONLINE;
    if temp <= OFFLINE.coerlb.alpha_tol
        OFFLINE.coerlb.failedpoints=[OFFLINE.coerlb.failedpoints; prefailedpoints];
        delete OFFLINE_recovery.mat;
        delete ONLINE_recovery.mat;
        if size(prefailedpoints,1)>0
           disp(['Warning: tolerance is not satisfied in SCM for every sample points, the LB is zero or negative, we put these points in the black list, sample i=',num2str(i)])
        end
       return
    end    
     
   if (iter==maxit)
       disp(['WARNING:Increase Kbarmax and restart'])
   end
end

delete OFFLINE_recovery.mat;
delete ONLINE_recovery.mat;
return