% -----rbMIT Software Copyright MIT 2006-07----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% REDUCED BASIS APPROXIMATION AND A POSTERIORI ERROR ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% FOR PARAMETRIZED PARTIAL DIFFERENTIAL EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Anthony T. PATERA - Gianluigi ROZZA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% (C) M.I.T. Massachusetts Institute of Technology 2006-07%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
% function [alphaLB, alphaUB, rat, ONLINE]=alphaLB_coer(sample,DEF, ONLINE, InsideOutsideD, Get_Theta_q)
%
% Part II, Coercive and Compliant Affine Linear Elliptic Problems
% 
%
% Online evaluation of \alphaLB by SCM -Successive Constraints Method for the approximation of the
% coercivity constant for the a posteriori error bounds.


function [alphaLB, alphaUB, rat, ONLINE]=alphaLB_coer(sample, DEF, ONLINE, Get_Theta_q)
Kloc = ONLINE.coerlb.Kloc;% M_alpha
Jloc = ONLINE.coerlb.Jloc; %M_+
tol=ONLINE.coerlb.alpha_tol;
    % compute LB
    THETA=feval(Get_Theta_q, DEF, sample);
    THETA_Q = THETA(:,1:DEF.Q_affine.a);
    THETA=feval(Get_Theta_q, DEF, ONLINE.coerlb.mu_bar);
    THETA_Q_bar = THETA(:,1:DEF.Q_affine.a);
    
    xval = ONLINE.coerlb.xi_bar;
    zval = ONLINE.coerlb.zeta_bar;

    for i = 1:size(sample,1)
        num_mu_bar = min(size(ONLINE.coerlb.mu_bar,1),Kloc);
         % get the "active" mu_bar list
        d = zeros(size(ONLINE.coerlb.mu_bar,1),1);
        for j = 1:size(ONLINE.coerlb.mu_bar,1)
            d(j) = norm(sample(i,:)-ONLINE.coerlb.mu_bar(j,:),2);
        end
        [temp, index] = sort(d);
        
        % num_mu_near is the number of 0 constraint points around a sample point
        num_mu_near = Jloc;
        % get the "active" mu_bar list
        d = zeros(size(sample,1),1);
        for j = 1:size(ONLINE.coerlb.mu_samples,1)
            d(j) = norm(sample(i,:)-ONLINE.coerlb.mu_samples(j,:),2);
        end
        [temp, mu_near_index] = sort(d);
        mu_near = ONLINE.coerlb.mu_samples(mu_near_index(2:num_mu_near+1),:);
        
        if num_mu_near > 0
             THETA=feval(Get_Theta_q, DEF, mu_near);
            THETA_Q_near =THETA(:,1:DEF.Q_affine.a);
        end
        
        f = THETA_Q(i,:)';
        A = zeros(num_mu_bar+num_mu_near+DEF.Q_affine.a*2,DEF.Q_affine.a);
        b = zeros(1,num_mu_bar+num_mu_near+DEF.Q_affine.a*2);
        for j = 1:num_mu_bar
            A(j,:) = -THETA_Q_bar(index(j),:);
            b(j) = -ONLINE.coerlb.alpha_bar(index(j));
        end
        % mu_near constraints
        for j = 1:num_mu_near
            A(num_mu_bar+j,:) = -THETA_Q_near(j,:);
            b(num_mu_bar+j) = 0;
        end
        for j = 1:DEF.Q_affine.a
            A(num_mu_bar+num_mu_near+(j-1)*2+1,j) = 1;
            A(num_mu_bar+num_mu_near+(j-1)*2+2,j) = -1;
            b(num_mu_bar+num_mu_near+(j-1)*2+1) = ONLINE.coerlb.sigmaUB(j);
            b(num_mu_bar+num_mu_near+(j-1)*2+2) = -ONLINE.coerlb.sigmaLB(j);
        end
        options = optimset('display','off');
        [zopt,fval] = linprog(f,A,b,[],[],[],[],[],options);
        alphaLB(i) = fval;
        % calculate sigmaUB
        alphaUB(i) = 1e5;
        for j = 1:size(ONLINE.coerlb.mu_bar,1)
            temp = zval(j,:)*THETA_Q(i,:)';
            if alphaUB(i) > temp
               alphaUB(i) = temp;
            end
        end
        rat = (alphaUB(i)-alphaLB(i))/alphaUB(i);
        store=2;
        if (rat<=1)
            if(rat>tol)
                if DEF.plotdemo==0
            disp(['Warning: tolerance for SCM is not satisfied: we provide in any case a LB. Sample is mu = ', num2str(sample(i,:))])
                end
            if (size(ONLINE.coerlb.failedpoints,1)==0)
             ONLINE.coerlb.failedpoints=[ONLINE.coerlb.failedpoints; sample(i,:)];    
            end
            
                for j=1:size(ONLINE.coerlb.failedpoints,1)
                id=strmatch(sample(i,:),ONLINE.coerlb.failedpoints(j,:));
                if (id==1)
                store=id;
                end
                end
                if store==2
            ONLINE.coerlb.failedpoints=[ONLINE.coerlb.failedpoints; sample(i,:)];  
                end
            end
    
        else
            if DEF.plotdemo==0
            disp(['Warning: tolerance for SCM is not satisfied: the LB is zero or negative, we provide UB. Sample is mu = ', num2str(sample(i,:))])
            end
            if (size(ONLINE.coerlb.failedpoints,1)==0)
             ONLINE.coerlb.failedpoints=[ONLINE.coerlb.failedpoints; sample(i,:)];    
            end
            for j=1:size(ONLINE.coerlb.failedpoints,1)
                id=strmatch(sample(i,:),ONLINE.coerlb.failedpoints(j,:));
                if (id==1)
                   store=id;
                end
            end
                if store==2
            ONLINE.coerlb.failedpoints=[ONLINE.coerlb.failedpoints; sample(i,:)];  
                end
           
           % ONLINE.coerlb.failedpoints=[ONLINE.coerlb.failedpoints; sample(i,:)];
            alphaLB(i)=alphaUB(i);
        end
        
    end
    return;