% -----rbMIT_System Software Copyright MIT 2006-07----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% REDUCED BASIS APPROXIMATION AND A POSTERIORI ERROR ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% FOR PARAMETRIZED PARTIAL DIFFERENTIAL EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Anthony T. PATERA - Gianluigi ROZZA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% (C) M.I.T. Massachusetts Institute of Technology 2006-07 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function  [sol_bound, du_ONLINE] = calc_res_du_dual_norm(sample_num,samples,ONLINE,DEF, Get_Theta_q, output_Get_Theta_q))
%
% Part II, Coercive and Compliant Affine Linear Elliptic Problems
%
% function   [sol_bound, du_ONLINE] = calc_res_du_dual_norm(sample_num,samples,ONLINE,DEF, Get_Theta_q, output_Get_Theta_q))
%
%
% Calculation of dual norm of residual for a posteriori error estimation and error bounds.
% ON-LINE calculation
% 
% INPUTS: 
% 
%
% - samples : vectors of   N_max x (Q_a+Q_f) quantities
%             used to compute  coefficients of bilinear forms and rhs
%             (Q_a: No. of \theta_a^q, Q_f: No. of \theta_f, N_max:  max number of samples ) 
%            
%
% - DEF : Problem definition
%         DEF.Q_affine: [1x1 struct] -> Q_affine.a= Number of \theta_a^q's
%                                           -> Q_affine.f= Number of \theta_f^q's
%
%
% - ONLINE.RB  REDUCED BASIS DATA STRUCTURE 
%
%   ONLINE.RB =     The following fields are used:
% 
%              
%                
%              RB.matrix: [1x1 struct]
%    
% 
%                    
%
%                    (to be used online for error bounds)
%                    ->Zqfprime_Fq: {Q_f x 1 x 1}  Zqf'*Fq{q}
%                    ->Zqfprime_Aq_Z: {Q_a x 1 x N} Zqf'*Aq{q}*Z
%                    ->Zqa'_Aq_Z: {Q_a x N x N}   Zqa'*(Aq{q}*Z)
%                    (to be used online for RB solutions)
%                    ->Zqa'_Aq_Z: {N x N}   Zqa'*(Aq{q}*Z)
%                    ->Aqn: {Q_a x N x N} Z'*Aq{q}*Z
%                    ->FN: [Nx1]  Z'*Fq
%
% OUTPUTS: sol_bound: error bound based on dual norm of residual

function [sol_bound, du_ONLINE] = calc_res_du_coer_dual_norm(sample_num, sample, du_ONLINE, pr_ONLINE, du_DEF, pr_DEF, Get_Theta_q, output_Get_Theta_q)

%
%
% 
  theta=feval(Get_Theta_q, pr_DEF, sample(sample_num,:));
  thetal=feval(output_Get_Theta_q, du_DEF, sample(sample_num,:));
  theta_a= theta(:,1:pr_DEF.Q_affine.a);
  theta_l= thetal(:,1:du_DEF.Q_affine.l);
  
  
  
  AN = du_ONLINE.RB.matrix.Aqn{1}*theta_a(1);
  for q=2:pr_DEF.Q_affine.a
    AN = AN + du_ONLINE.RB.matrix.Aqn{q}*theta_a(q);
  end
  
  LN = du_ONLINE.RB.matrix.LN{1}*theta_l(1);
  for q=2:du_DEF.Q_affine.l
    LN = LN + du_ONLINE.RB.matrix.LN{q}*theta_l(q);
  end
  
  uN = AN\LN;
 
  
  res_ll = 0;
    for q=1:du_DEF.Q_affine.l
      for qprime=1:du_DEF.Q_affine.l
        res_ll = res_ll + theta_l(q)*theta_l(qprime)*du_ONLINE.RB.matrix.Zqlprime_Lq{q,qprime};
      end
    end
    

  res_al = 0;
  for q=1:pr_DEF.Q_affine.a
    for qprime=1:du_DEF.Q_affine.l
      res_al = res_al + theta_a(q)*theta_l(qprime)*...
               du_ONLINE.RB.matrix.Zqlprime_Aq_Z{q,qprime}*uN;
    end
  end
  
  res_aa = 0;
  for q = 1:pr_DEF.Q_affine.a
    for qprime = 1:pr_DEF.Q_affine.a
      res_aa = res_aa + theta_a(q)*theta_a(qprime)*...
               (uN'*du_ONLINE.RB.matrix.Zqaprime_Aq_Z{q,qprime}*uN);
    end
  end  
  
  eps_pr_sqr = real(res_ll -2*res_al + res_aa);
  if eps_pr_sqr < 1E-15
    eps_pr_sqr = 0;
  end  
  
  
%\alpha lower bound for compliant coercive pure affine case
%[alpha_mu, alphaUB, rat, ONLINE]=alphaLB_coer(sample(sample_num,:),DEF, ONLINE, Get_Theta_q);
  delta_sN = eps_pr_sqr/du_ONLINE.coerlb.alphaLB(sample_num);
  sN = uN'*LN;
 
  delta_uN = real(sqrt(eps_pr_sqr)/du_ONLINE.coerlb.alphaLB(sample_num));
  rb_norm = sqrt(uN'*du_ONLINE.RB.matrix.X_norm_rb*uN);
  if du_ONLINE.space.absrelflg==0
  sol_bound = delta_uN;
  else
  sol_bound = delta_uN/(rb_norm); 
  end
  
  