% -----rbMIT_System Software Copyright MIT 2006-07----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% REDUCED BASIS APPROXIMATION AND A POSTERIORI ERROR ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% FOR PARAMETRIZED PARTIAL DIFFERENTIAL EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Anthony T. PATERA - Gianluigi ROZZA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% (C) M.I.T. Massachusetts Institute of Technology 2006-07 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function sol_bound = calc_res_pr_dual_norm(sample_num,samples,ONLINE,DEF)
%
% Part I, Parametrically Coercive and Compliant Affine Linear Elliptic Problems
%
% function sol_bound = calc_res_pr_dual_norm(sample_num,samples,ONLINE,DEF)
%
%
% Calculation of dual norm of residual for a posteriori error estimation and error bounds.
% ON-LINE calculation
% 
% INPUTS: 
% 
%
% - samples : vectors of   N_max x (Q_a+Q_f) quantities
%             used to compute  coefficients of bilinear forms and rhs
%             (Q_a: No. of \theta_a^q, Q_f: No. of \theta_f, N_max:  max number of samples ) 
%            
%
% - DEF : Problem definition
%         DEF.Q_affine: [1x1 struct] -> Q_affine.a= Number of \theta_a^q's
%                                           -> Q_affine.f= Number of \theta_f^q's
%
%
% - ONLINE.RB  REDUCED BASIS DATA STRUCTURE 
%
%   ONLINE.RB =     The following fields are used:
% 
%              
%                
%              RB.matrix: [1x1 struct]
%    
% 
%                    
%
%                    (to be used online for error bounds)
%                    ->Zqfprime_Fq: {Q_f x 1 x 1}  Zqf'*Fq{q}
%                    ->Zqfprime_Aq_Z: {Q_a x 1 x N} Zqf'*Aq{q}*Z
%                    ->Zqa'_Aq_Z: {Q_a x N x N}   Zqa'*(Aq{q}*Z)
%                    (to be used online for RB solutions)
%                    ->Zqa'_Aq_Z: {N x N}   Zqa'*(Aq{q}*Z)
%                    ->Aqn: {Q_a x N x N} Z'*Aq{q}*Z
%                    ->FN: [Nx1]  Z'*Fq
%
% OUTPUTS: sol_bound: error bound based on dual norm of residual

function sol_bound = calc_res_pr_dual_norm(sample_num,samples,ONLINE,DEF)

%
%
% 
  
  mu = samples(sample_num,1:DEF.Q_affine.a);
  theta_a = mu;
  theta_f = samples(sample_num,1+DEF.Q_affine.a:end);
  
  AN = ONLINE.RB.matrix.Aqn{1}*theta_a(1);
  for q=2:DEF.Q_affine.a
    AN = AN + ONLINE.RB.matrix.Aqn{q}*theta_a(q);
  end
  
  FN = ONLINE.RB.matrix.FN{1}*theta_f(1);
  for q=2:DEF.Q_affine.f
    FN = FN + ONLINE.RB.matrix.FN{q}*theta_f(q);
  end
  
  uN = AN\FN;
 
  
  res_ff = 0;
    for q=1:DEF.Q_affine.f
      for qprime=1:DEF.Q_affine.f
        res_ff = res_ff + theta_f(q)*theta_f(qprime)*ONLINE.RB.matrix.Zqfprime_Fq{q,qprime};
      end
    end
    

  res_af = 0;
  for q=1:DEF.Q_affine.a
    for qprime=1:DEF.Q_affine.f
      res_af = res_af + theta_a(q)*theta_f(qprime)*...
               ONLINE.RB.matrix.Zqfprime_Aq_Z{q,qprime}*uN;
    end
  end
  
  res_aa = 0;
  for q = 1:DEF.Q_affine.a
    for qprime = 1:DEF.Q_affine.a
      res_aa = res_aa + theta_a(q)*theta_a(qprime)*...
               (uN'*ONLINE.RB.matrix.Zqaprime_Aq_Z{q,qprime}*uN);
    end
  end  
  
  eps_pr_sqr = real(res_ff -2*res_af + res_aa);
  if eps_pr_sqr < 1E-15
    eps_pr_sqr = 0;
  end  
%\alpha lower bound for compliant coercive pure affine case
  alpha_mu = alphaLB_parcoer(theta_a,DEF.aux.thetamubar(1:DEF.Q_affine.a));

  delta_sN = eps_pr_sqr/alpha_mu;
  sN = uN'*FN;
  if ONLINE.space.absrelflg==0
  sol_bound = delta_sN;
  else
  sol_bound = delta_sN/(sN); 
  end
  
  