% -----rbMIT Software Copyright MIT 2006-07----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% REDUCED BASIS APPROXIMATION AND A POSTERIORI ERROR ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% FOR PARAMETRIZED PARTIAL DIFFERENTIAL EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Anthony T. PATERA - Gianluigi ROZZA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% (C) M.I.T. Massachusetts Institute of Technology 2006-07 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function  [du_ONLINE, pr_ONLINE, du_OFFLINE] =  coer_noncompliant_offline(mu_index,samples, du_OFFLINE, pr_OFFLINE, du_ONLINE, pr_ONLINE, du_DEF, pr_DEF, flagHaveBases)                              
% Part II, Coercive and Non-Compliant Affine Linear Elliptic Problems
% 
%
% OFF-LINE CALCULATION of RB MATRICES
% 
% INPUTS: 
%
% - mu_index: index indicating which column of samples should be taken
%
% - samples : vectors of  N_max x (Q_a+Q_f) quantities
%             used to compute  coefficients of bilinear forms and rhs
%             (Q_a: No. of \theta_a^q, Q_f: No. of \theta_f, N_max:  max number of samples ) 
%             
% - flagHaveBases: indicates if the basis is new (=0) or we are just adding
%                  basis functions.
%
% - FEM:  FINITE ELEMENT DATA STRUCTURE (TO PROVIDE)
%   FEM = 
% 
%     The following fields are used:
%     FEM.matrix: 
%
%     FEM.matrix.Aq: {Q_a x \cal N \cal N}
%     FEM.matrix.F:  {Q_f x \calN x1}
%     FEM.matrix.Xnorm: [\calN x \calN]
%
% - DEF Problem Definition
%     DEF.Q_affine:  
%     DEF.Q_affine.a: Q_a
%     DEF.Q_affine.f: Q_f
%     
%
% - ONLINE.RB  REDUCED BASIS DATA STRUCTURE 
%
%
%
% OUTPUTS: 
%
% - ONLINE.RB  REDUCED BASIS DATA STRUCTURE 
%
%  RB=      The following fields are used or created/filled:
%
%              RB.matrix: [1x1 struct]
%    
% 
%                    
%                    (to be used online for error bounds)
%                    ->Zqfprime_Fq: {Q_f x 1 x 1}  Zqf'*Fq{q}
%                    ->Zqfprime_Aq_Z: {Q_a x 1 x N} Zqf'*Aq{q}*Z
%                    ->Zqa'_Aq_Z: {Q_a x N x N}   Zqa'*(Aq{q}*Z)
%                    (to be used online for RB solutions)
%                    ->Zqa'_Aq_Z: {N x N}   Zqa'*(Aq{q}*Z)
%                    ->Aqn: {Q_a x N x N} Z'*Aq{q}*Z
%                    ->FN: [Nx1]  Z'*Fq
%                    


 function [du_ONLINE, pr_ONLINE, du_OFFLINE] = coer_noncompliant_offline(mu_index,samples, samples_l, du_OFFLINE, pr_OFFLINE, du_ONLINE, pr_ONLINE, du_DEF, pr_DEF, flagHaveBases)                              
  m=pr_OFFLINE.FEM.matrix.m;                           
  for q=1:pr_DEF.Q_affine.a                             
  Aq{q} = (pr_OFFLINE.FEM.matrix.Aq{q}(m,m))';
  
  end
  Xnorm = pr_OFFLINE.FEM.matrix.Xnorm;
  HT=pr_OFFLINE.FEM.matrix.HT;
 
  
  for q=1:du_DEF.Q_affine.l
  Lq{q}=-full(du_OFFLINE.FEM.matrix.Lq{q}(m)); %FEM.matrix.L{q}
  end

  for q=1:pr_DEF.Q_affine.f
  Fq{q}=full(pr_OFFLINE.FEM.matrix.Fq{q}(m)); %FEM.matrix.F{q}
  end
  
  theta_a= samples(mu_index,1:pr_DEF.Q_affine.a);
  theta_l = samples_l(mu_index,1:du_DEF.Q_affine.l); 
  
  A = Aq{1}*theta_a(1);
  for q=2:pr_DEF.Q_affine.a
    A = A + Aq{q}*theta_a(q);
  end
  L = Lq{1}*theta_l(1);
  for q=2:du_DEF.Q_affine.l
    L = L + Lq{q}*theta_l(q);
  end
  zn = A\L;  
  
  
  
  
  %%% orthogonalization Graham-Schimdt
  if flagHaveBases == 1
    for i_n =1:du_OFFLINE.space.N
      %normalise zn w.r.t previously orthogonal columns
      ri = du_OFFLINE.space.Z(:,i_n)'*(HT*Xnorm*zn);
      zn = zn - ri*du_OFFLINE.space.Z(:,i_n);
    end
    zn = zn/sqrt(zn'*(HT*Xnorm)*zn);
    du_OFFLINE.space.Z = [du_OFFLINE.space.Z zn];
    du_OFFLINE.space.N = du_OFFLINE.space.N + 1;
    
  else %first basis
    
    zn = zn/sqrt(zn'*(HT*Xnorm)*zn);
    du_OFFLINE.space.Z = zn;
    du_OFFLINE.space.N = 1;
    
    for q=1:du_DEF.Q_affine.l
      du_OFFLINE.space.matrix.Zql(:,q) = Xnorm \ (HT\Lq{q});
    end
  
  
    for q=1:du_DEF.Q_affine.l
      for qprime=1:du_DEF.Q_affine.l
        du_ONLINE.RB.matrix.Zqlprime_Lq{q,qprime} = du_OFFLINE.space.matrix.Zql(:,qprime)'*Lq{q};
      end
    end
   
    
  end
    
  
  
  
  Z = du_OFFLINE.space.Z;
  
  
  for q=1:pr_DEF.Q_affine.a
    for qprime=1:du_DEF.Q_affine.l
     du_ONLINE.RB.matrix.Zqlprime_Aq_Z{q,qprime} = ...
         (du_OFFLINE.space.matrix.Zql(:,qprime)'*Aq{q})*Z;
    end
  end
  
  for q=1:pr_DEF.Q_affine.a
    du_ONLINE.RB.matrix.Aqn{q} = Z'*(Aq{q}*Z);
    du_OFFLINE.space.matrix.Zqa{q} = Xnorm \ (HT\(Aq{q}*Z));
  end
      
  for q=1:pr_DEF.Q_affine.a
    for qprime=1:pr_DEF.Q_affine.a
      du_ONLINE.RB.matrix.Zqaprime_Aq_Z{q,qprime} = ...
          du_OFFLINE.space.matrix.Zqa{qprime}'*(Aq{q}*Z);
    end
  end
  
  for q=1:du_DEF.Q_affine.l
    du_ONLINE.RB.matrix.LN{q} = Z'*Lq{q};
  end  
  
  
  
  %additional ingredients for output evaluation
  
  for q=1:du_DEF.Q_affine.l
    pr_ONLINE.RB.matrix.LN{q} = pr_OFFLINE.space.Z'*Lq{q};
  end  
  
  for q=1:pr_DEF.Q_affine.f
    du_ONLINE.RB.matrix.FN{q} = Z'*Fq{q};
  end  
  
  for q=1:pr_DEF.Q_affine.a
    du_ONLINE.RB.matrix.Aqns{q} =  pr_OFFLINE.space.Z'*(Aq{q}*Z);
  end
  