% -----rbMIT Software Copyright MIT 2006-07----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 
function genreport(probname,OPTS)

if not(exist('OPTS'))
    OPTS.genpdf = 0;
    OPTS.openpdf = 0;
    OPTS.author = 'John Doe';
else
    if not(isfield(OPTS,'genpdf'))
        OPTS.genpdf = 0;
    end
    if not(isfield(OPTS,'openpdf'))
        OPTS.openpdf = 0;
    end
    if not(isfield(OPTS,'author'))
        OPTS.author = 'John Doe';
    end
end
if OPTS.genpdf == 0
    OPTS.openpdf = 0;
end
if isfield(OPTS,'name')
    probrealname = OPTS.name;
end
author = OPTS.author;

pathname = which('genreport'); % phuong 11/2/08
pathname
parentpathname = pathname(1:end-36); % phuong 11/2/08
webservertemplatepathname = [parentpathname,'/rbMIT_Library/genreport']; % phuong 11/2/08
probpath = [parentpathname,'/',probname];
probpath
reportpath0 = [parentpathname,'/',probname,'/report']; % phuong 11/2/08
reportpath = [parentpathname,'/',probname,'/report']; % phuong 11/2/08

% create report directory and load SYMB data
load([probpath,'/',probname,'_SYMB']);
warning off;
mkdir(reportpath0);
mkdir(reportpath);
warning on;

%% gen_title
if not(exist('probrealname'))
    instruct.gen_title = ['The \\textcolor{red}{``',probname,'''''} problem'];
else
    instruct.gen_title = ['The ',probrealname,' problem'];
end

%% gen_author
if not(exist('author'))
    instruct.gen_author = '\\textcolor{red}{Author name}';
else
    instruct.gen_author = author;
end

%% gen_region_tex
if length(original_input.geometry) > 1
    tmp = ['regions $\\calR_{\\ell}$, $1 \\leq \\ell \\leq ',num2str(length(original_input.geometry)),'$, which will serve to define the geometry or introduce inhomogeneous physical properties'];
else
    tmp = 'region $\\calR_{1}$';
end
instruct.gen_region_tex = tmp;

%% gen_curvy_boundaries
if not(isfield(original_input,'curvedatsym'))
    tmp = '';
else
    tmp = sprintf('\n\nIn this problem the ');
    for i = 1:length(original_input.curvedatsym)
        curvedatsym{i} = original_input.curvedatsym{i};
    end
    curvedat = original_input.curvedat;
    tedgeval = original_input.tedgeval;
    % find set of internal edges
    all_edge = original_input.geometry{1};
    for i = 2:length(original_input.geometry)
        all_edge = [all_edge, original_input.geometry{i}];
    end
    internal_edge_list_count = 0;
    internal_edge_list = [];
    for i = 1:size(original_input.edge,1)
        if length(find(i == all_edge))>1
            internal_edge_list_count = internal_edge_list_count + 1;
            internal_edge_list(internal_edge_list_count) = i;
        end
    end
    I = find(eval(tedgeval(:,1))>0);
    for i = 1:length(I)
        if ismember(I,internal_edge_list)
            I(i,2) = 1;
        else
            I(i,2) = 0;
        end
    end
    curvy_internal_edges = I(find(I(:,2) == 1));
    curvy_boundaries = I(find(I(:,2) == 0));
    tmp1 = '';
    for i = 1:length(curvy_internal_edges)
        tmp1 = [tmp1,', $\\Gamma_{',num2str(curvy_internal_edges(i)),'}$'];
    end
    if length(tmp1)>0
        tmp1 = tmp1(3:end);
    end
    tmp2 = '';
    for i = 1:length(curvy_boundaries)
        tmp2 = [tmp2,', $\\Gamma_{',num2str(curvy_boundaries(i)),'}$'];
    end
    if length(tmp2)>0
        tmp2 = tmp2(3:end);
    end
    if length(curvy_boundaries) > 0
        if length(curvy_boundaries) == 1
            tmp = [tmp,'boundary segment ',tmp2];
        else
            tmp = [tmp,'boundary segments ',tmp2];
        end
    end
    if length(curvy_internal_edges) > 0
        if length(curvy_boundaries) > 0
            tmp = [tmp,' and '];
        end
        if length(curvy_internal_edges) == 1
            tmp = [tmp,'internal edge ',tmp1];
        else
            tmp = [tmp,'internal edges ',tmp1];
        end
    end
    if size(I,1) > 1
        tmp = [tmp,' are curved (all other boundary segments and internal interfaces are straight lines). '];
    else
        tmp = [tmp,' is curved (all other boundary segments and internal interfaces are straight lines). '];
    end
    I = I(:,1);
    for i = 1:size(curvedat,1)
        II = find(eval(tedgeval(I,1)) == i);
        II = I(II);
        if length(II)>0
            tmp1 = '';
            for j = 1:length(II)
                tmp1 = [tmp1,', $\\Gamma_{',num2str(II(j)),'}$'];
            end
            if length(II) == 1
                tmp = [tmp,'The segment ',tmp1(3:end),' is '];
            else
                tmp = [tmp,'The segments ',tmp1(3:end),' are '];
            end
            tmp2O1 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.O(1))),'\\\\'),original_input.P,'\\\\');
            tmp2O2 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.O(2))),'\\\\'),original_input.P,'\\\\');
            tmp2Q11 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.Q(1,1))),'\\\\'),original_input.P,'\\\\');
            tmp2Q12 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.Q(1,2))),'\\\\'),original_input.P,'\\\\');
            tmp2Q21 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.Q(2,1))),'\\\\'),original_input.P,'\\\\');
            tmp2Q22 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.Q(2,2))),'\\\\'),original_input.P,'\\\\');
            tmp2S11 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.S(1,1))),'\\\\'),original_input.P,'\\\\');
            tmp2S12 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.S(1,2))),'\\\\'),original_input.P,'\\\\');
            tmp2S21 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.S(2,1))),'\\\\'),original_input.P,'\\\\');
            tmp2S22 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.S(2,2))),'\\\\'),original_input.P,'\\\\');            
            tmp2G1 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.G(1))),'\\\\'),original_input.P,'\\\\');
            tmp2G2 = replacemulatex(cleanlatex(latex(simplify(original_input.curvedatsym{i}.G(2))),'\\\\'),original_input.P,'\\\\');
            tmp = [tmp,'given by the parametrization',sprintf('\n\\\\begin{equation}'),...
                   '\\left[\\begin{array}{c} x_1 \\\\ x_2 \\\\ \\end{array}\\right] = ',...
                   '\\underbrace{\\left[\\begin{array}{c} ',tmp2O1,' \\\\ ',tmp2O2,' \\\\ \\end{array}\\right]}_{O^{',num2str(i),'}(\\bfmu)} + ',...
                   '\\underbrace{\\left[\\begin{array}{cc} ',tmp2Q11,' & ',tmp2Q12,' \\\\ ',tmp2Q21,' & ',tmp2Q22,' \\\\ \\end{array}\\right]}_{Q^{',num2str(i),'}(\\bfmu)}',...
                   '\\underbrace{\\left[\\begin{array}{cc} ',tmp2S11,' & ',tmp2S12,' \\\\ ',tmp2S21,' & ',tmp2S22,' \\\\ \\end{array}\\right]}_{S^{',num2str(i),'}(\\bfmu)}',...
                   '\\left[\\begin{array}{c} ',tmp2G1,' \\\\ ',tmp2G2,' \\\\ \\end{array}\\right]',...                   
                   sprintf(',\\\\nonumber\n\\\\end{equation}\n'),'where '];            
        end
        if length(II) > 1
            for j = 1:length(II)
                tmp2x = cleanlatex(latex(tedgeval(II(j),2)),'\\\\');
                tmp2y = cleanlatex(latex(tedgeval(II(j),3)),'\\\\');
                tmp = [tmp,'for $\\Gamma_{',num2str(II(j)),'}$, $t \\in [',tmp2x,',',tmp2y,']$, '];
            end
        elseif length(II) == 1
            tmp2x = cleanlatex(latex(tedgeval(II(1),2)),'\\\\');
            tmp2y = cleanlatex(latex(tedgeval(II(1),3)),'\\\\');
            tmp = [tmp,'$t \\in [',tmp2x,',',tmp2y,']$. '];
        end
    end
    tmp = [tmp(1:end-2),'.'];
end
instruct.gen_curvy_boundaries = tmp;

%% gen_P
instruct.gen_P = ['$P = ',num2str(original_input.P),' $'];

%% gen_parameter_list
tmp = findsym(psym);
tmpflag = zeros(original_input.P,1);
for i = 1:original_input.P
    if ismember(['mu',num2str(i)],tmp)
        tmpflag(i) = 1;
        tmp = regexprep(tmp,['mu',num2str(i)],['$\\\\mu_{',num2str(i),'}$']);
    end
end
tmpflag0 = find(tmpflag == 0);
tmpflag1 = find(tmpflag == 1);
if length(tmpflag1)>0
    tmp = ['Here ',tmp,' are geometry parameters defined in Figure~1: '];
    for i = 1:size(tmpflag1,1)
        tmp = [tmp, '$\\mu_{',num2str(tmpflag1(i)),'}$ is \\textcolor{red}{input mu',num2str(tmpflag1(i)),' explanation}, '];
    end
    tmp = tmp(1:end-2);
end
if length(tmpflag0)>0
    if length(tmpflag1)>0
        tmp = [tmp,'. The remaining (always non-dimensional) parameters are given by: ',];
    else
        tmp = 'The (always non-dimensional) parameters are given by: ';
    end
    for i = 1:size(tmpflag0,1)
        tmp = [tmp, '$\\mu_{',num2str(tmpflag0(i)),'}$ is \\textcolor{red}{input mu',num2str(tmpflag0(i)),' explanation}, '];
    end
    tmp = [tmp,'where \\textcolor{red}{input list of ``where''''}',];
end
instruct.gen_parameter_list = tmp;

%% gen_parameter_domain
tmp = '$\\calD = ';
for i = 1:original_input.P
    tmp = [tmp,'[',num2str(original_input.mu_min(i)),', ',num2str(original_input.mu_max(i)),']\\times '];
end
tmp = tmp(1:end-7);
tmp(end) = '$';
instruct.gen_parameter_domain = tmp;

%% gen_input_fieldname
switch original_input.probtype
    case {'TH','TH2'}
        instruct.gen_input_fieldname = '\\textcolor{red}{input fieldname}';
    case {'LE1','LE2','LE3','LE4','LE5'}
        instruct.gen_input_fieldname = 'displacement';
end

%% gen_input_gradfieldname
switch original_input.probtype
    case {'TH','TH2'}
        instruct.gen_input_gradfieldname = '\\textcolor{red}{input grad fieldname}';
    case {'LE1','LE2','LE3','LE4','LE5'}
        instruct.gen_input_gradfieldname = 'stress';
end

%% gen_input_name
switch original_input.probtype
    case {'TH','TH2'}
        instruct.gen_input_name = '$\\ivar(\\bfmu)$';
    case {'LE1','LE2','LE3','LE4','LE5'}
        instruct.gen_input_name = '$\\ivar(\\bfmu) = (\\ivar_1(\\bfmu),\\ivar_2(\\bfmu))$';
end

%% gen_axisymm_fieldname
switch original_input.probtype
    case {'TH','TH2','LE1','LE2','LE3','LE4'}
        instruct.gen_axisymm_fieldname = '';
    case {'LE5'}
        instruct.gen_axisymm_fieldname = 'Here $\\ivar_1(\\bfmu)$ and $\\ivar_2(\\bfmu)$ correspond to the radial displacement and axial displacement, respectively.';
end


%% gen_PDE_name
switch original_input.probtype
    case 'TH'
        instruct.gen_PDE_name = 'a generalized Laplacian';        
    case 'TH2'
        instruct.gen_PDE_name = 'an axisymmetric generalized Laplacian';    
    case {'LE1','LE2','LE3','LE4'}
        instruct.gen_PDE_name = 'the linear elasticity PDE';
    case 'LE5'
        instruct.gen_PDE_name = 'the axisymmetric linear elasticity PDE';    
end
% cuong
if isfield(original_input,'dt') & isfield(original_input,'nt')
    instruct.gen_PDE_name = 'a time-dependent linear PDE';
end
% cuong

%% gen_govern_eqns
switch original_input.probtype
    case {'TH','TH2'}
        instruct.gen_govern_eqns = sprintf('\\\\begin{eqnarray}\n');
        for igeo = 1:length(original_input.geometry)
            % cuong
            if isfield(original_input,'dt') & isfield(original_input,'nt') 
                if (isfield(original_input,'rhoc')==0) || (original_input.rhoc(igeo) == 1) 
                    if strcmp(original_input.probtype, 'TH')
                    tmp1=['\\frac{\\partial \\ivar(\\bfmu)}{\\partial t}'];
                    else
                    tmp1=['x_1 \\frac{\\partial \\ivar(\\bfmu)}{\\partial t}'];
                    end
                else
                    if strcmp(original_input.probtype, 'TH')
                    tmp1=[replacemulatex(latex(sym(original_input.rhoc(igeo))),original_input.P,'\\\\'), '\\frac{\\partial \\ivar(\\bfmu)}{\\partial t}'];
                    else
                    tmp1=[replacemulatex(latex(sym(original_input.rhoc(igeo))),original_input.P,'\\\\'), 'x_1 \\frac{\\partial \\ivar(\\bfmu)}{\\partial t}'];
                    end
                end
            else
                tmp1='';
            end
            tmp = [tmp1 '-\\frac{\\partial}{\\partial x_i}\\bigg(\\underbrace{\\left[\\begin{array}{cc}',...
                replacemulatex(latex(sym(original_input.kappa{igeo}(1,1))),original_input.P,'\\\\'),' & ',...
                replacemulatex(latex(sym(original_input.kappa{igeo}(1,2))),original_input.P,'\\\\'),' \\\\',...
                replacemulatex(latex(sym(original_input.kappa{igeo}(2,1))),original_input.P,'\\\\'),' & ',...
                replacemulatex(latex(sym(original_input.kappa{igeo}(2,2))),original_input.P,'\\\\'),...
                ' \\end{array}\\right]}_{\\kappa^{',num2str(igeo),'}_{ij}}\\frac{\\partial}{\\partial x_j}\\ivar(\\bfmu)\\bigg)'];
            for i = 1:2
                if original_input.kappa{igeo}(i,3) ~= 0
                    if original_input.kappa{igeo}(i,3) == 1
                        tmp1 = '';
                    else
                        tmp1 = replacemulatex(latex(sym(original_input.kappa{igeo}(i,3))),original_input.P,'\\\\');
                    end
                    tmp = [tmp,'+',tmp1,'\\frac{\\partial}{\\partial x_1}\\ivar(\\bfmu)'];
                end
                if original_input.kappa{igeo}(3,i) ~= 0
                    if original_input.kappa{igeo}(3,i) == 1
                        tmp1 = '';
                    else
                        tmp1 = replacemulatex(latex(sym(original_input.kappa{igeo}(3,i))),original_input.P,'\\\\');
                    end
                    tmp = [tmp,'+',tmp1,'\\frac{\\partial}{\\partial x_2}\\ivar(\\bfmu)'];
                end
            end
            if original_input.kappa{igeo}(3,3) ~= 0
                if original_input.kappa{igeo}(3,3) == 1
                    tmp1 = '';
                else
                    tmp1 = replacemulatex(latex(sym(original_input.kappa{igeo}(3,3))),original_input.P,'\\\\');
                end
                tmp = [tmp,'+',tmp1,'\\ivar'];
            end
            % cuong
            if isfield(original_input,'dt') & isfield(original_input,'nt') 
                if (original_input.fareaload(igeo) == 0)
                    tmp = [tmp,'= 0 & {\\rm in} & \\calR_{',num2str(igeo),'}, ', ' t \\in [0, T]'];
                else
                    if strcmp(original_input.probtype, 'TH')
                    tmp = [tmp,'= ',replacemulatex(latex(original_input.fareaload(igeo)),original_input.P,'\\\\'),'g(t) & {\\rm in} & \\calR_{',num2str(igeo),'}, ', ' t \\in [0, T]'];
                    else
                    tmp = [tmp,'= ',replacemulatex(latex(original_input.fareaload(igeo)),original_input.P,'\\\\'),'x_1 g(t) & {\\rm in} & \\calR_{',num2str(igeo),'}, ', ' t \\in [0, T]'];    
                    end
                end
            else
                tmp = [tmp,'=',replacemulatex(latex(original_input.fareaload(igeo)),original_input.P,'\\\\'),' & {\\rm in} & \\calR_{',num2str(igeo),'}'];
            end
            % cuong
            if igeo < length(original_input.geometry)
                if (igeo>1) && (igeo-floor(igeo/3)*3 == 0)
                    instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,',\\nonumber\n',sprintf('\\\\end{eqnarray}\n\\\\begin{eqnarray}\n')];
                else
                    instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,',\\nonumber\\\\ '];
                end
            else
                instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,',\\nonumber'];
            end
        end
        % cuong
        if isfield(original_input,'dt') & isfield(original_input,'nt') 
            % initial condition            
            instruct.gen_govern_eqns = [instruct.gen_govern_eqns, sprintf('\\n\\\\end{eqnarray}'),sprintf('\n'),'with initial condition'];
           initsol = sym2fracsym(sym(regexprep(original_input.initsol,{'x1','x2','.\^'},{'x','y','^'})));
           theta_u = sym2fracsym(sym(original_input.theta_u));
            %theta_u =0;
           %initsol(1)=0;
            if initsol(1)==0
                tmpstr=['u(t=0) = 0'];
            else
                if length(initsol)==1
                    tmpstr=['u(t=0) = '];
                    tmp1=replacemulatex(latex(sym(theta_u)),original_input.P,'\\\\');
                    tmp2=replacemulatex(latex(sym(initsol)),original_input.P,'\\\\');
                    tmpstr=[tmpstr, tmp1, '(',tmp2,')'];
                else                
                    tmp1=replacemulatex(latex(sym(theta_u(1))),original_input.P,'\\\\');
                    tmp2=replacemulatex(latex(sym(initsol(1))),original_input.P,'\\\\');
                    tmpstr=['u(t=0) = ', tmp1, tmp2];
                    for ii=2:length(initsol)                
                        tmp1=replacemulatex(latex(sym(theta_u(ii))),original_input.P,'\\\\');
                        tmp2=replacemulatex(latex(sym(initsol(ii))),original_input.P,'\\\\');                
                        tmpstr=[tmpstr, ' + ', tmp1, '(',tmp2,')'];
                    end
                end
            end
            tmp=[sprintf('\\\\begin{equation}\n'),tmpstr,sprintf(',\\\\nonumber\n\\\\end{equation}')];
            instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp];           
            if (original_input.fareaload(igeo) == 0)
                instruct.gen_govern_eqns = [instruct.gen_govern_eqns, 'with summation ($i,j = 1,2$) over repeated indices.'];
            else
                instruct.gen_govern_eqns = [instruct.gen_govern_eqns, 'with summation ($i,j = 1,2$) over repeated indices. Here the control input $g(t)$ is a function of time $t$. The problem considers any square-integrable function for $g(t)$.'];
            end            
            tmp1=[' In practice, the PDE is replaced by a discrete--time (Euler backward) approximation with time-steps of size $\\Delta t$ = ' num2str(original_input.dt) '.'];
            tmp2=[' Note that the final time is $T$ = ' num2str(original_input.dt*original_input.nt) ' and that the number of time-steps is $n_t$ =' num2str(original_input.nt) '.'];
            instruct.gen_govern_eqns = [instruct.gen_govern_eqns, tmp1, tmp2];
        else
            instruct.gen_govern_eqns = [instruct.gen_govern_eqns, sprintf('\\n\\\\end{eqnarray}'),sprintf('\n'),'with summation ($i,j = 1,2$) over repeated indices.'];
        end
        % cuong
        instruct.gen_govern_eqns = regexprep(instruct.gen_govern_eqns,'+-','-');        
    case {'LE1','LE2','LE3','LE4'}
        tmpname = {'isotropic plane strain ','isotropic plane stress ','orthotropic plane strain ','orthotropic plane stress '};
        instruct.gen_govern_eqns = sprintf('\\\\begin{eqnarray}\n');
        for igeo = 1:length(original_input.geometry)
            tmp = ['-\\frac{\\partial}{\\partial x_j}\\bigg(C^{',num2str(igeo),'}_{ijkl}(\\bfmu)\\frac{\\partial}{\\partial x_l}ivar_k(\\bfmu)\\bigg)'];
            tmp1 = '0';
            % begin - should not work now
            if original_input.matprop{igeo}.aux(1) ~= 0
                if original_input.matprop{igeo}.aux(1) == 1
                    tmp10 = '';
                else
                    tmp10 = replacemulatex(latex(original_input.matprop{igeo}.aux(1)),original_input.P,'\\\\');
                end
                tmp1 = [tmp10,'\\ivar_1'];
            end
            tmp2 = '0';
            if original_input.matprop{igeo}.aux(2) ~= 0
                if original_input.matprop{igeo}.aux(2) == 1
                    tmp20 = '';
                else
                    tmp20 = replacemulatex(latex(original_input.matprop{igeo}.aux(2)),original_input.P,'\\\\');
                end
                tmp2 = [tmp20,'\\ivar_2'];
            end
            if (not(strcmp(tmp1,'0'))) || (not(strcmp(tmp2,'0')))
                tmp = [tmp,'+','\\underbrace{\\left[\\begin{array}{c} ',tmp1,'\\\\ ',tmp2,'\\\\ \\end{array}\\right]}_{\\lambda_i^{',num2str(igeo),'}}\\delta_{ij}\\ivar(\\bfmu)'];
            end
            % end - should not work now
            tmpflag = zeros(length(geometry),1);
            if (original_input.fareaload(igeo,1) == 0) && (original_input.fareaload(igeo,2) == 0)
                tmp = [tmp,'= 0 & {\\rm in} & \\calR_{',num2str(igeo),'}, \\quad i = 1,2'];
            else
                tmp = [tmp,'= b_i^{',num2str(igeo),'}}',' & {\\rm in} & \\calR_{',num2str(igeo),'}, \\quad i = 1,2'];
                tmpflag(igeo) = 1;
            end
            if igeo < length(original_input.geometry)
                if (igeo>1) && (igeo-floor(igeo/3)*3 == 0)
                    instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,',\\nonumber\n',sprintf('\\\\end{eqnarray}\n\\\\begin{eqnarray}\n')];
                else
                    instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,',\\nonumber\\\\ '];
                end
            else
                instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,'\\nonumber,'];
            end
        end
        instruct.gen_govern_eqns = [instruct.gen_govern_eqns, sprintf('\\n\\\\end{eqnarray}\n'),'where '];
        for igeo = 1:length(original_input.geometry)
            if tmpflag(igeo)
                instruct.gen_govern_eqns = [instruct.gen_govern_eqns, '$b^{',igeo,'} = [',replacemulatex(latex(original_input.fareaload(igeo,1)),original_input.P,'\\\\'),', ',replacemulatex(latex(original_input.fareaload(igeo,2)),original_input.P,'\\\\'),']$ and '];
            end
        end        
        if length(original_input.geometry) > 1
            tmp = ['$C^{\\ell}_{ijkl}(\\bfmu)$, $1 \\leq \\ell \\leq ',num2str(length(original_input.geometry)),'$ are the effective ',tmpname{str2num(original_input.probtype(3))},'elasticity tensors defined by',sprintf('\n\\\\begin{eqnarray}\n')];
        else
            tmp = ['$C^{1}_{ijkl}(\\bfmu)$ is the effective ',tmpname{str2num(original_input.probtype(3))},'elasticity tensor defined by',sprintf('\n\\\\begin{eqnarray}\n')];
        end
        switch original_input.probtype
            case 'LE1'
                tmpstr = ['C^{\\ell}_{ijkl}(\\bfmu) = \\left[\\begin{array}{cccc}',sprintf('\n'),'\\dfrac{E^{\\ell}(\\nu^{\\ell}-1)}{(1+\\nu^{\\ell})(-1+2\\nu^{\\ell})} & 0 & 0 & -\\dfrac{E^{\\ell}\\nu^{\\ell}}{(1+\\nu^{\\ell})(-1+2\\nu^{\\ell})} \\\\',...
                          sprintf('\n'),'0 & \\dfrac{E^{\\ell}}{2(1+\\nu^{\\ell})} & \\dfrac{E^{\\ell}}{2(1+\\nu^{\\ell})} & 0 \\\\',...
                          sprintf('\n'),'0 & \\dfrac{E^{\\ell}}{2(1+\\nu^{\\ell})} & \\dfrac{E^{\\ell}}{2(1+\\nu^{\\ell})} & 0 \\\\',...
                          sprintf('\n'),'-\\dfrac{E^{\\ell}\\nu^{\\ell}}{(1+\\nu^{\\ell})(-1+2\\nu^{\\ell})} & 0 & 0 & \\dfrac{E^{\\ell}(\\nu^{\\ell}-1))}{(1+\\nu^{\\ell})(-1+2\\nu^{\\ell})} \\\\',sprintf('\n'),'\\end{array} \\right].\\nonumber'];
            case 'LE2'
                tmpstr = ['C^{\\ell}_{ijkl}(\\bfmu) = \\left[\\begin{array}{cccc}',sprintf('\n'),'\\dfrac{E^{\\ell}}{1-(\\nu^{\\ell})^2} & 0 & 0 & \\dfrac{E^{\\ell}\\nu^{\\ell}}{1-(\\nu^{\\ell})^2} \\\\',...
                          sprintf('\n'),'0 & \\dfrac{E^{\\ell}}{2(1+\\nu^{\\ell})} & \\dfrac{E^{\\ell}}{2(1+\\nu^{\\ell})} & 0 \\\\',...
                          sprintf('\n'),'0 & \\dfrac{E^{\\ell}}{2(1+\\nu^{\\ell})} & \\dfrac{E^{\\ell}}{2(1+\\nu^{\\ell})} & 0 \\\\',...
                          sprintf('\n'),'\\dfrac{E^{\\ell}\\nu^{\\ell}}{1-(\\nu^{\\ell})^2} & 0 & 0 & \\dfrac{E^{\\ell}}{1-(\\nu^{\\ell})^2} \\\\',sprintf('\n'),'\\end{array}\\right].\\nonumber'];
            case 'LE3'
                tmpstr = ['C^{\\ell}_{ijkl}(\\bfmu) = \\left[\\begin{array}{cccc}',sprintf('\n'),...
                          sprintf('\n'),'\\dfrac{(E_{11}^{\\ell})^2(-E_{22}^{\\ell}+E_{33}^{\\ell}(\\nu_{23}^{\\ell})^2)}{A} & 0 & 0 & -\\dfrac{E_{11}^{\\ell}E_{22}^{\\ell}(E_{22}^{\\ell}\\nu_{12}^{\\ell}+E_{33}^{\\ell}\\nu_{13}^{\\ell}\\nu_{23}^{\\ell})}{A} \\\\',...
                          sprintf('\n'),'0 & G_{12}^{\ell} & G_{12}^{\ell} & 0 \\\\',...
                          sprintf('\n'),'0 & G_{12}^{\ell} & G_{12}^{\ell} & 0 \\\\',...
                          sprintf('\n'),'-\\dfrac{E_{11}^{\\ell}E_{22}^{\\ell}(E_{22}^{\\ell}\\nu_{12}^{\\ell}+E_{33}^{\\ell}\\nu_{13}^{\\ell}\\nu_{23}^{\\ell})}{A} & 0 & 0 & \\dfrac{(E_{22}^{\\ell})^2(-E_{11}^{\\ell}+E_{33}^{\\ell}(\\nu_{13}^{\\ell})^2)}{A} \\\\',...
                          sprintf('\n'),'\\end{array}\\right] \\nonumber \\\\',...
                          sprintf('\n'),'A = -E^{\\ell}_{11} E^{\\ell}_{22} + E^{\\ell}_{11}\\nu^{\\ell}_{23}E^{\\ell}_{33} + \\nu^{\\ell}_{12}E^{\\ell}_{22} + 2\\nu^{\\ell}_{12}E^{\\ell}_{22}\\nu^{\\ell}_{23}\\nu^{\\ell}_{13}E^{\\ell}_{33} + (\\nu^{\\ell}_{13})^2E^{\\ell}_{22}E^{\\ell}_{33} \\\\',sprintf('\n'),'\\end{array}\\right].\\nonumber'];
            case 'LE4'
                tmpstr = ['C^{\\ell}_{ijkl(\\bfmu)} = \\left[\\begin{array}{cccc}',sprintf('\n'),'\\dfrac{(E_{11}^{\\ell})^2}{-E^{\\ell}_{11}+(\\nu_{12}^{\\ell})^2} & 0 & 0 & -\\dfrac{E_{11}^{\\ell}E_{22}^{\\ell}\\nu_{12}^{\\ell}}{-E_{11}^{\\ell}+(\\nu_{12}^{\\ell})^2E_{22}^{\\ell}} \\\\',...
                          sprintf('\n'),'0 & G_{12}^{\\ell} & G_{12}^{\\ell} & 0 \\\\',...
                          sprintf('\n'),'0 & G_{12}^{\\ell} & G_{12}^{\\ell} & 0 \\\\',...
                          sprintf('\n'),'-\\dfrac{E_{11}^{\\ell}E_{22}^{\\ell}\\nu_{12}^{\\ell}}{-E_{11}^{\\ell}+(\\nu_{12}^{\\ell})^2E_{22}^{\\ell}} & 0 & 0 & \\dfrac{E_{11}^{\\ell}E_{22}^{\\ell}}{-E^{\\ell}_{11}+(\\nu_{12}^{\\ell})^2} \\\\',sprintf('\n'),'\\end{array}\\right].\\nonumber'];
        end
        tmp = [tmp,tmpstr,sprintf('\n'),'\\end{eqnarray}',sprintf('\n')];
        tmp = [tmp,'Here '];
        switch original_input.probtype
            case {'LE1','LE2'}
                for i = 1:length(original_input.geometry)
                    tmp = [tmp, '$E^{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(1)),original_input.P,'\\\\'),...
                           '$, $\\nu^{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(2)),original_input.P,'\\\\'),...
                           '$, '];
                end
            case {'LE3'}
                for i = 1:length(original_input.geometry)
                    tmp = [tmp, '$E^{11}{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(1)),original_input.P,'\\\\'),...
                           '$, $E^{22}{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(2)),original_input.P,'\\\\'),...
                           '$, $E^{33}{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(3)),original_input.P,'\\\\'),...
                           '$, $\\nu^{12}{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(4)),original_input.P,'\\\\'),...
                           '$, $\\nu^{23}{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(5)),original_input.P,'\\\\'),...
                           '$, $\\nu^{13}{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(6)),original_input.P,'\\\\'),...
                           '$, $G_{12}^{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(7)),original_input.P,'\\\\'),...
                           '$, '];
                end
            case {'LE4'}
                for i = 1:length(original_input.geometry)
                    tmp = [tmp, '$E^{11}{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(1)),original_input.P,'\\\\'),...
                           '$, $E^{22}{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(2)),original_input.P,'\\\\'),...
                           '$, $\\nu^{12}{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(3)),original_input.P,'\\\\'),...
                           '$, $G_{12}^{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(4)),original_input.P,'\\\\'),...
                           '$, '];
                end
        end
        if length(original_input.geometry) > 1
            tmp = [tmp(1:end-2), ' are the non-dimensional Young''s modulus and Poisson ratio for regions $\\calR_1$, \\ldots, \\cal_{',num2str(length(original_input.geometry)),'}$, respectively.'];
        else
            tmp = [tmp(1:end-2), ' are the non-dimensional Young''s modulus and Poisson ratio for the region $\\calR_1$.'];
        end            
        instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp];
        instruct.gen_govern_eqns = regexprep(instruct.gen_govern_eqns,'+-','-');
    case 'LE5'
        tmpname = {'axisymmetric '};
        instruct.gen_govern_eqns = sprintf('\\\\begin{eqnarray}\n');
        for igeo = 1:length(original_input.geometry)
            tmp = ['\\lambda_1^{',num2str(igeo),'}\\frac{\\partial^2}{\\partial x_1^2}\\ivar_1(\\bfmu)+',...
                   '\\lambda_2^{',num2str(igeo),'}\\frac{\\partial^2}{\\partial x_1 \\partial x_2}\\ivar_2(\\bfmu)+',...
                   '\\lambda_3^{',num2str(igeo),'}\\bigg(\\frac{\\partial^2}{\\partial x_2^2}\\ivar_1(\\bfmu)+',...
                   '\\frac{\\partial^2}{\\partial x_1 \\partial x_2}\\ivar_2(\\bfmu)\\bigg)+',...
                   '\\lambda_2^{',num2str(igeo),'}\\frac{\\partial}{\\partial x_1}(\\frac{\\ivar_1(\\bfmu)}{x_1})'];
            if (original_input.fareaload(igeo,1) == 0)
                tmp = [tmp,'&=& 0 \\nonumber \\\\'];
            else
                tmp = [tmp,'&=& ',replacemulatex(latex(original_input.fareaload(igeo,1)),original_input.P,'\\\\'),' \\nonumber \\\\'];
            end
            tmp = [tmp,...
                   '\\lambda_3^{',num2str(igeo),'}\\bigg(\\frac{\\partial^2}{\\partial x_1 \\partial x_2}\\ivar_1(\\bfmu)+',...
                   '\\frac{\\partial^2}{\\partial x_1^2}\\ivar_2(\\bfmu)\\bigg)+',...
                   '\\lambda_2^{',num2str(igeo),'}\\frac{\\partial^2}{\\partial x_1 \\partial x_2}\\ivar_1(\\bfmu)+',...
                   '\\lambda_1^{',num2str(igeo),'}\\frac{\\partial^2}{\\partial x_2^2}\\ivar_2(\\bfmu)+',...
                   '\\frac{\\lambda_2^{',num2str(igeo),'}+\\lambda_3^{',num2str(igeo),'}}{x_1}\\frac{\\partial}{\\partial x_2}\\ivar_1(\\bfmu)+',...
                   '\\frac{\\lambda_3^{',num2str(igeo),'}}{x_1}\\frac{\\partial}{\\partial x_1}\\ivar_2(\\bfmu)'];
            if (original_input.fareaload(igeo,2) == 0)
                tmp = [tmp,'&=& 0 \\nonumber'];
            else
                tmp = [tmp,'&=& ',replacemulatex(latex(original_input.fareaload(igeo,2)),original_input.P,'\\\\'),' \\nonumber'];
            end
            if igeo < length(original_input.geometry)
                if (igeo>1) && (igeo-floor(igeo/3)*3 == 0)
                    instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,',\\nonumber\n',sprintf('\\\\end{eqnarray}\n\\\\begin{eqnarray}\n')];
                else
                    instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,',\\nonumber\\\\ '];
                end
            else
                instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,'\\nonumber,'];
            end
        end
        instruct.gen_govern_eqns = [instruct.gen_govern_eqns, sprintf('\\n\\\\end{eqnarray}\n'),'where',sprintf('\\\\begin{eqnarray}\n')];
        
        if length(original_input.geometry) > 1
            tmpE = 'E^{\\ell}'; tmpnu = '\\nu^{\\ell}';
        else
            tmpE = 'E^{1}'; tmpnu = '\\nu^{1}';
        end
        tmp = ['\\lambda_1^{',num2str(igeo),'} &=& \\frac{',tmpE,'(1-',tmpnu,')}{(1+',tmpnu,')(1-2',tmpnu,')}, \\nonumber \\\\ ',...
               '\\lambda_2^{',num2str(igeo),'} &=& \\frac{',tmpE,'}{(1+',tmpnu,')(1-2',tmpnu,')}, \\nonumber \\\\ ',...
               '\\lambda_3^{',num2str(igeo),'} &=& \\frac{',tmpE,'}{2(1+',tmpnu,')}. \\nonumber'];
        instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp,sprintf('\\\\end{eqnarray}\n'),'Here '];
        tmp = [];
        for i = 1:length(original_input.geometry)
            tmp = [tmp, '$E^{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(1)),original_input.P,'\\\\'),...
                        '$, $\\nu^{',num2str(i),'} = ',replacemulatex(latex(original_input.matprop{igeo}.main(2)),original_input.P,'\\\\'),...
                        '$, '];
        end
        if length(original_input.geometry) > 1
            tmp = [tmp(1:end-2), ' are the non-dimensional Young''s modulus and Poisson ratio for regions $\\calR_1$, \\ldots, \\cal_{',num2str(length(original_input.geometry)),'}$, respectively.'];
        else
            tmp = [tmp(1:end-2), ' are the non-dimensional Young''s modulus and Poisson ratio for the region $\\calR_1$.'];
        end
        instruct.gen_govern_eqns = [instruct.gen_govern_eqns,tmp];
end

%% gen_Neumann_tex
switch original_input.probtype
    case {'TH','TH2'}
        instruct.gen_Neumann_tex = '$\\textbf{n}_i\\kappa_{ij}\\dfrac{\\partial}{\\partial x_j}\\ivar(\\bfmu)$,';
    case {'LE1','LE2','LE3','LE4'}
        instruct.gen_Neumann_tex = '$C_{ijkl}(\\bfmu)\\dfrac{\\partial}{\\partial x_l}\\ivar_k(\\bfmu)\\textbf{n}_j$,';
    case {'LE5'}
        tmp = ['\\sigma_{1} &=& \\bigg(',...
               '\\lambda_1\\frac{\\partial}{\\partial x_1}\\ivar_1(\\bfmu)+',...
               '\\lambda_2\\frac{\\partial}{\\partial x_2}\\ivar_2(\\bfmu)+',...
               '\\frac{\\lambda_2}{x_1}\\ivar_1(\\bfmu)\\bigg)\\textbf{n}_1 +',...
               '\\bigg(',...
               '\\lambda_3\\frac{\\partial}{\\partial x_1}\\ivar_2(\\bfmu)+',...
               '\\lambda_3\\frac{\\partial}{\\partial x_2}\\ivar_1(\\bfmu)\\bigg)\\textbf{n}_2 \\nonumber \\\\',...
               '\\sigma_{2} &=& \\bigg(',...
               '\\lambda_3\\frac{\\partial}{\\partial x_1}\\ivar_2(\\bfmu)+',...
               '\\lambda_3\\frac{\\partial}{\\partial x_2}\\ivar_1(\\bfmu)\\bigg)\\textbf{n}_1 +',...
               '\\bigg(',...
               '\\lambda_2\\frac{\\partial}{\\partial x_1}\\ivar_1(\\bfmu)+',...
               '\\lambda_1\\frac{\\partial}{\\partial x_2}\\ivar_2(\\bfmu)+',...
               '\\frac{\\lambda_2}{x_1}\\ivar_1(\\bfmu)\\bigg)\\textbf{n}_2 \\nonumber'];
        instruct.gen_Neumann_tex = [' components', sprintf('\\\\begin{eqnarray}\n'), tmp, sprintf('\\\\end{eqnarray}\n')];
end

%% gen_bc_list
switch original_input.probtype
    case {'TH','TH2'}
        tmpstr = 'On ';
        tmp = find(original_input.dirichlet.r==0);
        if length(tmp)>0
            if length(tmp)>1
                tmpstr = [tmpstr,'boundaries $\\Gamma_{',num2str(original_input.dirichlet.edge(tmp(1))),'}$'];
            else
                tmpstr = [tmpstr,'boundary $\\Gamma_{',num2str(original_input.dirichlet.edge(tmp(1))),'}$'];
            end
            for i = 2:length(tmp)-1
                tmpstr = [tmpstr,', $\\Gamma_{',num2str(original_input.dirichlet.edge(tmp(i))),'}$',];
            end
            if length(tmp)>1
                tmpstr = [tmpstr,' and $\\Gamma_{',num2str(original_input.dirichlet.edge(tmp(end))),'}$'];
            end
            tmpstr = [tmpstr,' we impose homogeneous Dirichlet conditions $\\ivar(\\bfmu)=0$, while on '];
        end
        % find set of internal edges
        all_edge = original_input.geometry{1};
        for i = 2:length(original_input.geometry)
            all_edge = [all_edge, original_input.geometry{i}];
        end
        internal_edge_list_count = 0;
        internal_edge_list = [];
        for i = 1:size(original_input.edge,1)
            if length(find(i == all_edge))>1
                internal_edge_list_count = internal_edge_list_count + 1;
                internal_edge_list(internal_edge_list_count) = i;
            end
        end
        tmp = setdiff(setdiff(1:size(original_input.edge,1),union(original_input.nload.edge,original_input.dirichlet.edge)),internal_edge_list);
        if length(tmp)>0
            if length(tmp)>1
                tmpstr = [tmpstr,'boundaries $\\Gamma_{',num2str(tmp(1)),'}$'];
            else
                tmpstr = [tmpstr,'boundary $\\Gamma_{',num2str(tmp(1)),'}$'];
            end
            for i = 2:length(tmp)-1
                tmpstr = [tmpstr,', $\\Gamma_{',num2str(tmp(i)),'}$'];
            end
            if length(tmp)>1
                tmpstr = [tmpstr,' and $\\Gamma_{',num2str(tmp(end)),'}$'];
            end
            tmpstr = [tmpstr,' we impose homogeneous Neumann conditions, $\\textbf{n}_i\\kappa_{ij}\\dfrac{\\partial}{\\partial x_j}\\ivar(\\bfmu) = 0$. '];
        end
        bc_nonhomo_dirichlet = find(original_input.dirichlet.r~=0);
        bc_nonhomo_neumann = find(original_input.nload.q==0);
        bc_robin = find(original_input.nload.q~=0);
        if (length(bc_nonhomo_dirichlet)+length(bc_nonhomo_neumann)+length(bc_robin)) > 0
            tmpstr = [tmpstr,'In addition we impose non-homogeneous '];
            count = 0;
            if length(bc_nonhomo_dirichlet) > 1
                tmpstr = [tmpstr,'Dirichlet conditions,',sprintf('\n'),'\\begin{eqnarray}'];
             elseif length(bc_nonhomo_dirichlet) == 1
                tmpstr = [tmpstr,'Dirichlet condition,',sprintf('\n'),'\\begin{eqnarray}'];
            end
            for i = 1:length(bc_nonhomo_dirichlet)
                % cuong                
                if isfield(original_input,'dt') & isfield(original_input,'nt')
                    if original_input.dirichlet.r(bc_nonhomo_dirichlet(i)) == 1
                        tmpdt = '';
                    elseif original_input.dirichlet.r(bc_nonhomo_dirichlet(i)) == -1
                        tmpdt = '-';
                    else
                        tmpdt = replacemulatex(latex(original_input.dirichlet.r(bc_nonhomo_dirichlet(i))),original_input.P,'\\\\');
                    end
                    tmpstr = [tmpstr,sprintf('\n'),' \\ivar(\\bfmu) = ',tmpdt,...
                              'g(t) & {\\rm on}& \\Gamma_{',num2str(original_input.dirichlet.edge(bc_nonhomo_dirichlet(i))),'}'];
                else
                    tmpstr = [tmpstr,sprintf('\n'),' \\ivar(\\bfmu) = ',...
                    replacemulatex(latex(original_input.dirichlet.r(bc_nonhomo_dirichlet(i))),original_input.P,'\\\\'),...
                    ' & {\\rm on}& \\Gamma_{',num2str(original_input.dirichlet.edge(bc_nonhomo_dirichlet(i))),'}'];
                end
                % cuong
                if i < length(bc_nonhomo_dirichlet)
                    tmpstr = [tmpstr,',\\nonumber\\\\'];
                else
                    tmpstr = [tmpstr,'\\nonumber,'];
                end                
                count = count + 1;
            end
            if length(bc_nonhomo_dirichlet) > 0
                tmpstr = [tmpstr,sprintf('\n\\\\end{eqnarray}\n')];
            end

            if length(bc_nonhomo_neumann) > 1
                tmpstr = [tmpstr,'Neumann conditions,',sprintf('\n'),'\\begin{eqnarray}'];
            elseif length(bc_nonhomo_neumann) == 1
                tmpstr = [tmpstr,'Neumann condition,',sprintf('\n'),'\\begin{eqnarray}'];
            end
            for i = 1:length(bc_nonhomo_neumann)
                tmp = zeros(1,length(original_input.geometry));
                for igeo = 1:length(original_input.geometry)
                    tmp(igeo) = ismember(original_input.nload.edge(bc_nonhomo_neumann(i)),original_input.geometry{igeo});
                end
                tmp = find(tmp==1);
                % cuong
                if isfield(original_input,'dt') & isfield(original_input,'nt')
                    if original_input.nload.g(bc_nonhomo_neumann(i)) == 1
                        tmpdt = '';
                    elseif original_input.nload.g(bc_nonhomo_neumann(i)) == -1
                        tmpdt = '-';
                    else
                        tmpdt = replacemulatex(latex(original_input.nload.g(bc_nonhomo_neumann(i))),original_input.P,'\\\\');
                    end
                    tmp = ['\\textbf{n}_i\\kappa^{',num2str(tmp(1)),'}_{ij}\\frac{\\partial\\ivar}{\\partial x_j}(\\bfmu) = ',tmpdt,...
                           'g(t) & {\\rm on} & \\Gamma_{',num2str(original_input.nload.edge(bc_nonhomo_neumann(i))),'}'];                
                else
                    tmp = ['\\textbf{n}_i\\kappa^{',num2str(tmp(1)),'}_{ij}\\frac{\\partial\\ivar}{\\partial x_j}(\\bfmu) = ',...
                           replacemulatex(latex(original_input.nload.g(bc_nonhomo_neumann(i))),original_input.P,'\\\\'),...
                           ' & {\\rm on} & \\Gamma_{',num2str(original_input.nload.edge(bc_nonhomo_neumann(i))),'}'];                
                end
                % cuong
                tmpstr = [tmpstr,sprintf('\n %s',tmp)];
                if i < length(bc_nonhomo_neumann)
                    tmpstr = [tmpstr,',\\nonumber\\\\'];
                else
                    tmpstr = [tmpstr,'\\nonumber,'];
                end
                count = count + 1;
            end
            if length(bc_nonhomo_neumann) > 0
                tmpstr = [tmpstr,sprintf('\n\\\\end{eqnarray}\n')];
            end
            
            if length(bc_robin) > 1
                tmpstr = [tmpstr,'Robin conditions,',sprintf('\n'),'\\begin{eqnarray}'];
            elseif length(bc_robin) == 1
                tmpstr = [tmpstr,'Robin condition,',sprintf('\n'),'\\begin{eqnarray}'];
            end
            for i = 1:length(bc_robin)                
                tmp = zeros(1,length(original_input.geometry));
                for igeo = 1:length(original_input.geometry)
                    tmp(igeo) = ismember(original_input.nload.edge(bc_robin(i)),original_input.geometry{igeo});
                end
                tmp = find(tmp==1);
                tmp1 = replacemulatex(latex(original_input.nload.q(bc_robin(i))),original_input.P,'\\\\');
                % cuong
                if (length(tmp1) == 1) && (tmp1 == '1')
                    if isfield(original_input,'dt') & isfield(original_input,'nt') 
                        if original_input.nload.g(bc_robin(i)) == 1
                            tmpdt = '';
                        elseif original_input.nload.g(bc_robin(i)) == -1
                            tmpdt = '-';
                        else
                            tmpdt = replacemulatex(latex(original_input.nload.g(bc_robin(i))),original_input.P,'\\\\');
                        end
                        tmp = ['\\textbf{n}_i\\kappa^{',num2str(tmp(1)),'}_{ij}\\frac{\\partial\\ivar}{\\partial x_j}(\\bfmu) + \\ivar = ',tmpdt,...
                            'g(t) & {\\rm on} & \\Gamma_{',num2str(original_input.nload.edge(bc_robin(i))),'}'];
                    else
                        tmp = ['\\textbf{n}_i\\kappa^{',num2str(tmp(1)),'}_{ij}\\frac{\\partial\\ivar}{\\partial x_j}(\\bfmu) + \\ivar = ',replacemulatex(latex(original_input.nload.g(bc_robin(i))),original_input.P,'\\\\'),...
                            ' & {\\rm on} & \\Gamma_{',num2str(original_input.nload.edge(bc_robin(i))),'}'];
                    end
                elseif (length(tmp1) == 2) && (tmp1 == '-1')
                    if isfield(original_input,'dt') & isfield(original_input,'nt')    
                        if original_input.nload.g(bc_robin(i)) == 1
                            tmpdt = '';
                        elseif original_input.nload.g(bc_robin(i)) == -1
                            tmpdt = '-';
                        else
                            tmpdt = replacemulatex(latex(original_input.nload.g(bc_robin(i))),original_input.P,'\\\\');
                        end
                        tmp = ['\\textbf{n}_i\\kappa^{',num2str(tmp(1)),'}_{ij}\\frac{\\partial\\ivar}{\\partial x_j}(\\bfmu) - \\ivar = ',tmpdt,...
                            'g(t) & {\\rm on} & \\Gamma_{',num2str(original_input.nload.edge(bc_robin(i))),'}'];
                    else
                        tmp = ['\\textbf{n}_i\\kappa^{',num2str(tmp(1)),'}_{ij}\\frac{\\partial\\ivar}{\\partial x_j}(\\bfmu) - \\ivar = ',replacemulatex(latex(original_input.nload.g(bc_robin(i))),original_input.P,'\\\\'),...
                            ' & {\\rm on} & \\Gamma_{',num2str(original_input.nload.edge(bc_robin(i))),'}'];
                    end
                else
                    if isfield(original_input,'dt') & isfield(original_input,'nt')
                        if original_input.nload.g(bc_robin(i)) == 1
                            tmpdt = '';
                        elseif original_input.nload.g(bc_robin(i)) == -1
                            tmpdt = '-';
                        else
                            tmpdt = replacemulatex(latex(original_input.nload.g(bc_robin(i))),original_input.P,'\\\\');
                        end
                        tmp = ['\\textbf{n}_i\\kappa^{',num2str(tmp(1)),'}_{ij}\\frac{\\partial\\ivar}{\\partial x_j}(\\bfmu) + (',...
                            tmp1,')\\ivar = ',tmpdt,...
                            'g(t) & {\\rm on} & \\Gamma_{',num2str(original_input.nload.edge(bc_robin(i))),'}'];
                    else
                        tmp = ['\\textbf{n}_i\\kappa^{',num2str(tmp(1)),'}_{ij}\\frac{\\partial\\ivar}{\\partial x_j}(\\bfmu) + (',...
                            tmp1,')\\ivar = ',replacemulatex(latex(original_input.nload.g(bc_robin(i))),original_input.P,'\\\\'),...
                            ' & {\\rm on} & \\Gamma_{',num2str(original_input.nload.edge(bc_robin(i))),'}'];
                    end
                end
                % cuong
                tmpstr = [tmpstr,sprintf('\n %s',tmp)];
                if i < length(bc_robin)
                    tmpstr = [tmpstr,',\\nonumber\\\\'];
                else
                    tmpstr = [tmpstr,'\\nonumber,'];
                end
                count = count + 1;
            end
            if length(bc_robin) > 0
                tmpstr = [tmpstr,sprintf('\n\\\\end{eqnarray}\n')];
                tmpstr = [tmpstr,'corresponding to \\textcolor{red}{flux explanation}.',sprintf('\n')];
            end            
        end
        if isfield(original_input,'dt') & isfield(original_input,'nt') & (original_input.fareaload(igeo) == 0)
            tmpstr = [tmpstr, 'Here the control input $g(t)$ is a function of time $t$. The problem considers any square-integrable function for $g(t)$.'];
        end
        instruct.gen_bc_list = tmpstr;
    case {'LE1','LE2','LE3','LE4'}
        tmpstr = 'On ';
        for i = 1:length(original_input.dirichlet.edge)
            tmpstr = [tmpstr,'boundary $\\Gamma_{',num2str(original_input.dirichlet.edge(i)),'}$'];
            if strcmp(char(original_input.dirichlet.r(i,2)),'0')
                tmp = '';
            else
                tmp = 'non-';
            end    
            if strcmp(char(original_input.dirichlet.r(i,1)),'x')
                tmpstr = [tmpstr,' we impose ',tmp,'homogeneous Dirichlet conditions $\\ivar_1(\\bfmu)=',replacemulatex(latex(original_input.dirichlet.r(i,2)),original_input.P,'\\\\'),'$; on '];
            elseif strcmp(char(original_input.dirichlet.r(i,1)),'y')
                tmpstr = [tmpstr,' we impose ',tmp,'homogeneous Dirichlet conditions $\\ivar_2(\\bfmu)=',replacemulatex(latex(original_input.dirichlet.r(i,2)),original_input.P,'\\\\'),'$; on '];
            end            
        end
        tmpstr = [tmpstr(1:end-5),'. In addition, on '];
        for i = 1:length(original_input.nload.edge)
            tmpstr = [tmpstr,'boundary $\\Gamma_{',num2str(original_input.nload.edge(i)),'}$'];
            if not(strcmp(char(original_input.nload.g(i,1)),'0'))
                tmp = replacemulatex(latex(original_input.nload.g(i,1)),original_input.P,'\\\\');
                if strcmp(tmp,'1')
                    tmp = '';
                elseif strcmp(tmp,'-1')
                    tmp = '-';
                end
                tmpstr = [tmpstr,' we impose non-homogeneous Neumann conditions,',sprintf('\n'),'\\begin{equation}',sprintf('\n'),...                  
                          'C_{ijkl}(\\bfmu)\\dfrac{\\partial}{\\partial x_l}\\ivar_k(\\bfmu)\\textbf{n}_j = ',tmp,'\\textbf{n}_i,\\nonumber',sprintf('\n'),...
                          '\\end{equation}',sprintf('\n'),' on '];
            elseif not(strcmp(char(original_input.nload.g(i,2)),'0'))
                tmp = replacemulatex(latex(original_input.nload.g(i,2)),original_input.P,'\\\\');
                if strcmp(tmp,'1')
                    tmp = '';
                elseif strcmp(tmp,'-1')
                    tmp = '-';
                end
                tmpstr = [tmpstr,' we impose non-homogeneous Neumann conditions,',sprintf('\n'),'\\begin{equation}',sprintf('\n'),...                  
                          'C_{ijkl}(\\bfmu)\\dfrac{\\partial}{\\partial x_l}\\ivar_k(\\bfmu)\\textbf{n}_j = ',tmp,'\\textbf{n}_i,\\nonumber',sprintf('\n'),...
                          '\\end{equation}',sprintf('\n'),' on '];
            end
        end
        if tmpstr(end-31) == ','
            tmpstr(end-31) = '.';
        end
        if strcmp(tmpstr(end-3:end),' on ')
            tmpstr = tmpstr(1:end-4);
        end
        tmpstr = [tmpstr,'We impose homogeneous Neumann conditions on any remaining boundary segments/vector components.'];
        instruct.gen_bc_list = tmpstr;
    case {'LE5'}
        if ismember(0,original_input.preal(:,1))
            tmp = 'On the axial axis ($x_1 = 0$) we impose homogeneous Dirichlet conditions $\\ivar_1(\\bfmu)= 0$. ';
        else
            tmp = [];
        end
        tmpstr = [tmp, 'On '];
        for i = 1:length(original_input.dirichlet.edge)
            tmpstr = [tmpstr,'boundary $\\Gamma_{',num2str(original_input.dirichlet.edge(i)),'}$'];
            if strcmp(char(original_input.dirichlet.r(i,2)),'0')
                tmp = '';
            else
                tmp = 'non-';
            end    
            if strcmp(char(original_input.dirichlet.r(i,1)),'x')
                tmpstr = [tmpstr,' we impose ',tmp,'homogeneous Dirichlet conditions $\\ivar_1(\\bfmu)=',replacemulatex(latex(original_input.dirichlet.r(i,2)),original_input.P,'\\\\'),'$; on '];
            elseif strcmp(char(original_input.dirichlet.r(i,1)),'y')
                tmpstr = [tmpstr,' we impose ',tmp,'homogeneous Dirichlet conditions $\\ivar_2(\\bfmu)=',replacemulatex(latex(original_input.dirichlet.r(i,2)),original_input.P,'\\\\'),'$; on '];
            end            
        end
        tmpstr = [tmpstr(1:end-5),'. In addition, on '];
        for i = 1:length(original_input.nload.edge)
            tmpstr = [tmpstr,'boundary $\\Gamma_{',num2str(original_input.nload.edge(i)),'}$'];
            if not(strcmp(char(original_input.nload.g(i,1)),'0'))
                tmp = replacemulatex(latex(original_input.nload.g(i,1)),original_input.P,'\\\\');
                if strcmp(tmp,'1')
                    tmp = '';
                elseif strcmp(tmp,'-1')
                    tmp = '-';
                end
                tmpstr = [tmpstr,' we impose non-homogeneous Neumann conditions,',sprintf('\n'),'\\begin{eqnarray}',sprintf('\n'),...                  
                          '\\sigma_i = ',tmp,'\\textbf{n}_i,\\nonumber',sprintf('\n'),...
                          '\\end{eqnarray}',sprintf('\n'),' on '];
            elseif not(strcmp(char(original_input.nload.g(i,2)),'0'))
                tmp = replacemulatex(latex(original_input.nload.g(i,2)),original_input.P,'\\\\');
                if strcmp(tmp,'1')
                    tmp = '';
                elseif strcmp(tmp,'-1')
                    tmp = '-';
                end
                tmpstr = [tmpstr,' we impose non-homogeneous Neumann conditions,',sprintf('\n'),'\\begin{eqnarray}',sprintf('\n'),...                  
                          '\\sigma_i = ',tmp,'\\textbf{n}_i,\\nonumber',sprintf('\n'),...
                          '\\end{eqnarray}',sprintf('\n'),' on '];
            end
        end
        if tmpstr(end-31) == ','
            tmpstr(end-31) = '.';
        end
        if strcmp(tmpstr(end-3:end),' on ')
            tmpstr = tmpstr(1:end-4);
        end
        tmpstr = [tmpstr,'We impose homogeneous Neumann conditions on any remaining boundary segments/vector components.'];
        instruct.gen_bc_list = tmpstr;
end


%% gen_output_func
if isfield(original_input,'dLIFT')
    dirfluxflag = 1;
else
    dirfluxflag = 0;
end
if isfield(original_input,'oload')
    oloadflag = 1;
else
    oloadflag = 0;
end
if isfield(original_input,'oareaload')
    oarealoadflag = 1;
else
    oarealoadflag = 0;
end
switch original_input.probtype
    case {'TH','TH2'}
        if strcmp(original_input.probtype,'TH2')
            extratmp = sym('x');
        else
            extratmp = 1;
        end
        tmpstr = '';
        if not(dirfluxflag)
            if oloadflag
                original_input.oload = sym(original_input.oload);
                tmp1 = '';
                for i = 1:size(original_input.oload,1)
                    if length(findsym(original_input.oload(i,2)*extratmp)) == 0
                        if original_input.oload(i,2) == 1
                            tmp = '+';
                            tmp2 = '';
                        elseif original_input.oload(i,2) == -1
                            tmp = '-';
                            tmp2 = '';
                        else
                            tmp = '+';
                            tmp2 = replacemulatex(latex(original_input.oload(i,2)*extratmp),original_input.P,'\\\\');
                        end
                    else
                        tmp = '+';
                        tmp2 = replacemulatex(latex(original_input.oload(i,2)*extratmp),original_input.P,'\\\\');
                    end
                    tmp1 = [tmp1,tmp,'\\int_{\\Gamma_{',char(original_input.oload(i,1)),'}}',tmp2,'\\ivar(\\bfmu)'];
                end
                tmpstr = [tmpstr,tmp1];
            end
            if oarealoadflag
                original_input.oareaload = sym(original_input.oareaload);
                tmp1 = '';
                for i = 1:size(original_input.oareaload,1)
                    if length(findsym(original_input.oareaload(i,2)*extratmp)) == 0
                        if original_input.oareaload(i,2) == 1
                            tmp = '+'; tmp2 = '';
                        elseif original_input.oareaload(i,2) == -1
                            tmp = '-'; tmp2 = '';
                        else
                            tmp = '+'; tmp2 = replacemulatex(latex(original_input.oareaload(i,2)*extratmp),original_input.P,'\\\\');
                        end
                    else
                        tmp = '+'; tmp2 = replacemulatex(latex(original_input.oareaload(i,2)*extratmp),original_input.P,'\\\\');
                    end
                    tmp1 = [tmp1,tmp,'\\int_{\\calR_{',char(original_input.oareaload(i,1)),'}}',tmp2,'\\ivar(\\bfmu)'];
                end
                tmpstr = [tmpstr,tmp1];
            end
            if tmpstr(1) == '+'
                tmpstr = tmpstr(2:end);
            end
        else
            original_input.dLIFT = sym(original_input.dLIFT);
            tmp1 = '';
            for i = 1:size(original_input.dLIFT,1)
                if length(findsym(original_input.dLIFT(i,2))) == 0
                    if original_input.dLIFT(i,2) == 1
                        tmp = '+'; tmp2 = '';
                    elseif original_input.dLIFT(i,2) == -1
                        tmp = '-'; tmp2 = '';
                    else
                        tmp = '+'; tmp2 = replacemulatex(latex(original_input.dLIFT(i,2)),original_input.P,'\\\\');
                    end
                else
                    tmp = '+'; tmp2 = replacemulatex(latex(original_input.dLIFT(i,2)),original_input.P,'\\\\');
                end
                tmp1 = [tmp1,tmp,'\\int_{\\Gamma_{',char(original_input.dLIFT(i,1)),'}}',tmp2,'\\textbf{n}_i\\kappa_{ij}\\dfrac{\\partial}{\\partial x_j}\\ivar(\\bfmu)'];
            end
            tmpstr = [tmpstr,tmp1];
            if tmpstr(1) == '+'
                tmpstr = tmpstr(2:end);
            end
        end
        % cuong
        if isfield(original_input,'dt') & isfield(original_input,'nt')
            tmp1=['\\int_0^T \\bigg( h(t)'];
            tmp2=['\\bigg) dt'];
            tmpstr = [tmp1, tmpstr, tmp2];
        end
        % cuong
        instruct.gen_output_func =  [sprintf('\\\\begin{equation}\n'),'\\ovar(\\bfmu) = ',tmpstr,sprintf(',\\\\nonumber\n\\\\end{equation}')];
        if isfield(original_input,'dt') & isfield(original_input,'nt') 
            if oarealoadflag==1
                tmp = ['$h(t)$ is a function of time $t$, \\textcolor{red}{input output explanation}. The problem considers any function (including Dirac delta) for $h(t)$'];
            else
                tmp = ['$h(t)$ is a function of time $t$, \\textcolor{red}{input output explanation}. The problem considers any square-integrable function for $h(t)$'];
            end
            instruct.output_explanation = tmp;
        else
            instruct.output_explanation = ['\\textcolor{red}{input output explanation}'];
        end
    case {'LE1','LE2','LE3','LE4','LE5'}
        if strcmp(original_input.probtype,'LE5')
            extratmp = sym('x');
        else
            extratmp = 1;
        end
        tmpstr = '';
        if not(dirfluxflag)
            if oloadflag
                original_input.oload = sym(original_input.oload);
                tmp1 = '';
                for i = 1:size(original_input.oload,1)
                    if length(findsym(original_input.oload(i,2)*extratmp)) == 0
                        if original_input.oload(i,2) == 1
                            tmp = '+'; tmp2 = '';
                        elseif original_input.oload(i,2) == -1
                            tmp = '-'; tmp2 = '';
                        elseif original_input.oload(i,2) == 0
                            tmp = ''; tmp2 = '';
                        else
                            tmp = '+'; tmp2 = replacemulatex(latex(original_input.oload(i,2)*extratmp),original_input.P,'\\\\');
                        end
                    else
                        tmp = '+'; tmp2 = replacemulatex(latex(original_input.oload(i,2)*extratmp),original_input.P,'\\\\');
                    end
                    if length(tmp)>0
                        tmp1 = [tmp1,tmp,'\\int_{\\Gamma_{',char(original_input.oload(i,1)),'}}',tmp2,'\\ivar_i(\\bfmu)\\textbf{e}_i'];
                    end
                end
                tmpstr = [tmpstr,tmp1];
                tmp1 = '';
                for i = 1:size(original_input.oload,1)
                    if length(findsym(original_input.oload(i,3)*extratmp)) == 0
                        if original_input.oload(i,3) == 1
                            tmp = '+'; tmp2 = '';
                        elseif original_input.oload(i,3) == -1
                            tmp = '-'; tmp2 = '';
                        elseif original_input.oload(i,3) == 0
                            tmp = ''; tmp2 = '';
                        else
                            tmp = '+'; tmp2 = replacemulatex(latex(original_input.oload(i,3)*extratmp),original_input.P,'\\\\');
                        end
                    else
                        tmp = '+'; tmp2 = replacemulatex(latex(original_input.oload(i,3)*extratmp),original_input.P,'\\\\');
                    end
                    if length(tmp)>0
                        tmp1 = [tmp1,tmp,'\\int_{\\Gamma_{',char(original_input.oload(i,1)),'}}',tmp2,'\\ivar_i(\\bfmu)\\textbf{n}_i'];
                    end
                end
                tmpstr = [tmpstr,tmp1];
            end
            if oarealoadflag
                original_input.oareaload = sym(original_input.oareaload);
                tmp1 = '';
                for i = 1:size(original_input.oareaload,1)
                    tmp1 = [tmp1,'+\\int_{\\calR_',char(original_input.oareaload(i,1)),'}{\\left[\\begin{array}{c} ',replacemulatex(latex(original_input.oareaload(i,2)*extratmp),'\\\\'),'\\\\ ',replacemulatex(latex(original_input.oareaload(i,3)*extratmp),'\\\\'),'\\\\ \\end{array}\\right]} }}\\ivar(\\bfmu)'];
                end
                tmpstr = [tmpstr,tmp1];
            end
            if tmpstr(1) == '+'
                tmpstr = tmpstr(2:end);
            end
        else
            original_input.dLIFT = sym(original_input.dLIFT);
            tmp1 = '';
            for i = 1:size(original_input.dLIFT,1)
                if length(findsym(original_input.dLIFT(i,2))) == 0
                    if original_input.dLIFT(i,2) == 1
                        tmp = '+';
                    elseif original_input.dLIFT(i,2) == -1
                        tmp = '-';
                    elseif original_input.dLIFT(i,2) == 0
                        tmp = '';
                    else
                        tmp = ['+',replacemulatex(latex(original_input.dLIFT(i,2)),original_input.P,'\\\\')];
                    end
                else
                    tmp = ['+',replacemulatex(latex(original_input.dLIFT(i,2)),original_input.P,'\\\\')];
                end
                if length(tmp)>0
                    if strcmp(original_input.probtype,'LE5')
                        tmp1 = [tmp1,tmp,'\\int_{\\Gamma_{',char(original_input.dLIFT(i,1)),'}}\\sigma_1'];
                    else
                        tmp1 = [tmp1,tmp,'\\int_{\\Gamma_{',char(original_input.dLIFT(i,1)),'}}C_{ijkl}(\\bfmu)\\dfrac{\\partial}{\\partial x_l}\\ivar_k(\\bfmu)\\textbf{n}_j'];
                    end
                end
            end
            tmpstr = [tmpstr,tmp1];
            tmp1 = '';
            for i = 1:size(original_input.dLIFT,1)
                if length(findsym(original_input.dLIFT(i,3))) == 0
                    if original_input.dLIFT(i,3) == 1
                        tmp = '+';
                    elseif original_input.dLIFT(i,3) == -1
                        tmp = '-';
                    elseif original_input.dLIFT(i,3) == 0
                        tmp = '';
                    else
                        tmp = ['+',replacemulatex(latex(original_input.dLIFT(i,3)),original_input.P,'\\\\')];
                    end
                else
                    tmp = ['+',replacemulatex(latex(original_input.dLIFT(i,3)),original_input.P,'\\\\')];
                end
                if length(tmp)>0
                    if strcmp(original_input.probtype,'LE5')
                        tmp1 = [tmp1,tmp,'\\int_{\\Gamma_{',char(original_input.dLIFT(i,1)),'}}\\sigma_2'];
                    else
                        tmp1 = [tmp1,tmp,'\\int_{\\Gamma_{',char(original_input.dLIFT(i,1)),'}}C_{ijkl}(\\bfmu)\\dfrac{\\partial}{\\partial x_l}\\ivar_k(\\bfmu)\\textbf{n}_j'];
                    end
                end
            end
            tmpstr = [tmpstr,tmp1];
            if tmpstr(1) == '+'
                tmpstr = tmpstr(2:end);
            end
        end
        instruct.gen_output_func =  [sprintf('\\\\begin{equation}\n'),'\\ovar(\\bfmu) = ',tmpstr,sprintf(',\\\\nonumber\n\\\\end{equation}')];
end

%% gen_Nt
instruct.gen_Nt = ['$\\calN_{\\rm t} = ',num2str(size(Btmp,1)),'$'];

%% gen_webserver_http_link
tmp = ['http://augustine.mit.edu/workedproblems/rbMIT/',probname,'/F_rbMIT_',probname,'_webserver.htm'];
instruct.gen_webserver_http_link = ['\\href{',tmp,'}{webserver}'];

%% gen_rbU_http_link
tmp = ['http://augustine.mit.edu/workedproblems/rbMIT/',probname,'/rbMIT_data/rbU_',probname,'.m'];
instruct.gen_rbU_http_link = ['\\href{',tmp,'}{rbU file}'];

% generate figures

% generate geometry figure
curvyres = 16;
numgeodr = length(original_input.geometry);
x1min = min(preal(:,1)); x2min = min(preal(:,2));
x1max = max(preal(:,1)); x2max = max(preal(:,2));
x1len = x1max - x1min; x2len = x2max - x2min;
x1minb = x1min - .15*x1len; x1maxb = x1max + .15*x1len;
x2minb = x2min - .15*x2len; x2maxb = x2max + .15*x2len;
gcf1 = figure('Visible','Off'); figure(gcf1); hold on;
gcf2 = figure('Visible','Off'); figure(gcf2); hold on;
for igdr = 1:numgeodr
    numeddr = size(original_input.geometry{igdr},2);
    for iedr = 1:numeddr
        eddr = original_input.geometry{igdr}(iedr);
        if(original_input.tedgeval(eddr,1) == 0)
            x1pt = [original_input.preal(original_input.edge(eddr,1),1),original_input.preal(original_input.edge(eddr,2),1)];
            x2pt = [original_input.preal(original_input.edge(eddr,1),2),original_input.preal(original_input.edge(eddr,2),2)];
            if ismember(eddr,original_input.nload.edge)
                figure(gcf1); plot(x1pt,x2pt,'b-','Linewidth',2);                
                figure(gcf2); plot(x1pt,x2pt,'r--','Linewidth',2);
            else
                figure(gcf1); plot(x1pt,x2pt,'b-','Linewidth',2);
                figure(gcf2); plot(x1pt,x2pt,'b-','Linewidth',2);
            end
            for itdraw = 1:curvyres
                expt{eddr}(itdraw,:) = [x1pt(1)+(itdraw-0.5)*(x1pt(2)-x1pt(1))/curvyres, x2pt(1)+(itdraw-0.5)*(x2pt(2)-x2pt(1))/curvyres];
                enpt{eddr}(itdraw,:) = [x2pt(1)-x2pt(2),x1pt(2)-x1pt(1)]; enpt{eddr}(itdraw,:) = enpt{eddr}(itdraw,:)/norm(enpt{eddr}(itdraw,:));
                etpt{eddr}(itdraw,:) = [x1pt(2)-x1pt(1),x2pt(2)-x2pt(1)]; etpt{eddr}(itdraw,:) = etpt{eddr}(itdraw,:)/norm(etpt{eddr}(itdraw,:));
            end
        else
            labdraw = double(original_input.tedgeval(eddr,1));
            dtdraw = double((original_input.tedgeval(eddr,3) - original_input.tedgeval(eddr,2)))/curvyres;
            for itdraw = 1:curvyres
                tvalr1 = double(original_input.tedgeval(eddr,2)) + (itdraw-1)*dtdraw;
                tvalr2 = tvalr1 + dtdraw;
                xfirstptsym = simplify(subs(original_input.curvedatsym{labdraw}.gfun,'t',tvalr1));
                xsecondptsym = simplify(subs(original_input.curvedatsym{labdraw}.gfun,'t',tvalr2));
                x1ptsym = [xfirstptsym(1),xsecondptsym(1)]; x2ptsym = [xfirstptsym(2),xsecondptsym(2)];
                tmp1 = x1ptsym; tmp2 = x2ptsym;
                for i = 1:P
                    tmp1 = subs(tmp1,strcat('mu',num2str(i)),original_input.muref(i),0); % phuong 2/2/09
                    tmp2 = subs(tmp2,strcat('mu',num2str(i)),original_input.muref(i),0); % phuong 2/2/09
                end;
                if ismember(eddr,original_input.nload.edge)
                    figure(gcf1); plot(tmp1,tmp2,'b-','Linewidth',2);                    
                    figure(gcf2); plot(tmp1,tmp2,'r--','Linewidth',2);
                else
                    figure(gcf1); plot(tmp1,tmp2,'b-','Linewidth',2);
                    figure(gcf2); plot(tmp1,tmp2,'b-','Linewidth',2);
                end
                enpt{eddr}(itdraw,:) = [tmp2(1)-tmp2(2),tmp1(2)-tmp1(1)]; enpt{eddr}(itdraw,:) = enpt{eddr}(itdraw,:)/norm(enpt{eddr}(itdraw,:));               
                etpt{eddr}(itdraw,:) = [tmp1(2)-tmp1(1),tmp2(2)-tmp2(1)]; etpt{eddr}(itdraw,:) = etpt{eddr}(itdraw,:)/norm(etpt{eddr}(itdraw,:));
                expt{eddr}(itdraw,:) = [sum(tmp1)/2, sum(tmp2)/2];
            end;
        end;
    end;
end;
figure(gcf1); plot(original_input.preal(:,1),original_input.preal(:,2),'k.','MarkerSize',20);
figure(gcf2); plot(original_input.preal(:,1),original_input.preal(:,2),'k.','MarkerSize',20);
figure(gcf1); axis([x1minb,x1maxb,x2minb,x2maxb]); axis off; axis equal;
figure(gcf2); axis([x1minb,x1maxb,x2minb,x2maxb]); axis off; axis equal;
%% plot domains and boundary and BC on figure 2
figure(gcf2);
for i = 1:size(original_input.edge,1)
    %% plot Dirichlet bc on figure 2
    if ismember(i,dirichlet.edge)
        for itdraw = 1:curvyres
            switch original_input.probtype
                case {'TH','TH2'}
                    plot([expt{i}(itdraw,1),expt{i}(itdraw,1)+0.05*norm([x1len,x2len])*(enpt{i}(itdraw,1)+etpt{i}(itdraw,1))/2],[expt{i}(itdraw,2),expt{i}(itdraw,2)+0.05*norm([x1len,x2len])*(enpt{i}(itdraw,2)+etpt{i}(itdraw,2))/2],'-k');
                case {'LE1','LE2','LE3','LE4'}
                    [tmploc,tmp] = find(original_input.dirichlet.edge == i);
                    for j = 1:length(tmploc)
                        if strcmp(char(original_input.dirichlet.r(tmploc(j),1)),'x')
                            plot(expt{i}(itdraw,1)-x1len/60,expt{i}(itdraw,2),'>k');
                        elseif strcmp(char(original_input.dirichlet.r(tmploc(j),1)),'y')
                            plot(expt{i}(itdraw,1),expt{i}(itdraw,2)-x1len/60,'^k');
                        end                            
                    end
            end
        end
    end
    text(expt{i}(floor(curvyres/2),1)-x1len/20,expt{i}(floor(curvyres/2),2),['$','\Gamma_{',num2str(i),'}$'],'Interpreter','latex','FontSize',12,'Fontname','Times New Roman','BackgroundColor','white');
end
%% plot domains and boundary
for i = 1:size(original_input.geometry,2)
    tmp = unique(reshape(original_input.edge(original_input.geometry{i},:),length(original_input.geometry{i})*2,1));
    tmp = [sum(preal(tmp,1)),sum(preal(tmp,2))]/length(tmp);
    if original_input.gflag(i)
        text(tmp(1),tmp(2),['$','\mathcal{R}_{',num2str(i),'}$'],'Interpreter','latex','FontSize',18,'Fontname','Times New Roman','BackgroundColor','white','HorizontalAlignment','center');
    end
end
%% plot symbolic coordinate on figure 1
figure(gcf1);
for i = 1:size(original_input.preal,1)
    tmp = ['$(',latex(original_input.psym(i,1)),', ',latex(original_input.psym(i,2)),')$'];
    for j = 1:P
        tmp = regexprep(tmp, strcat('{\\it mu',num2str(j),'}'), strcat('\\mu_',num2str(j)));
    end
    text(original_input.preal(i,1),original_input.preal(i,2)+0.05*x2len,tmp,'Interpreter','latex','FontSize',12,'Fontname','Times New Roman','HorizontalAlignment','center','FontWeight','bold');%,'BackgroundColor','white');
end

hold off;
saveas(gcf1,[probpath,'/report/geometry1'],'pdf'); % phuong 11/2/08
saveas(gcf1,[probpath,'/report/geometry1'],'epsc'); % phuong 11/2/08
close(gcf1);
saveas(gcf2,[probpath,'/report/geometry2'],'pdf'); % phuong 11/2/08
saveas(gcf2,[probpath,'/report/geometry2'],'epsc'); % phuong 11/2/08
close(gcf2);

figure(gcf1);
pdemesh(p,e,t);
axis off; axis equal;
saveas(gcf1,[probpath,'/report/mesh'],'pdf'); % phuong 11/2/08
saveas(gcf1,[probpath,'/report/mesh'],'epsc'); % phuong 11/2/08
close(gcf1);

filerep(instruct,[webservertemplatepathname,'/F_rbMIT_report.tex'],[reportpath,'/F_rbMIT_',probname,'_report.tex'],'\${','}\$'); % phuong 11/2/08

% try to compile using pdflatex
if OPTS.genpdf
    [ispdflatex,tmp] = system(['pdflatex -help']);
    ispdflatex = not(ispdflatex);
    if ispdflatex
        currentdir = cd;
        cd(reportpath);
        [tmp1,tmp2] = system(['pdflatex F_rbMIT_',probname,'_report.tex']); % phuong 11/2/08
        if OPTS.openpdf
            system(['F_rbMIT_',probname,'_report.pdf']); % phuong 11/2/08
        end
        cd(currentdir);
    end
end

function inputstr = replacemulatex(inputstr,P,inputslash)
if not(exist('inputslash'))
    inputslash = '\';
end
inputstr = regexprep(inputstr,'\it','');
inputstr = regexprep(inputstr,'\,','');
inputstr = regexprep(inputstr,{'x','y'},{'x_1','x_2'}); % phuong 20-8
for i = 1:P
    if ismember(['mu',num2str(i)],inputstr)
        inputstr = regexprep(inputstr,['mu',num2str(i)],[inputslash,'mu_{',num2str(i),'}']);
    end
end

function inputstr = cleanlatex(inputstr,inputslash)
if not(exist('inputslash'))
    inputslash = '\\';
end
inputstr = regexprep(inputstr,'\\it','');
inputstr = regexprep(inputstr,'\\,','');
inputstr = regexprep(inputstr,'\\',inputslash);