
function [eigarray, eiglbarray, eigvecarray, niterarray] = my_lanczos_alpha(amat, bch, neigs, nitermx, tol, ichflag, binit)
% -----rbMIT Software Copyright MIT 2006-07----
% DBP Huynh, NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% ---------- 

% We solve for the first neigs eigenvalues (= singular values squared) of  
%   [bch'\amat*bch\]*eigvec = eigval*Id*eigvec 
% (which is readily obtained from our standard form for the singular value
% problem).

% In case the initial solution (RB of [amat]eigvec2 = 
% eigval*bch'*bch*eigvec2) is feed, need to convert both initial and final
% solutions!

opts.disp = 0;
dim1 = length(amat);
eigarray = zeros(1, neigs);
eigmu = zeros(neigs,1);
eiglbarray = zeros(1, neigs);
eiglbmu = zeros(neigs,1);
niterarray = zeros(1,1);

if not(exist('binit'))
    binit = rand(dim1,1);
else
    % be careful as the two eigenproblems are not the same!
    binit = bch*binit;
end

[eigmu,eiglbmu,eigvec,niter] = lanczos(amat, bch, neigs, nitermx, tol, ichflag, binit);
eigarray = eigmu(:);
eiglbarray = eiglbmu(:);
% be careful as the two eigenproblems are not the same!
eigvecarray = bch\eigvec(:,1);
niterarray = niter;
avtime = toc;
    
function [eigmu,eiglbmu,eigvec,niter] = lanczos(amat, bch, neigs, nitermx, tol, ichflag, binit)

opts.disp = 0;
[n,m] = size(amat); 
% q is the set of (orthogonal) Lanczos vectors./
q = zeros(n,nitermx);
tri = zeros(nitermx,nitermx);
betaim1 = 0;
qim1 = zeros(n,1);
normb = sqrt(binit'*binit);
qi = binit/normb;
q(:,1) = qi;

bch_t = bch';
niter = nitermx;

for i = 1:nitermx;
% Find next Lanczos vector and next row of Lanczos tridiagonal matrix:
    v = bch_t\(amat*(bch\qi)) - betaim1*qim1;
    alphai = qi'*v;
    v = v  - alphai*qi;
    betai = sqrt(v'*v);     
    tri(i,i) = alphai;
    if(i ~= 1);
        tri(i,i-1) = betaim1;
    end;
    if(i ~= nitermx);
        tri(i,i+1) = betai;
    end;                      
    betaim1 = betai;
    qim1 = qi;
    qi = v/betai;
    if(i ~= nitermx)
        q(:,i+1) = qi;
    end;
% Compute eigenvalues of current (i x i) Lanczos tridiagonal "stiffness" matrx tri:
    if i > neigs;
        [v,d] = eigs(tri(1:i,1:i),eye(i),neigs,'SM',opts);
%         [d,v] = trideig(tri_d,tri_od(2:i));
        [dd,perm] = sort(diag(d));
% Order first neigs eigenvalues:
        eigmu = dd;
% Order first neigs eigenvectors:
        for j = 1:neigs;
            eigvecmu(1:i,j) = v(1:i,perm(j));
        end;
% Compute error bound errbnd for Ritz values relative to nearest exact
% (truth) eigenvalue (see Demmel)
        for j = 1:neigs;
            errbnd(j) = betai*abs(eigvecmu(i,j));
            eiglbmu(j) = eigmu(j)  - errbnd(j);
            ratio(j) = errbnd(j)/(tol*eigmu(j)/(1 + tol));         
        end;
% Get the true eigvector
        eigvec = q(:,1:size(eigvecmu,1))*eigvecmu;
% Terminate if ratio less than unity for all eigenvalues:
        if max(ratio) <= 1;
            niter = i;
% Confirm Lanczos orthogonality relationships as check on code:
            if ichflag == 1;
                tri
                tricons = q'*(bch_t\(amat'*(bch\(bch_t\(amat*(bch\q))))))
                idcons = q'*q
             end;
             return;
         end;
    end;
end;