function varargout=RB_parabolic(varargin)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
global pr_ONLINE pr_DEF du_ONLINE du_DEF cr_probname;

if nargin<2
    error('Warning: Too few input arguments.');
end
if (nargout > 8)
   error('Warning: Too many output arguments.')
end
if strcmp(cr_probname,varargin{1})~=1
    clear global pr_ONLINE pr_DEF du_ONLINE du_DEF cr_probname;
    global pr_ONLINE pr_DEF du_ONLINE du_DEF cr_probname;
end

% Handling input arguments
[pr_ONLINE,pr_DEF,du_ONLINE,du_DEF,probname,outputname,mu,N_pr,N_du,...
          contrl_pr,contrl_du,Li,J,ct_du0,options] = checkInputs(varargin{:});
cr_probname=probname;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
InsideOutsideD=str2func(strcat(probname,'_','InsideOutsideD'));
Get_Theta_q=str2func(strcat(probname,'_','Get_Theta_q'));
output_Get_Theta_q=str2func(strcat(probname,'_',outputname,'_','Get_Theta_q'));

% start timing pre-processing
cputms = zeros(1,1);
t0 = cputime; 
dt=pr_ONLINE.RB.timestep.dt;
nt=pr_ONLINE.RB.timestep.nt;
suD=feval(strcat(probname,'_',outputname,'_','addout'),mu);
if contrl_du==1
    ct_du=ones(1,nt);
elseif contrl_du==2 
    ct_du=zeros(1,nt);
    ct_du(1)=1;    
else
    ct_du=contrl_du;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% REDUCED BASIS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if du_DEF.dirfluxflag==1 % flux output
    if J==0
        error('Point-wise (in time) output is not available for dLIFT');
    else    
        for i=1:nt-2 
            contrl_du2(i)=(ct_du(i+1)-ct_du(i))/dt;
        end
        contrl_du2(nt-1)=(0.5*ct_du(nt)-ct_du(nt-1))/dt;
        contrl_du2(nt)=(0-ct_du(nt))/dt;
        contrl_du2=-contrl_du2;
        
%         contrl_du3(1)=(ct_du(1)-ct_du0)/dt;
%         for i=1:nt-1 
%             contrl_du3(i+1)=(ct_du(i+1)-ct_du(i))/dt;
%         end
%         contrl_du3=-contrl_du3;
%         contrl_du3(nt)=contrl_du3(nt)+2*ct_du(nt)/dt;
    end
    % First dual problem due to stiffness matrix
    [sN1,sN0,DN_s1,DN_pr,DN_du,aN,bN]=online_parabolic(mu,N_pr,N_du,pr_DEF,du_DEF,pr_ONLINE,...
     du_ONLINE,InsideOutsideD,Get_Theta_q,output_Get_Theta_q,J,Li,contrl_pr,contrl_du,ct_du0);
    
    % Second dual problem due to mass matrix
    output_Get_Theta_LM_q=str2func(strcat(probname,'_',outputname,'_','Get_Theta_LM_q'));
    N_du=min(N_du,du_ONLINE.du2_ONLINE.space.Nmax);
    [sN2,sN02,DN_s2]=online_parabolic(mu,N_pr,N_du,pr_DEF,du_DEF.du2_PROBDEF,pr_ONLINE,...
        du_ONLINE.du2_ONLINE,InsideOutsideD,Get_Theta_q,output_Get_Theta_LM_q,J,Li,...
        contrl_pr,contrl_du2,-2*contrl_du(1)/dt);    
    sN=sN1+sN2;
    DN_s=DN_s1+DN_s2;    
    
    %[sN1 sN0 sN2 sN02]
%     [sN3,sN03,DN_s3]=online_parabolic(mu,N_pr,N_du,pr_DEF,du_DEF.du2_PROBDEF,pr_ONLINE,...
%         du_ONLINE.du2_ONLINE,InsideOutsideD,Get_Theta_q,output_Get_Theta_LM_q,J,Li,...
%         contrl_pr,contrl_du3,contrl_du3(1)-2*ct_du0/dt);    
%     [sN2 sN3]
else
    [sN,sN0,DN_s,DN_pr,DN_du,aN,bN,res]=online_parabolic(mu,N_pr,N_du,pr_DEF,du_DEF,pr_ONLINE,...
     du_ONLINE,InsideOutsideD,Get_Theta_q,output_Get_Theta_q,J,Li,contrl_pr,contrl_du,ct_du0);    
end

if contrl_pr==1  
    ct_pr=ones(1,nt);
elseif contrl_pr==2 
    ct_pr=zeros(1,nt);
    ct_pr(1)=1;    
else
    ct_pr=contrl_pr;
end
suD=suD.*ct_pr;

% Add output due to nonhomogenous dirichlet condition
if J==0 % point-wise output
    sN = sN + suD;   
    sN0 = sN0 + suD;   
else
    sct=0.5*ct_du0;
    for i=1:length(Li)
        sct=sct+ct_du(Li(i))*suD(Li(i));
        for j=1:J-1
            sct=sct+ct_du(Li(i)+j)*suD(Li(i)+j);
        end
        if J>0
            sct=sct+0.5*ct_du(Li(i)+J)*suD(Li(i)+J);
        end        
    end    
    suD=dt*sct;
    sN0=sN0+suD;
    sN=sN+suD;
end

varargout{1}=sN;
varargout{2}=DN_s;
varargout{3}=DN_pr;
varargout{4}=DN_du;
varargout{5}=aN;
varargout{6}=bN;
varargout{7}=sN0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% HANDLING OPTIONS %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
% options = 0: online (default)
% options = 1: printout
% options = 2: RB and FE
% options = 3: testing

if options==1
    disp('------ RB output    | Output error bound ----');
    disp([sN' DN_s']);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% CHECKING RB WITH FE %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if du_DEF.dirfluxflag~=1
    if options>=2
        s1=strcat(probname,'_','pr_OFFLINE'); load(s1);
        eval(['pr_OFFLINE=' s1 ';']);         clear(s1); 
        s2=strcat(probname,'_',outputname,'_','du_OFFLINE'); load(s2);
        eval(['du_OFFLINE=' s2 ';']);                        clear(s2); 
    end
    if options==2
        [s]=check_parabolic_online(mu,N_pr,N_du,pr_DEF,du_DEF,pr_ONLINE,pr_OFFLINE,...
            du_OFFLINE,InsideOutsideD,Get_Theta_q,output_Get_Theta_q,Li,contrl_pr,contrl_du,J,ct_du0);    
        s=s+suD;
        disp('------ RB output    |   Output error   | Output error bound ----');
        disp([sN' (s-sN)' DN_s']);
        varargout{3}=abs(s-sN);
    end
    if options==3
        [s,sN02,sN2,DN_s2,normR2,normRdu2,aN2,bN2,res2]=check_parabolic_online(mu,N_pr,N_du,pr_DEF,du_DEF,...
                      pr_ONLINE,pr_OFFLINE,du_OFFLINE,InsideOutsideD,Get_Theta_q,output_Get_Theta_q,...
                      Li,contrl_pr,contrl_du,J,ct_du0);
        s=s+suD;
        sN2=sN2+suD;
        sN02=sN02+suD;
        disp('------ RB output    |   Output error   | Output error bound ----');
        disp([sN' (s-sN0)' (s-sN)' DN_s']);
        disp('Testing: output and output bound');
        disp([max(max(abs(sN0-sN02))) max(max(abs(sN-sN2))) max(max(abs(DN_s-DN_s2)))])
        %disp([max(max(abs(aN-aN2))) max(max(abs(bN-bN2))) max(max(abs(res-res2)))])
        if J~=0
            disp([max(max(abs(aN-aN2))) max(max(abs(bN-bN2))) max(max(abs(res-res2)))])
        else
            disp([max(max(abs(aN-aN2))) max(max(abs(bN-0.5*dt*bN2))) max(max(abs(res-0.5*dt*res2)))])
        end
        %[DN_pr' DN_du' normR2' normRdu2']
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% HANDLING INPUT ARGUMENTS %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
function [pr_ONLINE,pr_DEF,du_ONLINE,du_DEF,probname,outputname,mu,N_pr,N_du,...
          contrl_pr,contrl_du,Li,J,ct_du0,options] = checkInputs(varargin)

global pr_ONLINE pr_DEF du_ONLINE du_DEF;

probname=varargin{1};
outputname=varargin{2};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% ADD PATHS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath('../rbMIT_Library')
addpath('../rbMIT_Library/parabolic')
addpath('../rbMIT_Library/symbolic')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% LOAD ONLINE DATA %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(pr_ONLINE) | isempty(pr_DEF)
    s1=strcat(probname,'_','pr_ONLINE');
    s2=strcat(probname,'_','pr_PROBDEF');
    load(s1); load(s2);
    eval(['pr_ONLINE=' s1 ';']);
    eval(['pr_DEF=' s2 ';']);
    clear(s1); clear(s2); 
end

if isempty(du_ONLINE) | isempty(du_DEF)
    d0=fileparts([pwd,filesep]);
    addpath([d0,strcat('/',outputname)]);
    s1=strcat(probname,'_',outputname,'_','du_ONLINE');
    s2=strcat(probname,'_',outputname,'_','du_PROBDEF');
    load(s1); load(s2); 
    eval(['du_ONLINE=' s1 ';']);
    eval(['du_DEF=' s2 ';']);
    clear(s1); clear(s2); 
end

nt=pr_ONLINE.RB.timestep.nt;
dt=pr_ONLINE.RB.timestep.dt;
t=dt:dt:nt*dt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% GET INPUT ARGUMENTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (nargin<3)
    mu=pr_DEF.mu_min+(pr_DEF.mu_max-pr_DEF.mu_min).*rand(size(pr_DEF.mu_max));
else
    mu=varargin{3};
end
if isempty(mu)
    mu=pr_DEF.mu_min+(pr_DEF.mu_max-pr_DEF.mu_min).*rand(size(pr_DEF.mu_max));
end
if nargin<4
    N_pr=pr_ONLINE.space.Nmax; % default: N=N_max
else
    N_pr=varargin{4};
end
if isempty(N_pr)
    N_pr=pr_ONLINE.space.Nmax;
end
if nargin<5
    N_du=du_ONLINE.space.Nmax; % default: N=N_max     
else
    N_du=varargin{5};
end
if isempty(N_du)
    N_du=du_ONLINE.space.Nmax;
end
if nargin<6    
    contrl_pr=1; % default: unity constant control
else
    gt=varargin{6};
    if isempty(gt)
        contrl_pr=1; 
    else
        if ischar(gt)
            contrl_pr=eval(gt).*ones(1,length(t));
        else
            contrl_pr=gt;
        end
    end    
end
contrl_du=1; % default: unity constant control
J=nt-1;      % integral output
Li=1;
ct_du0=1;
if nargin>=7
    ht=varargin{7};
    if ht==1 % pointwise output
        %if pr_DEF.oareaload==0
        %    error('Warning: pointwise (in time) output is not available for oload and dLIFT');
        %end     
        J=0; 
        Li=1:nt;                
    elseif isempty(ht)==0
        if ischar(ht)
            contrl_du=eval(ht).*ones(1,length(t));
            t=0;
            ct_du0=eval(ht);
        else
            contrl_du=ht;
            ct_du0=ht(1);
        end
    end
end
if nargin<8
    options=0; % default: online (silence mode)
else
    options=varargin{8};
end
if isempty(options)
    options=0; 
end
