function varargout=RB_parabolic_test(varargin)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
global VIS pr_OFFLINE pr_ONLINE pr_DEF du_ONLINE du_DEF du_OFFLINE;

if nargin<1
    error('Warning: Too few input arguments.');
end
if (nargout > 3)
   error('Warning: Too many output arguments.')
end

% Handling input arguments
[VIS,pr_OFFLINE,pr_ONLINE,du_ONLINE,pr_DEF,du_DEF,probname,outputname,mu,N_pr,...
          contrl_pr,nk] = checkInputs(varargin{:});      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
InsideOutsideD=str2func(strcat(probname,'_','InsideOutsideD'));
Get_Theta_q=str2func(strcat(probname,'_','Get_Theta_q'));
output_Get_Theta_q=str2func(strcat(probname,'_',outputname,'_','Get_Theta_q'));

% start timing pre-processing
cputms = zeros(1,1);
t0 = cputime; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% REDUCED BASIS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[aN,normR]=online_parabolic_primal(mu, N_pr, pr_ONLINE, pr_DEF, ...
     InsideOutsideD, Get_Theta_q, nk, contrl_pr);    
nt=pr_ONLINE.RB.timestep.nt; 
dt=pr_ONLINE.RB.timestep.dt; 
nk_du=nt-nk+1;    
N_du=du_ONLINE.space.Nmax;
[bN,normRdu]=online_parabolic_dual(mu, N_du, pr_DEF, du_DEF, du_ONLINE,...
     InsideOutsideD, Get_Theta_q, output_Get_Theta_q, nk_du, contrl_pr, 0); 
nk_du=nt-1;

aN=aN(:,nk); 
uRB=pr_OFFLINE.space.Z(:,1:N_pr)*aN;
[alpha_LB, alphaUB, rat, ONLINE]=alphaLB_coer(mu, pr_DEF, pr_ONLINE, Get_Theta_q);
dt=pr_ONLINE.RB.timestep.dt;
delta_u=sqrt((dt/alpha_LB)*sum(abs(normR(1:nk))));            
sol_bound = delta_u;
disp(['Error bound in X norm is ', num2str(sol_bound)]);

pmu=geoplot(probname,mu);
B=VIS.B;  
p=VIS.p;
t=VIS.t;
UD=VIS.UD;
e=VIS.e;
uT=B*uRB+UD;
num_dim=size(uT,1)/size(pmu,2);
startp=1;
endp=size(pmu,2);

psiRB=du_OFFLINE.space.Z(:,1:N_du)*bN;
psiRB=B*psiRB(:,nk_du)+UD;

varargout{1}=uT;
varargout{2}=sol_bound;

figure(1);clf;
% PLOTSOLUTION(pmu',t(1:3,:)',uT);
% axis equal; axis tight; axis on; colorbar;
pdesurf(pmu,t,uT-UD)
title(probname)
colormap(jet)
colorbar('vert')
lighting('phong')
axis('equal'); 
axis tight;

figure(2);clf;
PLOTSOLUTION(pmu',t(1:3,:)',psiRB-UD);
axis equal; axis tight; axis on; colorbar;
% pdesurf(pmu,t,psiRB-UD)
% title(probname)
% colormap(jet)
% colorbar('vert')
% lighting('phong')
% axis('equal'); 
% axis tight;

if 1
    [u,A,M,F,psi]=get_parabolic_primal_solution(mu, pr_DEF, pr_OFFLINE, InsideOutsideD, ...
                   Get_Theta_q,contrl_pr,output_Get_Theta_q,du_DEF,du_OFFLINE,contrl_pr,0);         
    uE=B*u(:,nk)+UD;
    figure(3);clf;
    PLOTSOLUTION(pmu',t(1:3,:)',uE-uT);
    axis equal; axis tight; axis on; colorbar;                    
%     pdesurf(pmu,t,psiRB)
%     title(probname)
%     colormap(jet)
%     colorbar('vert')
%     lighting('phong')
%     axis('equal'); 
%     axis tight;
    
    uE=B*psi(:,nk_du);
    figure(4);clf;
    PLOTSOLUTION(pmu',t(1:3,:)',uE);
    axis equal; axis tight; axis on; colorbar;                    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% HANDLING INPUT ARGUMENTS %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
function [VIS,pr_OFFLINE,pr_ONLINE,du_ONLINE,pr_DEF,du_DEF,probname,outputname,mu,N_pr,...
          contrl_pr,nk] = checkInputs(varargin)

global VIS pr_OFFLINE pr_ONLINE pr_DEF du_OFFLINE du_ONLINE du_DEF

probname=varargin{1};
outputname=varargin{2};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% ADD PATHS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath('../rbMIT_Library')
addpath('../rbMIT_Library/parabolic')
addpath('../rbMIT_Library/symbolic')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% LOAD ONLINE DATA %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(VIS) | isempty(pr_OFFLINE) | isempty(pr_ONLINE) | isempty(pr_DEF)
    d0=fileparts([pwd,filesep]);
    addpath([d0,strcat('/',outputname)]);
    s1=strcat(probname,'_','pr_ONLINE');
    s2=strcat(probname,'_','pr_PROBDEF');
    s3=strcat(probname,'_','pr_OFFLINE');
    s4=strcat(probname,'_','VIS');
    s5=strcat(probname,'_',outputname,'_','du_OFFLINE');
    s6=strcat(probname,'_',outputname,'_','du_PROBDEF');
    s7=strcat(probname,'_',outputname,'_','du_ONLINE');
    load(s1); load(s2); load(s3); load(s4); load(s5); load(s6); load(s7);
    eval(['pr_ONLINE=' s1 ';']);
    eval(['pr_DEF=' s2 ';']);
    eval(['pr_OFFLINE=' s3 ';']);    
    eval(['VIS=' s4 ';']);
    eval(['du_OFFLINE=' s5 ';']);
    eval(['du_DEF=' s6 ';']);
    eval(['du_ONLINE=' s7 ';']);
    clear(s1); clear(s2); clear(s3); clear(s4); clear(s5); clear(s6); clear(s7);
    %rmpath([d0,strcat('/',outputname)]);
end

nt=pr_ONLINE.RB.timestep.nt;
dt=pr_ONLINE.RB.timestep.dt;
t=dt:dt:nt*dt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% GET INPUT ARGUMENTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (nargin<3)
    mu=pr_DEF.mu_min+(pr_DEF.mu_max-pr_DEF.mu_min).*rand(size(pr_DEF.mu_max));
else
    mu=varargin{3};
end
if isempty(mu)
    mu=pr_DEF.mu_min+(pr_DEF.mu_max-pr_DEF.mu_min).*rand(size(pr_DEF.mu_max));
end
if nargin<4
    N_pr=pr_ONLINE.space.Nmax; % default: N=N_max
else
    N_pr=varargin{4};
end
if isempty(N_pr)
    N_pr=pr_ONLINE.space.Nmax;
end
if nargin<5    
    contrl_pr=1; % default: unity constant control
else
    gt=varargin{5};
    if isempty(gt)
        contrl_pr=1; 
    else        
        contrl_pr=eval(gt).*ones(1,length(t));
    end    
end
if isempty(contrl_pr)
    contrl_pr=1; 
end
if nargin<6
    nk=pr_ONLINE.RB.timestep.nt; % default: final timestep
else
    nk=varargin{6};
end
if isempty(nk)
    nk=pr_ONLINE.RB.timestep.nt; 
end

function PLOTSOLUTION(p,t,u)
% Plot the temperature field

ss=[1 0 0; 0 1 0; 0 0 1];
tt=[1 2 3];
ns=size(ss,1);
nt=size(t,1);
dim=2;

pp=zeros(ns,dim,nt);
for comp=1:dim
  for trinode=1:dim+1
    dp=ss(:,trinode)*(p(t(:,trinode),comp))';        
    pp(:,comp,:)=pp(:,comp,:)+permute(dp,[1,3,2]);
  end
end

uu=zeros(ns,nt);
for trinode=1:dim+1    
    du=ss(:,trinode)*(u(t(:,trinode),1))';    
    uu=uu+du;
end

pvis=reshape(permute(pp,[1,3,2]),[ns*nt,2]);
tvis=kron(ones(nt,1),tt)+kron(ns*(0:nt-1)',0*tt+1);

cla,patch('vertices',pvis,'faces',tvis,'cdata',uu, ...
          'facecol','interp','edgec','none');

set(gcf,'rend','z');
drawnow



