function varargout=RB_parabolic_vis(varargin)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
global VIS pr_OFFLINE pr_ONLINE pr_DEF cr_probname;

if nargin<1
    error('Warning: Too few input arguments.');
end
if (nargout > 3)
   error('Warning: Too many output arguments.')
end
if strcmp(cr_probname,varargin{1})~=1
    clear global pr_ONLINE pr_DEF du_ONLINE du_DEF cr_probname;
    global pr_ONLINE pr_DEF du_ONLINE du_DEF cr_probname;
end
    
% Handling input arguments
[VIS,pr_OFFLINE,pr_ONLINE,pr_DEF,probname,mu,N_pr,...
          contrl_pr,nk,options] = checkInputs(varargin{:});      
cr_probname=probname;

if strcmp(probname,'tdepbs1d')
    sol_bound=vis_tdepbs1d(varargin{:});
    varargout{1}=sol_bound;
    return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
InsideOutsideD=str2func(strcat(probname,'_','InsideOutsideD'));
Get_Theta_q=str2func(strcat(probname,'_','Get_Theta_q'));

% start timing pre-processing
cputms = zeros(1,1);
t0 = cputime; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% REDUCED BASIS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[aN,normR]=online_parabolic_primal(mu, N_pr, pr_ONLINE, pr_DEF, ...
     InsideOutsideD, Get_Theta_q, nk, contrl_pr);    
[alpha_LB, alphaUB, rat, ONLINE]=alphaLB_coer(mu, pr_DEF, pr_ONLINE, Get_Theta_q);
dt=pr_ONLINE.RB.timestep.dt;
nt=pr_ONLINE.RB.timestep.nt;
delta_u=sqrt((dt/alpha_LB)*sum(abs(normR(1:nk))));            
sol_bound = delta_u;
disp(['Error bound in energy norm at timestep ' num2str(nk) ' is ' num2str(sol_bound)]);
varargout{1}=sol_bound;
%varargout{2}=uT;

if contrl_pr==1  
    ct=ones(1,nt);
elseif contrl_pr==2 
    ct=zeros(1,nt);
    ct(1)=1;    
else
    ct=contrl_pr;
end

pmu=geoplot(probname,mu);
B=VIS.B;  
p=VIS.p;
t=VIS.t;
UD=VIS.UD;
e=VIS.e;
    
if strcmp(options,'mv')==1
    uRB=pr_OFFLINE.space.Z(:,1:N_pr)*aN;
    %uT=B*uRB+repmat(UD,[1 nk]);            
    uT=B*uRB+kron(ct(1:nk),UD); 
    umax=max(max(uT));
    umin=min(min(uT));
    pp=pmnode(pmu',t(1:3,:)');
    %Mv=moviein(nk);
    figure(1);clf
    caxis([umin umax]);
    colorbar;    
    for j=1:nk
        plotsol(pp,t(1:3,:)',uT(:,j));        
        axis equal; axis tight; 
    end    
    return;
else
    uRB=pr_OFFLINE.space.Z(:,1:N_pr)*aN(:,nk);
    uT=B*uRB+ct(nk)*UD;    
    
    figure(1);clf;
    %pdemesh(pmu,e,t);
    geoplot(probname,mu);
    axis equal; axis tight; axis on;

    num_dim=size(uT,1)/size(pmu,2);
    startp=1;
    endp=size(pmu,2);
    for k=1:num_dim    
        figure(k+1);clf;
        pdesurf(pmu,t,uT(startp:endp))
        title(probname)
        colormap(jet)
        colorbar('vert')
        lighting('phong')
        startp=startp+size(pmu,2);
        endp=endp+size(pmu,2);
        axis('equal'); 
        axis tight;
    end
end

if 0
    [u]=get_parabolic_primal_solution(mu, pr_DEF, pr_OFFLINE, InsideOutsideD, ...
                        Get_Theta_q, contrl_pr);         
    uE=B*u(:,nk)+UD;
    figure(3);clf;
    PLOTSOLUTION(pmu',t(1:3,:)',uE-uT);
    axis equal; axis tight; axis on; colorbar;                    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% HANDLING INPUT ARGUMENTS %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
function [VIS,pr_OFFLINE,pr_ONLINE,pr_DEF,probname,mu,N_pr,...
          contrl_pr,nk,options] = checkInputs(varargin)

global VIS pr_OFFLINE pr_ONLINE pr_DEF

probname=varargin{1};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% ADD PATHS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%addpath('../rbMIT_Library')
%addpath('../rbMIT_Library/parabolic')
%addpath('../rbMIT_Library/symbolic')
d0=fileparts([pwd,filesep]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% LOAD ONLINE DATA %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(VIS) | isempty(pr_OFFLINE) | isempty(pr_ONLINE) | isempty(pr_DEF)
    s1=strcat(probname,'_','pr_ONLINE');
    s2=strcat(probname,'_','pr_PROBDEF');
    s3=strcat(probname,'_','pr_OFFLINE');
    s4=strcat(probname,'_','VIS');
    load(s1); load(s2); load(s3); load(s4);
    eval(['pr_ONLINE=' s1 ';']);
    eval(['pr_DEF=' s2 ';']);
    eval(['pr_OFFLINE=' s3 ';']);
    eval(['VIS=' s4 ';']);
    clear(s1); clear(s2); clear(s3); clear(s4);
end

nt=pr_ONLINE.RB.timestep.nt;
dt=pr_ONLINE.RB.timestep.dt;
t=dt:dt:nt*dt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% GET INPUT ARGUMENTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (nargin<2)
    mu=pr_DEF.mu_min+(pr_DEF.mu_max-pr_DEF.mu_min).*rand(size(pr_DEF.mu_max));
else
    mu=varargin{2};
end
if isempty(mu)
    mu=pr_DEF.mu_min+(pr_DEF.mu_max-pr_DEF.mu_min).*rand(size(pr_DEF.mu_max));
end
if nargin<3
    N_pr=pr_ONLINE.space.Nmax; % default: N=N_max
else
    N_pr=varargin{3};
end
if isempty(N_pr)
    N_pr=pr_ONLINE.space.Nmax;
end
if nargin<4    
    contrl_pr=1; % default: unity constant control
else
    gt=varargin{4};
    if isempty(gt)
        contrl_pr=1; 
    else        
        if ischar(gt)
            contrl_pr=eval(gt).*ones(1,length(t));
        else
            contrl_pr=gt;
        end
    end    
end
if isempty(contrl_pr)
    contrl_pr=1; 
end
if nargin<5
    nk=pr_ONLINE.RB.timestep.nt; % default: final timestep
else
    nk=varargin{5};
end
if isempty(nk)
    nk=pr_ONLINE.RB.timestep.nt; 
end
if nargin<6
    options=0;
else
    options=varargin{6};
end
if isempty(options)
    options=0; 
end


function pp=pmnode(p,t)
ss=[1 0 0; 0 1 0; 0 0 1];
tt=[1 2 3];
ns=size(ss,1);
nt=size(t,1);
dim=2;

pp=zeros(ns,dim,nt);
for comp=1:dim
  for trinode=1:dim+1
    dp=ss(:,trinode)*(p(t(:,trinode),comp))';        
    pp(:,comp,:)=pp(:,comp,:)+permute(dp,[1,3,2]);
  end
end

function plotsol(pp,t,u)
% Plot the temperature field
ss=[1 0 0; 0 1 0; 0 0 1];
tt=[1 2 3];
ns=size(ss,1);
nt=size(t,1);
dim=2;

uu=zeros(ns,nt);
for trinode=1:dim+1    
    du=ss(:,trinode)*(u(t(:,trinode),1))';    
    uu=uu+du;
end

pvis=reshape(permute(pp,[1,3,2]),[ns*nt,2]);
tvis=kron(ones(nt,1),tt)+kron(ns*(0:nt-1)',0*tt+1);

cla,patch('vertices',pvis,'faces',tvis,'cdata',uu, ...
          'facecol','interp','edgec','none');

set(gcf,'rend','z');
drawnow


