function [bN,normRdu]=check_parabolic_dual(mu,N,pr_DEF,du_DEF,pr_OFFLINE,du_OFFLINE, ...
    InsideOutsideD,Get_Theta_q,output_Get_Theta_q,contrl,J)

% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%  PROCESSING INPUTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%       
dt=pr_OFFLINE.FEM.timestep.dt;
nt=pr_OFFLINE.FEM.timestep.nt;
Xnorm = pr_OFFLINE.FEM.matrix.Xnorm;
HT=pr_OFFLINE.FEM.matrix.HT; 
if contrl==1      % constant control
    ct=ones(1,nt);
elseif contrl==2  % impulse control
    ct=zeros(1,nt);
    ct(nt)=1;    
else
    ct=contrl;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Qa=pr_DEF.Q_affine.a;
Qm=pr_DEF.Q_affine.m;
Qf=pr_DEF.Q_affine.f;
Ql=du_DEF.Q_affine.l;
sample=feval(Get_Theta_q, pr_DEF, mu);
theta_a = sample(1:Qa);
theta_m = sample(Qa+Qf+1:(Qa+Qf+Qm));
sample=feval(output_Get_Theta_q, du_DEF, mu);
theta_l = sample(1:Ql);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% FEM MATRICES AND VECTORS %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
A = theta_a(1)*pr_OFFLINE.FEM.matrix.Aq{1};
for q=2:Qa
    A = A + theta_a(q)*pr_OFFLINE.FEM.matrix.Aq{q};
end
M = theta_m(1)*pr_OFFLINE.FEM.matrix.Mq{1};
for q=2:Qm
    M = M + theta_m(q)*pr_OFFLINE.FEM.matrix.Mq{q};
end
L = theta_l(1)*du_OFFLINE.FEM.matrix.Lq{1};
for q=2:Ql
    L = L + theta_l(q)*du_OFFLINE.FEM.matrix.Lq{q};
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% DUAL PROBLEM %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AN_du=du_OFFLINE.space.Z(:,1:N)'*A*du_OFFLINE.space.Z(:,1:N);
MN_du=du_OFFLINE.space.Z(:,1:N)'*M*du_OFFLINE.space.Z(:,1:N);
LN_du=du_OFFLINE.space.Z(:,1:N)'*L;

KN_du=MN_du+(dt)*AN_du;
bN(:,nt)=KN_du\(0.5*dt*ct(nt)*LN_du);
if J==0
    for i=nt-1:-1:1
        bN(:,i)=KN_du\(MN_du*bN(:,i+1));   
    end
elseif J==1
    i=nt-1;
    bN(:,i)=KN_du\(0.5*dt*ct(i)*LN_du+MN_du*bN(:,i+1));
    for i=nt-2:-1:1
        bN(:,i)=KN_du\(MN_du*bN(:,i+1));   
    end
elseif J>=2
    for i=nt-1:-1:nt-J+1
        bN(:,i)=KN_du\(dt*ct(i)*LN_du+MN_du*bN(:,i+1));
    end
    i=nt-J;
    bN(:,i)=KN_du\(0.5*dt*ct(i)*LN_du+MN_du*bN(:,i+1));
    for i=nt-J-1:-1:1
        bN(:,i)=KN_du\(MN_du*bN(:,i+1));   
    end
end
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% ENERGY BOUNDS %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
psiN=du_OFFLINE.space.Z(:,1:N)*bN(:,nt);
R=0.5*ct(nt)*L-A*psiN-(1/dt)*M*psiN;
ehat=Xnorm\(HT\R);
normRdu(nt)=ehat'*R;
if J==0
    for i=nt-1:-1:1
        psiN1=du_OFFLINE.space.Z(:,1:N)*bN(:,i+1);
        psiN=du_OFFLINE.space.Z(:,1:N)*bN(:,i);
        R=-A*psiN-(1/dt)*(M*(psiN-psiN1));   
        ehat=Xnorm\(HT\R);
        normRdu(i)=ehat'*R;
    end
elseif J==1
    i=nt-1;
    psiN1=du_OFFLINE.space.Z(:,1:N)*bN(:,i+1);
    psiN=du_OFFLINE.space.Z(:,1:N)*bN(:,i);
    R=0.5*ct(i)*L-A*psiN-(1/dt)*(M*(psiN-psiN1));   
    ehat=Xnorm\(HT\R);
    normRdu(i)=ehat'*R;
    for i=nt-2:-1:1
        psiN1=du_OFFLINE.space.Z(:,1:N)*bN(:,i+1);
        psiN=du_OFFLINE.space.Z(:,1:N)*bN(:,i);
        R=-A*psiN-(1/dt)*(M*(psiN-psiN1));   
        ehat=Xnorm\(HT\R);
        normRdu(i)=ehat'*R;
    end
elseif J>=2
    for i=nt-1:-1:nt-J+1
        psiN1=du_OFFLINE.space.Z(:,1:N)*bN(:,i+1);
        psiN=du_OFFLINE.space.Z(:,1:N)*bN(:,i);
        R=ct(i)*L-A*psiN-(1/dt)*(M*(psiN-psiN1));   
        ehat=Xnorm\(HT\R);
        normRdu(i)=ehat'*R;        
    end
    i=nt-J;
    psiN1=du_OFFLINE.space.Z(:,1:N)*bN(:,i+1);
    psiN=du_OFFLINE.space.Z(:,1:N)*bN(:,i);
    R=0.5*ct(i)*L-A*psiN-(1/dt)*(M*(psiN-psiN1));   
    ehat=Xnorm\(HT\R);
    normRdu(i)=ehat'*R;
    for i=nt-J-1:-1:1
        psiN1=du_OFFLINE.space.Z(:,1:N)*bN(:,i+1);
        psiN=du_OFFLINE.space.Z(:,1:N)*bN(:,i);
        R=-A*psiN-(1/dt)*(M*(psiN-psiN1));   
        ehat=Xnorm\(HT\R);
        normRdu(i)=ehat'*R;
    end
end
