function [s,sN0,sN,DN_s,normR,normRdu,aN,bN,res]=check_parabolic_online(mu,N_pr,N_du,pr_DEF,du_DEF,...
    pr_ONLINE,pr_OFFLINE,du_OFFLINE,InsideOutsideD,Get_Theta_q,output_Get_Theta_q,...
    Li,contrl_pr,contrl_du,J,ct_du0)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%  PROCESSING INPUTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%       
dt=pr_OFFLINE.FEM.timestep.dt;
nt=pr_OFFLINE.FEM.timestep.nt;
Xnorm = pr_OFFLINE.FEM.matrix.Xnorm;
HT=pr_OFFLINE.FEM.matrix.HT; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Qa=pr_DEF.Q_affine.a;
Qm=pr_DEF.Q_affine.m;
Qf=pr_DEF.Q_affine.f;
Qu=pr_DEF.Q_affine.u;
Ql=du_DEF.Q_affine.l;
sample=feval(Get_Theta_q, pr_DEF, mu);
theta_a = sample(1:Qa);
theta_f = sample(Qa+1:(Qa+Qf));
theta_m = sample(Qa+Qf+1:(Qa+Qf+Qm));
theta_u = sample((1+Qa+Qf+Qm):(Qa+Qf+Qm+Qu));
sample=feval(output_Get_Theta_q, du_DEF, mu);
theta_l = sample(1:Ql);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% CONTROL INPUTS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if contrl_pr==1      % constant control
    ct=ones(1,nt);
elseif contrl_pr==2  % impulse control
    ct=zeros(1,nt);
    ct(1)=1;    
else
    ct=contrl_pr;
end
if contrl_du==1      % constant control
    ct_du=ones(1,nt);
elseif contrl_du==2  % impulse control
    ct_du=zeros(1,nt);
    ct_du(nt)=1;    
else
    ct_du=contrl_du;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% FEM MATRICES AND VECTORS %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
A = theta_a(1)*pr_OFFLINE.FEM.matrix.Aq{1};
for q=2:Qa
    A = A + theta_a(q)*pr_OFFLINE.FEM.matrix.Aq{q};
end
M = theta_m(1)*pr_OFFLINE.FEM.matrix.Mq{1};
for q=2:Qm
    M = M + theta_m(q)*pr_OFFLINE.FEM.matrix.Mq{q};
end
F = theta_f(1)*pr_OFFLINE.FEM.matrix.Fq{1};
for q=2:Qf
    F = F + theta_f(q)*pr_OFFLINE.FEM.matrix.Fq{q};
end
L = theta_l(1)*du_OFFLINE.FEM.matrix.Lq{1};
for q=2:Ql
    L = L + theta_l(q)*du_OFFLINE.FEM.matrix.Lq{q};
end
u0 = theta_u(1)*pr_OFFLINE.FEM.matrix.Uq{1};
for q=2:Qu
    u0 = u0 + theta_u(q)*pr_OFFLINE.FEM.matrix.Uq{q};
end

% FE solution
K=M+(dt)*A;
for i=1:nt    
    if i==1
        u(:,i)=K\(M*(u0)+(ct(i)*dt)*F);
    else
        u(:,i)=K\(M*u(:,i-1)+(ct(i)*dt)*F); 
    end    
end
% FE outputs
if J==0
    for i=1:length(Li)
        s(i)=ct_du(Li(i))*(L'*u(:,Li(i)));    
    end
else
    suD0=0.5*dt*ct_du0*(L'*(u0+pr_OFFLINE.FEM.matrix.UD));
    for i=1:length(Li)        
        s(i)=ct_du(Li(i))*(L'*u(:,Li(i)));
        for j=1:J-1
            s(i)=s(i)+ct_du(Li(i)+j)*(L'*u(:,Li(i)+j));
        end
        if J>0
            s(i)=s(i)+0.5*ct_du(Li(i)+J)*(L'*u(:,Li(i)+J));
        end
    end    
    s=dt*s+suD0;
end

if nargout==1
    return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% PRIMAL PROBLEM %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AN=pr_OFFLINE.space.Z(:,1:N_pr)'*A*pr_OFFLINE.space.Z(:,1:N_pr);
MN=pr_OFFLINE.space.Z(:,1:N_pr)'*M*pr_OFFLINE.space.Z(:,1:N_pr);
FN=pr_OFFLINE.space.Z(:,1:N_pr)'*F;
LN=pr_OFFLINE.space.Z(:,1:N_pr)'*L;
KN=MN+(dt)*AN;
aN0=pr_OFFLINE.space.Z(:,1:N_pr)'*M*u0;
for i=1:nt
    if i==1
        aN(:,i)=KN\(aN0+(ct(i)*dt)*FN);
        uN(:,i)=pr_OFFLINE.space.Z(:,1:N_pr)*aN(:,i);
        Rpr(:,i)=ct(i)*F-A*uN(:,i)-(1/dt)*(M*(uN(:,i)-u0));
    else
        aN(:,i)=KN\(MN*aN(:,i-1)+(ct(i)*dt)*FN); 
        uN(:,i)=pr_OFFLINE.space.Z(:,1:N_pr)*aN(:,i);
        Rpr(:,i)=ct(i)*F-A*uN(:,i)-(1/dt)*(M*(uN(:,i)-uN(:,i-1)));
    end     
    ehat=Xnorm\(HT\Rpr(:,i));    
    normR(i)=ehat'*Rpr(:,i);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% DUAL PROBLEM %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
A_du=A';M_du=M';
AN_du=du_OFFLINE.space.Z(:,1:N_du)'*A_du*du_OFFLINE.space.Z(:,1:N_du);
MN_du=du_OFFLINE.space.Z(:,1:N_du)'*M_du*du_OFFLINE.space.Z(:,1:N_du);
LN_du=du_OFFLINE.space.Z(:,1:N_du)'*L;
KN_du=MN_du+(dt)*AN_du;

if J==0 % pointwise
    bN(:,nt)=KN_du\(LN_du);
    for i=nt-1:-1:1
        bN(:,i)=KN_du\(MN_du*bN(:,i+1));   
    end
elseif J==1
    bN(:,nt)=KN_du\(0.5*dt*ct_du(nt)*LN_du);
    i=nt-1;
    bN(:,i)=KN_du\(0.5*dt*ct_du(i)*LN_du+MN_du*bN(:,i+1));
    for i=nt-2:-1:1
        bN(:,i)=KN_du\(MN_du*bN(:,i+1));   
    end
elseif J>=2
    bN(:,nt)=KN_du\(0.5*dt*ct_du(nt)*LN_du);
    for i=nt-1:-1:nt-J+1
        bN(:,i)=KN_du\(dt*ct_du(i)*LN_du+MN_du*bN(:,i+1));
    end
    i=nt-J;
    bN(:,i)=KN_du\(dt*ct_du(i)*LN_du+MN_du*bN(:,i+1));
    for i=nt-J-1:-1:1
        bN(:,i)=KN_du\(MN_du*bN(:,i+1));   
    end
end

if J==0 %pointwise
    psiN(:,nt)=du_OFFLINE.space.Z(:,1:N_du)*bN(:,nt);
    R=(1/dt)*L-A_du*psiN(:,nt)-(1/dt)*M_du*psiN(:,nt);
    ehat=Xnorm\(HT\R);
    normRdu(nt)=ehat'*R;
    for i=nt-1:-1:1
        psiN(:,i)=du_OFFLINE.space.Z(:,1:N_du)*bN(:,i);
        R=-A_du*psiN(:,i)-(1/dt)*(M_du*(psiN(:,i)-psiN(:,i+1)));   
        ehat=Xnorm\(HT\R);
        normRdu(i)=ehat'*R;
    end
elseif J==1
    psiN(:,nt)=du_OFFLINE.space.Z(:,1:N_du)*bN(:,nt);
    R=0.5*ct_du(nt)*L-A_du*psiN(:,nt)-(1/dt)*M_du*psiN(:,nt);
    ehat=Xnorm\(HT\R);
    normRdu(nt)=ehat'*R;
    i=nt-1;
    psiN(:,i)=du_OFFLINE.space.Z(:,1:N_du)*bN(:,i);
    R=0.5*ct_du(i)*L-A_du*psiN(:,i)-(1/dt)*(M_du*(psiN(:,i)-psiN(:,i+1)));   
    ehat=Xnorm\(HT\R);
    normRdu(i)=ehat'*R;
    for i=nt-2:-1:1
        psiN(:,i)=du_OFFLINE.space.Z(:,1:N_du)*bN(:,i);
        R=-A_du*psiN(:,i)-(1/dt)*(M_du*(psiN(:,i)-psiN(:,i+1)));   
        ehat=Xnorm\(HT\R);
        normRdu(i)=ehat'*R;
    end
elseif J>=2
    psiN(:,nt)=du_OFFLINE.space.Z(:,1:N_du)*bN(:,nt);
    R=0.5*ct_du(nt)*L-A_du*psiN(:,nt)-(1/dt)*M_du*psiN(:,nt);
    ehat=Xnorm\(HT\R);
    normRdu(nt)=ehat'*R;
    for i=nt-1:-1:nt-J+1
        psiN(:,i)=du_OFFLINE.space.Z(:,1:N_du)*bN(:,i);
        R=ct_du(i)*L-A_du*psiN(:,i)-(1/dt)*(M_du*(psiN(:,i)-psiN(:,i+1)));   
        ehat=Xnorm\(HT\R);
        normRdu(i)=ehat'*R;        
    end
    i=nt-J;
    psiN(:,i)=du_OFFLINE.space.Z(:,1:N_du)*bN(:,i);
    R=ct_du(i)*L-A_du*psiN(:,i)-(1/dt)*(M_du*(psiN(:,i)-psiN(:,i+1)));   
    ehat=Xnorm\(HT\R);
    normRdu(i)=ehat'*R;
    for i=nt-J-1:-1:1
        psiN(:,i)=du_OFFLINE.space.Z(:,1:N_du)*bN(:,i);
        R=-A_du*psiN(:,i)-(1/dt)*(M_du*(psiN(:,i)-psiN(:,i+1)));   
        ehat=Xnorm\(HT\R);
        normRdu(i)=ehat'*R;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% RESIDUAL CORRECTION %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
res(1)=Rpr(:,1)'*psiN(:,nt);
for k=2:nt
    res(k)=0;
    for kp=1:k
        res(k)=res(k)+Rpr(:,kp)'*psiN(:,nt-k+kp);
    end
end

% energy bounds
[alpha_LB, alphaUB, rat]=alphaLB_coer(mu, pr_DEF, pr_ONLINE, Get_Theta_q);
for k=1:nt
    DN_pr(k)=sqrt((dt/alpha_LB)*sum(abs(normR(1:k))));            
end
for k=nt:-1:1
    DN_du(k)=sqrt((dt/alpha_LB)*sum(abs(normRdu(end:-1:k))));
end
DeltaN_s=(DN_pr.*DN_du(end:-1:1));

% Output and output bounds
if J==0
    for i=1:length(Li)
        %s(i)=ct_du(Li(i))*(L'*u(:,Li(i)));
        sN0(i)=ct_du(Li(i))*(L'*uN(:,Li(i)));
        saN=ct_du(Li(i))*aN(:,Li(i));            
        sN02(i)=LN'*saN;        
        sN(i)=sN02(i)+dt*res(Li(i));
        DN_s(i)=DeltaN_s(Li(i));
    end
else
    for i=1:length(Li)
        sN0(i)=ct_du(Li(i))*(L'*uN(:,Li(i)));
        saN=ct_du(Li(i))*aN(:,Li(i));    
        for j=1:J-1
            %s(i)=s(i)+ct_du(Li(i)+j)*(L'*u(:,Li(i)+j));
            sN0(i)=sN0(i)+ct_du(Li(i)+j)*(L'*uN(:,Li(i)+j));
            saN=saN+ct_du(Li(i)+j)*aN(:,Li(i)+j);
        end
        if J>0
            %s(i)=s(i)+0.5*ct_du(Li(i)+J)*(L'*u(:,Li(i)+J));
            sN0(i)=sN0(i)+0.5*ct_du(Li(i)+J)*(L'*uN(:,Li(i)+J));
            saN=saN+0.5*ct_du(Li(i)+J)*aN(:,Li(i)+J);
        end
        sN02(i)=LN'*saN;        
        sN(i)=sN02(i)+res(Li(i)+J);
        DN_s(i)=DeltaN_s(Li(i)+J);
    end
    sN=sN*dt;
    %s=s*dt;
    sN0=sN02*dt;
end


%s0=0.5*dt*ct_du0*(L'*u0);
if J~=0
    sN0=sN0+suD0;
    sN=sN+suD0;
end
