function [aN,normR]=check_parabolic_primal(mu, N, pr_DEF, pr_OFFLINE, ...
                                InsideOutsideD, Get_Theta_q, contrl)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%  PROCESSING INPUTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                   
dt=pr_OFFLINE.FEM.timestep.dt;
nt=pr_OFFLINE.FEM.timestep.nt;
Xnorm=pr_OFFLINE.FEM.matrix.Xnorm;
HT=pr_OFFLINE.FEM.matrix.HT; 
if contrl==1      % constant control
    ct=ones(1,nt);
elseif contrl==2  % impulse control
    ct=zeros(1,nt);
    ct(1)=1;    
elseif contrl==3  % user's control  
    ct=pr_OFFLINE.controlinput.ct;
else
    error('Unknown control input option');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% FEM MATRICES AND VECTORS %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[u,A,M,F,u0]=get_parabolic_primal_solution(mu, pr_DEF, pr_OFFLINE, InsideOutsideD, ...
                        Get_Theta_q, contrl);
                    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% PRIMAL PROBLEM %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                    
AN=pr_OFFLINE.space.Z(:,1:N)'*A*pr_OFFLINE.space.Z(:,1:N);
MN=pr_OFFLINE.space.Z(:,1:N)'*M*pr_OFFLINE.space.Z(:,1:N);
FN=pr_OFFLINE.space.Z(:,1:N)'*F;
uN0=pr_OFFLINE.space.Z(:,1:N)'*M*u0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% ENERGY BOUNDS %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
KN=MN+(dt)*AN;
for i=1:nt
    if i==1
        aN(:,i)=KN\(uN0+(ct(i)*dt)*FN);
        uN(:,i)=pr_OFFLINE.space.Z(:,1:N)*aN(:,i);
        Rpr=ct(i)*F-A*uN(:,i)-(1/dt)*(M*(uN(:,i)-u0));
    else
        aN(:,i)=KN\(MN*aN(:,i-1)+(ct(i)*dt)*FN); 
        uN(:,i)=pr_OFFLINE.space.Z(:,1:N)*aN(:,i);
        Rpr=ct(i)*F-A*uN(:,i)-(1/dt)*(M*(uN(:,i)-uN(:,i-1)));
    end     
    ehat=Xnorm \(HT\Rpr);    
    normR(i)=ehat'*Rpr;
end




