function [u,A,M,F,psi]=get_parabolic_primal_solution(mu, DEF, OFFLINE, InsideOutsideD, ...
                  Get_Theta_q, contrl, output_Get_Theta_q, du_DEF, du_OFFLINE, contrl_du, J)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%  PROCESSING INPUTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                   
Qa=DEF.Q_affine.a;
Qm=DEF.Q_affine.m;
Qf=DEF.Q_affine.f;
Qu=DEF.Q_affine.u;
dt=OFFLINE.FEM.timestep.dt;
nt=OFFLINE.FEM.timestep.nt;
if contrl==1      % constant control
    ct=ones(1,nt);
elseif contrl==2  % impulse control
    ct=zeros(1,nt);
    ct(1)=1;    
else
    ct=contrl;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sample=feval(Get_Theta_q, DEF, mu);
theta_a = sample(1:Qa);
theta_f = sample(Qa+1:(Qa+Qf));
theta_m = sample(Qa+Qf+1:(Qa+Qf+Qm));
theta_u = sample((1+Qa+Qf+Qm):(Qa+Qf+Qm+Qu));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% ASSEMBLE AND SOLVE THE SYSTEM %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                   
A = theta_a(1)*OFFLINE.FEM.matrix.Aq{1};
for q=2:Qa
    A = A + theta_a(q)*OFFLINE.FEM.matrix.Aq{q};
end
M = theta_m(1)*OFFLINE.FEM.matrix.Mq{1};
for q=2:Qm
    M = M + theta_m(q)*OFFLINE.FEM.matrix.Mq{q};
end
F = theta_f(1)*OFFLINE.FEM.matrix.Fq{1};
for q=2:Qf
    F = F + theta_f(q)*OFFLINE.FEM.matrix.Fq{q};
end
u0 = theta_u(1)*OFFLINE.FEM.matrix.Uq{1};
for q=2:Qu
    u0 = u0 + theta_u(q)*OFFLINE.FEM.matrix.Uq{q};
end
% OFFLINE.FEM.matrix.Uq{1}
% OFFLINE.FEM.matrix.Uq{2}
% theta_u

%max(abs(F+A*OFFLINE.FEM.matrix.UD))
% theta_a
% theta_f
%F-A*OFFLINE.FEM.matrix.UD;
%max(abs(OFFLINE.FEM.matrix.UD+OFFLINE.FEM.matrix.Uq{2}))
%max(max(abs(M-OFFLINE.FEM.matrix.M_ref)))

%%%% Backward-Euler %%%%
K=M+(dt)*A;
for i=1:nt    
    if i==1
        u(:,i)=K\(M*u0+(ct(i)*dt)*F);
    else
        u(:,i)=K\(M*u(:,i-1)+(ct(i)*dt)*F); 
    end    
end

if nargin >=7
    if contrl_du==1      % constant control
        ct=ones(1,nt);
    elseif contrl_du==2  % impulse control
        ct=zeros(1,nt);
        ct(1)=1;    
    else
        ct=contrl_du;
    end
    Ql=du_DEF.Q_affine.l;
    sample=feval(output_Get_Theta_q, du_DEF, mu);    
    theta_l = sample(1:Ql);
    L = theta_l(1)*du_OFFLINE.FEM.matrix.Lq{1};
    for q=2:Ql
        L = L + theta_l(q)*du_OFFLINE.FEM.matrix.Lq{q};
    end
    Kdu=K';
    psi(:,nt)=Kdu\(0.5*ct(nt)*dt*L);
    if J==0
        for i=nt-1:-1:1
            psi(:,i)=Kdu\(M'*psi(:,i+1));   
        end
    else
        for i=nt-1:-1:nt-J+1
            psi(:,i)=Kdu\(ct(i)*dt*L+M'*psi(:,i+1));   
        end
        psi(:,nt-J)=Kdu\(ct(nt-J)*dt*L+M'*psi(:,nt-J+1));
        for i=nt-J-1:-1:1
            psi(:,i)=Kdu\(M'*psi(:,i+1));   
        end
    end
else
    psi=0;
end

