function [du_ONLINE, pr_ONLINE, du_OFFLINE]=offline_parabolic_dual(mu_index,samples,samples_l,...
         du_OFFLINE,pr_OFFLINE,du_ONLINE,pr_ONLINE,du_DEF,pr_DEF,flagHaveBases)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%  PROCESSING INPUTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
du_ONLINE.RB.timestep.dt=pr_OFFLINE.FEM.timestep.dt;
du_ONLINE.RB.timestep.nt=pr_OFFLINE.FEM.timestep.nt;
Qa=pr_DEF.Q_affine.a;
Qm=pr_DEF.Q_affine.m;
Qf=pr_DEF.Q_affine.f;
Qu=pr_DEF.Q_affine.u;
Ql=du_DEF.Q_affine.l;
nt=du_ONLINE.RB.timestep.nt;
dt=du_ONLINE.RB.timestep.dt;
Xnorm = pr_OFFLINE.FEM.matrix.Xnorm;
HT=pr_OFFLINE.FEM.matrix.HT; 
% impulse control
J=0;
ct=zeros(1,nt);
ct(nt)=1;    
if flagHaveBases==0
    du_OFFLINE.space.Z=[];
    du_OFFLINE.psit=[];
    du_OFFLINE.space.N=0;
    du_ONLINE.space.max_err_bound=1;
end
N=du_OFFLINE.space.N+1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
theta_a= samples(mu_index,1:Qa);
theta_f=samples(mu_index,(1+Qa):(Qa+Qf));
theta_m= samples(mu_index,(1+Qa+Qf):(Qa+Qf+Qm));
theta_u= samples(mu_index,(1+Qa+Qf+Qm):(Qa+Qf+Qm+Qu));
theta_l=samples_l(mu_index,1:Ql);

% Transpose Aq and Mq
for q=1:Qa
    Aq_du{q}=pr_OFFLINE.FEM.matrix.Aq{q}';
end
for q=1:Qm
    Mq_du{q}=pr_OFFLINE.FEM.matrix.Mq{q}';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% TWO-STAGE POD %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[psi]=parabolic_dual_solution(theta_a,theta_m,theta_l,Aq_du,...
      Mq_du,du_OFFLINE.FEM.matrix.Lq,nt,dt,J,ct);
etol=min(0.99,(du_ONLINE.space.max_err_bound)^2);  
[Psi,V,S]=pod(psi,HT*Xnorm,etol);
du_OFFLINE.psit=[du_OFFLINE.psit Psi];
[du_OFFLINE.space.Z,V,S]=pod(du_OFFLINE.psit,HT*Xnorm,N);
du_ONLINE.space.Nmax=size(du_OFFLINE.space.Z,2);
du_OFFLINE.space.N=size(du_OFFLINE.space.Z,2);
% du_OFFLINE.psit=[du_OFFLINE.psit psi];
% [du_OFFLINE.space.Z,V,S]=pod(du_OFFLINE.psit,HT*Xnorm,N);
% du_ONLINE.space.Nmax=size(du_OFFLINE.space.Z,2);
% du_OFFLINE.space.N=size(du_OFFLINE.space.Z,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% ONLINE RB DATA %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
U0=pr_OFFLINE.FEM.matrix.Uq{1};
for q=2:pr_DEF.Q_affine.u
    U0=[U0 pr_OFFLINE.FEM.matrix.Uq{q}];
end
for q=1:Qa
    du_ONLINE.RB.matrix.ANq{q}=du_OFFLINE.space.Z'*Aq_du{q}*du_OFFLINE.space.Z;
    du_ONLINE.RB.matrix.ANq_pr_du{q}=pr_OFFLINE.space.Z'*(Aq_du{q})*du_OFFLINE.space.Z;
end
for q=1:Qm
    du_ONLINE.RB.matrix.MNq{q}=du_OFFLINE.space.Z'*Mq_du{q}*du_OFFLINE.space.Z;    
    du_ONLINE.RB.matrix.MNq_pr_du{q}=pr_OFFLINE.space.Z'*(Mq_du{q})*du_OFFLINE.space.Z;
    du_ONLINE.RB.matrix.uN0{q}=du_OFFLINE.space.Z'*Mq_du{q}*U0;
end
for q=1:Qf
    du_ONLINE.RB.matrix.FNq{q}=pr_OFFLINE.FEM.matrix.Fq{q}'*du_OFFLINE.space.Z;
end
for q=1:Ql
    du_ONLINE.RB.matrix.LNq{q}=du_OFFLINE.space.Z'*du_OFFLINE.FEM.matrix.Lq{q};
    du_ONLINE.RB.matrix.LNq_pr{q}=pr_OFFLINE.space.Z'*du_OFFLINE.FEM.matrix.Lq{q};
    du_ONLINE.RB.matrix.LU0{q}=U0'*du_OFFLINE.FEM.matrix.Lq{q};
    du_ONLINE.RB.matrix.LUD{q}=pr_OFFLINE.FEM.matrix.UD'*du_OFFLINE.FEM.matrix.Lq{q};
end

%%%% dual norm of the dual residual %%%%
for q=1:Qa
    Zdua{q}=Xnorm\(HT\(Aq_du{q}*du_OFFLINE.space.Z));
end
for q=1:Qm
    Zdum{q}=Xnorm\(HT\(Mq_du{q}*du_OFFLINE.space.Z));
end
for q=1:Ql
    Zdul{q}=Xnorm\(HT\(du_OFFLINE.FEM.matrix.Lq{q}));
end

for q=1:Ql
  for p=1:Ql
    du_ONLINE.RB.matrix.cll{q,p}=du_OFFLINE.FEM.matrix.Lq{q}'*Zdul{p};
  end
end
for q=1:Qa
  for p=1:Qa
    du_ONLINE.RB.matrix.caa{q,p}=du_OFFLINE.space.Z'*Aq_du{q}'*Zdua{p};
  end
end
for q=1:Qm
  for p=1:Qm
    du_ONLINE.RB.matrix.cmm{q,p}=du_OFFLINE.space.Z'*Mq_du{q}'*Zdum{p};
  end
end
for q=1:Ql
  for p=1:Qm
    du_ONLINE.RB.matrix.clm{q,p}=du_OFFLINE.FEM.matrix.Lq{q}'*Zdum{p};
  end
end
for q=1:Ql
  for p=1:Qa
    du_ONLINE.RB.matrix.cla{q,p}=du_OFFLINE.FEM.matrix.Lq{q}'*Zdua{p};
  end
end
for q=1:Qa
  for p=1:Qm
    du_ONLINE.RB.matrix.cam{q,p}=du_OFFLINE.space.Z'*Aq_du{q}'*Zdum{p};
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% DUAL SOLUTION %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [psi]=parabolic_dual_solution(theta_a,theta_m,theta_l,Aq,Mq,Lq,nt,dt,J,ct)

Qa=length(theta_a);
Qm=length(theta_m);
Ql=length(theta_l);

A = theta_a(1)*Aq{1};
for q=2:Qa
    A = A + theta_a(q)*Aq{q};
end
M = theta_m(1)*Mq{1};
for q=2:Qm
    M = M + theta_m(q)*Mq{q};
end
L = theta_l(1)*Lq{1};
for q=2:Ql
    L = L + theta_l(q)*Lq{q};
end

K=(M+(dt)*A)';
[KL,KU]=lu(K);
psi(:,nt)=KU\(KL\(0.5*ct(nt)*dt*L));
if J==0
    for i=nt-1:-1:1
        psi(:,i)=KU\(KL\(M*psi(:,i+1)));   
    end
elseif J==1
    i=nt-1;
    psi(:,i)=KU\(KL\(0.5*ct(i)*dt*L+M*psi(:,i+1)));
    for i=nt-2:-1:1
        psi(:,i)=KU\(KL\(M*psi(:,i+1)));   
    end
elseif J>=2
    for i=nt-1:-1:nt-J+1
        psi(:,i)=KU\(KL\(ct(i)*dt*L+M*psi(:,i+1)));   
    end
    i=nt-J;
    psi(:,i)=KU\(KL\(0.5*ct(i)*dt*L+M*psi(:,i+1)));
    for i=nt-J-1:-1:1
        psi(:,i)=KU\(KL\(M*psi(:,i+1)));   
    end
end

