function [pr_OFFLINE,pr_ONLINE]=offline_parabolic_primal(mu_index,samples,...
      pr_OFFLINE,pr_ONLINE,pr_DEF,flagHaveBases)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%  PROCESSING INPUTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%       
pr_ONLINE.RB.timestep.dt=pr_OFFLINE.FEM.timestep.dt;
pr_ONLINE.RB.timestep.nt=pr_OFFLINE.FEM.timestep.nt;
Qa=pr_DEF.Q_affine.a;
Qm=pr_DEF.Q_affine.m;
Qf=pr_DEF.Q_affine.f;
Qu=pr_DEF.Q_affine.u;
Xnorm=pr_OFFLINE.FEM.matrix.Xnorm;
HT=pr_OFFLINE.FEM.matrix.HT; 
Ynorm=HT*Xnorm;
if flagHaveBases==0
    pr_OFFLINE.space.Z=[];
    pr_OFFLINE.ut=[];
    pr_OFFLINE.space.N=0;
    pr_ONLINE.space.max_err_bound=1;
end
N=pr_OFFLINE.space.N+1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
theta_a= samples(mu_index,1:Qa);
theta_f= samples(mu_index,(1+Qa):(Qa+Qf));
theta_m= samples(mu_index,(1+Qa+Qf):(Qa+Qf+Qm));
theta_u= samples(mu_index,(1+Qa+Qf+Qm):(Qa+Qf+Qm+Qu));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% TWO-STAGE POD %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[ut]=parabolic_primal_solution(theta_a, theta_f, theta_m, theta_u, pr_DEF, pr_OFFLINE);
etol=min(0.99,(pr_ONLINE.space.max_err_bound)^2);
[U,V,S]=pod(ut,Ynorm,etol);
pr_OFFLINE.ut=[pr_OFFLINE.ut U];
[pr_OFFLINE.space.Z,V,S]=pod(pr_OFFLINE.ut,Ynorm,N);
pr_ONLINE.space.Nmax=size(pr_OFFLINE.space.Z,2);
pr_OFFLINE.space.N=size(pr_OFFLINE.space.Z,2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% ONLINE RB DATA %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
U0=pr_OFFLINE.FEM.matrix.Uq{1};
for q=2:pr_DEF.Q_affine.u
    U0=[U0 pr_OFFLINE.FEM.matrix.Uq{q}];
end
u0=U0*theta_u';
for q=1:pr_DEF.Q_affine.a
    pr_ONLINE.RB.matrix.ANq{q} = pr_OFFLINE.space.Z'*pr_OFFLINE.FEM.matrix.Aq{q}*pr_OFFLINE.space.Z;
end
for q=1:pr_DEF.Q_affine.m
    pr_ONLINE.RB.matrix.MNq{q} = pr_OFFLINE.space.Z'*pr_OFFLINE.FEM.matrix.Mq{q}*pr_OFFLINE.space.Z;
    pr_ONLINE.RB.matrix.UNq{q} = pr_OFFLINE.space.Z'*pr_OFFLINE.FEM.matrix.Mq{q}*U0;
end
for q=1:pr_DEF.Q_affine.f
    pr_ONLINE.RB.matrix.FNq{q} = pr_OFFLINE.space.Z'*pr_OFFLINE.FEM.matrix.Fq{q};
end
pr_ONLINE.RB.matrix.X_norm_rb=pr_OFFLINE.space.Z'*Ynorm*pr_OFFLINE.space.Z;

%%%% dual norm of the primal residual %%%%
for q=1:pr_DEF.Q_affine.a
    Za{q}=Xnorm\(HT\(pr_OFFLINE.FEM.matrix.Aq{q}*pr_OFFLINE.space.Z));
end
for q=1:pr_DEF.Q_affine.m
    Zm{q}=Xnorm\(HT\(pr_OFFLINE.FEM.matrix.Mq{q}*pr_OFFLINE.space.Z));
    Zu{q}=Xnorm\(HT\(pr_OFFLINE.FEM.matrix.Mq{q}*U0));
end
for q=1:pr_DEF.Q_affine.f
    Zf{q}=Xnorm\(HT\pr_OFFLINE.FEM.matrix.Fq{q});
end

for q=1:pr_DEF.Q_affine.f
  for p=1:pr_DEF.Q_affine.f
    pr_ONLINE.RB.matrix.cff{q,p}=pr_OFFLINE.FEM.matrix.Fq{q}'*Zf{p};
  end
end
for q=1:pr_DEF.Q_affine.a
  for p=1:pr_DEF.Q_affine.a
    pr_ONLINE.RB.matrix.caa{q,p}=pr_OFFLINE.space.Z'*pr_OFFLINE.FEM.matrix.Aq{q}'*Za{p};
  end
end
for q=1:pr_DEF.Q_affine.m
  for p=1:pr_DEF.Q_affine.m
    pr_ONLINE.RB.matrix.cmm{q,p}=pr_OFFLINE.space.Z'*pr_OFFLINE.FEM.matrix.Mq{q}'*Zm{p};
  end
end
for q=1:pr_DEF.Q_affine.f
  for p=1:pr_DEF.Q_affine.a
    pr_ONLINE.RB.matrix.cfa{q,p}=pr_OFFLINE.FEM.matrix.Fq{q}'*Za{p};
  end
end
for q=1:pr_DEF.Q_affine.f
  for p=1:pr_DEF.Q_affine.m
    pr_ONLINE.RB.matrix.cfm{q,p}=pr_OFFLINE.FEM.matrix.Fq{q}'*Zm{p};
  end
end
for q=1:pr_DEF.Q_affine.a
  for p=1:pr_DEF.Q_affine.m
    pr_ONLINE.RB.matrix.cam{q,p}=pr_OFFLINE.space.Z'*pr_OFFLINE.FEM.matrix.Aq{q}'*Zm{p};
  end
end
for q=1:pr_DEF.Q_affine.f
  for p=1:pr_DEF.Q_affine.m
    pr_ONLINE.RB.matrix.cfu{q,p}=pr_OFFLINE.FEM.matrix.Fq{q}'*Zu{p};
  end
end
for q=1:pr_DEF.Q_affine.a
  for p=1:pr_DEF.Q_affine.m
    pr_ONLINE.RB.matrix.cau{q,p}=pr_OFFLINE.space.Z'*pr_OFFLINE.FEM.matrix.Aq{q}'*Zu{p};
  end
end
for q=1:pr_DEF.Q_affine.m
  for p=1:pr_DEF.Q_affine.m
    pr_ONLINE.RB.matrix.cmu{q,p}=pr_OFFLINE.space.Z'*pr_OFFLINE.FEM.matrix.Mq{q}'*Zu{p};
  end
end
for q=1:pr_DEF.Q_affine.m
  for p=1:pr_DEF.Q_affine.m
    pr_ONLINE.RB.matrix.cuu{q,p}=U0'*pr_OFFLINE.FEM.matrix.Mq{q}'*Zu{p};
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% PRIMAL SOLUTION %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [u]=parabolic_primal_solution(theta_a, theta_f, theta_m, theta_u, DEF, OFFLINE)

Qa=DEF.Q_affine.a;
Qm=DEF.Q_affine.m;
Qf=DEF.Q_affine.f;
Qu=DEF.Q_affine.u;
dt=OFFLINE.FEM.timestep.dt;
nt=OFFLINE.FEM.timestep.nt;

A = theta_a(1)*OFFLINE.FEM.matrix.Aq{1};
for q=2:Qa
    A = A + theta_a(q)*OFFLINE.FEM.matrix.Aq{q};
end
M = theta_m(1)*OFFLINE.FEM.matrix.Mq{1};
for q=2:Qm
    M = M + theta_m(q)*OFFLINE.FEM.matrix.Mq{q};
end
F = theta_f(1)*OFFLINE.FEM.matrix.Fq{1};
for q=2:Qf
    F = F + theta_f(q)*OFFLINE.FEM.matrix.Fq{q};
end
u0 = theta_u(1)*OFFLINE.FEM.matrix.Uq{1};
for q=2:Qu
    u0 = u0 + theta_u(q)*OFFLINE.FEM.matrix.Uq{q};
end

%impulse control
ct=zeros(1,nt);
ct(1)=1;    

%%%% Backward-Euler %%%%
K=M+(dt)*A;
[KL,KU]=lu(K);
for i=1:nt    
    if i==1
        u(:,i)=KU\(KL\(M*u0+(ct(i)*dt)*F));
    else
        u(:,i)=KU\(KL\(M*u(:,i-1)+(ct(i)*dt)*F)); 
    end    
end

