function [bN,normRdu,ct]=online_parabolic_dual(mu, Ndu, pr_DEF, du_DEF, du_ONLINE,...
    InsideOutsideD, Get_Theta_q, output_Get_Theta_q, nk_du, contrl, J, pr_OFFLINE, du_OFFLINE)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% CHECK INPUT ARGUMENTS %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[InsideBoolean] = feval(InsideOutsideD, pr_DEF, mu);
if InsideBoolean==0
   error(['WARNING : parameter value is out of range']);
end
Nmax=du_ONLINE.space.Nmax;
if (Ndu > Nmax) | (Ndu <=0)
   error(['Size of reduced basis space should be smaller than Nmax and greater than 0']);
end

Qa=pr_DEF.Q_affine.a;
Qm=pr_DEF.Q_affine.m;
Qf=pr_DEF.Q_affine.f;
Ql=du_DEF.Q_affine.l;
dt=du_ONLINE.RB.timestep.dt;
nt=du_ONLINE.RB.timestep.nt;
if J<0 | J > nt-1
    error('J must be such that 0<= J <= K-1');
end
if contrl==1      % constant control
    ct=ones(1,nt);
elseif contrl==2  % impulse control
    ct=zeros(1,nt);
    ct(nt)=1;    
else
    ct=contrl;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sample=feval(Get_Theta_q, du_DEF, mu);
theta_a = sample(1:Qa);
theta_f = sample(Qa+1:(Qa+Qf));
theta_m = sample(Qa+Qf+1:(Qa+Qf+Qm));
sample=feval(output_Get_Theta_q, du_DEF, mu);
theta_l = sample(1:Ql);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% FORM RB MATRICES & VECTORS %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AN_du = theta_a(1)*du_ONLINE.RB.matrix.ANq{1}(1:Ndu,1:Ndu);
for q=2:Qa
    AN_du = AN_du + theta_a(q)*du_ONLINE.RB.matrix.ANq{q}(1:Ndu,1:Ndu);
end
MN_du = theta_m(1)*du_ONLINE.RB.matrix.MNq{1}(1:Ndu,1:Ndu);
for q=2:Qm
    MN_du = MN_du + theta_m(q)*du_ONLINE.RB.matrix.MNq{q}(1:Ndu,1:Ndu);    
end
LN_du = theta_l(1)*du_ONLINE.RB.matrix.LNq{1}(1:Ndu);
for q=2:Ql
    LN_du = LN_du + theta_l(q)*du_ONLINE.RB.matrix.LNq{q}(1:Ndu);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% SOLVE THE RB SYSTEM %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
KN_du=MN_du+(dt)*AN_du;
[KNL,KNU]=lu(KN_du);
bN(:,nt)=KNU\(KNL\(0.5*ct(nt)*dt*LN_du));
if J==0
    for i=nt-1:-1:nk_du
        bN(:,i)=KNU\(KNL\(MN_du*bN(:,i+1)));   
    end
elseif J==1
    bN(:,nt-1)=KNU\(KNL\(0.5*ct(nt-1)*dt*LN_du+MN_du*bN(:,nt)));
    for i=nt-2:-1:nk_du
        bN(:,i)=KNU\(KNL\(MN_du*bN(:,i+1)));   
    end
elseif J>=2 
    for i=nt-1:-1:nt-J+1
        bN(:,i)=KNU\(KNL\(ct(i)*dt*LN_du+MN_du*bN(:,i+1)));
    end
    bN(:,nt-J)=KNU\(KNL\(ct(nt-J)*dt*LN_du+MN_du*bN(:,nt-J+1)));
    for i=nt-J-1:-1:nk_du
        bN(:,i)=KNU\(KNL\(MN_du*bN(:,i+1)));   
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% DUAL NORM OF THE RESIDUAL %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargout>1
dll=0;
for q=1:Ql
  for p=1:Ql
     dll=dll+theta_l(q)*theta_l(p)*du_ONLINE.RB.matrix.cll{q,p};
  end
end
daa=0;
for q=1:Qa
  for p=1:Qa
     daa=daa+theta_a(q)*theta_a(p)*du_ONLINE.RB.matrix.caa{q,p}(1:Ndu,1:Ndu);
  end
end
dmm=0;
for q=1:Qm
  for p=1:Qm
     dmm=dmm+theta_m(q)*theta_m(p)*du_ONLINE.RB.matrix.cmm{q,p}(1:Ndu,1:Ndu);
  end
end
dlm=0;
for q=1:Ql
  for p=1:Qm
     dlm=dlm+theta_l(q)*theta_m(p)*du_ONLINE.RB.matrix.clm{q,p}(1:Ndu);
  end
end
dla=0;
for q=1:Ql
  for p=1:Qa
     dla=dla+theta_l(q)*theta_a(p)*du_ONLINE.RB.matrix.cla{q,p}(1:Ndu);
  end
end
dam=0;
for q=1:Qa
  for p=1:Qm
     dam=dam+theta_a(q)*theta_m(p)*du_ONLINE.RB.matrix.cam{q,p}(1:Ndu,1:Ndu);
  end
end

normRdu(nt)=0.25*dll*ct(nt)^2-(1/dt)*ct(nt)*(dlm*bN(:,nt))-ct(nt)*(dla*bN(:,nt))...
            +bN(:,nt)'*daa*bN(:,nt)+(1/(dt*dt))*bN(:,nt)'*dmm*bN(:,nt)...
            +(2/dt)*bN(:,nt)'*dam*bN(:,nt);
if J==0        
    for i=nt-1:-1:nk_du
        normRdu(i)=bN(:,i)'*daa*bN(:,i)-(2/dt)*bN(:,i)'*dam*(bN(:,i+1)-bN(:,i))...
                   +(1/(dt*dt))*(bN(:,i+1)-bN(:,i))'*dmm*(bN(:,i+1)-bN(:,i));            
    end
elseif J==1
    i=nt-1;
    normRdu(i)=0.25*ct(i)^2*dll-(1/dt)*ct(i)*(dlm*(bN(:,i)-bN(:,i+1)))...
                  -ct(i)*(dla*bN(:,i))+bN(:,i)'*daa*bN(:,i)...
                  +(1/(dt*dt))*(bN(:,i)-bN(:,i+1))'*dmm*(bN(:,i)-bN(:,i+1))...                  
                  +(2/dt)*bN(:,i)'*dam*(bN(:,i)-bN(:,i+1));                  
    for i=nt-2:-1:nk_du
        normRdu(i)=bN(:,i)'*daa*bN(:,i)-(2/dt)*bN(:,i)'*dam*(bN(:,i+1)-bN(:,i))...
                   +(1/(dt*dt))*(bN(:,i+1)-bN(:,i))'*dmm*(bN(:,i+1)-bN(:,i));            
    end
elseif J>=2
    for i=nt-1:-1:nt-J
        normRdu(i)=ct(i)^2*dll-(2/dt)*ct(i)*(dlm*(bN(:,i)-bN(:,i+1)))...
                   -2*ct(i)*(dla*bN(:,i))+bN(:,i)'*daa*bN(:,i)...
                   +(1/(dt*dt))*(bN(:,i)-bN(:,i+1))'*dmm*(bN(:,i)-bN(:,i+1))...                  
                   +(2/dt)*bN(:,i)'*dam*(bN(:,i)-bN(:,i+1));                                   
    end
%     i=nt-J;
%     normRdu(i)=0.25*ct(i)^2*dll-(1/dt)*ct(i)*(dlm*(bN(:,i)-bN(:,i+1)))...
%                   -ct(i)*(dla*bN(:,i))+bN(:,i)'*daa*bN(:,i)...
%                   +(1/(dt*dt))*(bN(:,i)-bN(:,i+1))'*dmm*(bN(:,i)-bN(:,i+1))...                  
%                   +(2/dt)*bN(:,i)'*dam*(bN(:,i)-bN(:,i+1));                  
    for i=nt-J-1:-1:nk_du
        normRdu(i)=bN(:,i)'*daa*bN(:,i)-(2/dt)*bN(:,i)'*dam*(bN(:,i+1)-bN(:,i))...
                   +(1/(dt*dt))*(bN(:,i+1)-bN(:,i))'*dmm*(bN(:,i+1)-bN(:,i));            
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% TESTING: TO BE REMOVED ON LAUNCH %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% if Ndu>1
% [bN2,normRdu2]=check_parabolic_dual(mu,Ndu,pr_DEF,du_DEF,pr_OFFLINE,du_OFFLINE, ...
%     InsideOutsideD, Get_Theta_q, output_Get_Theta_q, contrl, J);     
% max(max(abs([bN-bN2])))                            
% max(max(abs([normRdu'-normRdu2'])))
% pause
% end

end
