function [sN,sN0,DN_s,DN_pr,DN_du,aN,bN]=online_parabolic_flux(mu,N_pr,N_du,pr_DEF,du_DEF,...
  pr_ONLINE,du_ONLINE,InsideOutsideD,Get_Theta_q,output_Get_Theta_q,output_Get_Theta_LM_q,...
  J,Li,contrl_pr,contrl_du,ct_du0)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
% number of timesteps and timestep size
dt=pr_ONLINE.RB.timestep.dt;
nt=pr_ONLINE.RB.timestep.nt;
nk_sol=Li+J;
if nk_sol(end)>nt | nk_sol(1)<1
    error('Timestep slots are out of range');
end    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% SCM FOR ALPHA LB %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[alpha_LB, alphaUB, rat, ONLINE]=alphaLB_coer(mu, pr_DEF, pr_ONLINE, Get_Theta_q);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% ONLINE FOR PRIMAL PROBLEM %%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[aN,normR,ct_pr]=online_parabolic_primal(mu, N_pr, pr_ONLINE, pr_DEF, ...
     InsideOutsideD, Get_Theta_q, nk_sol(end), contrl_pr);
nk_du=nt-nk_sol(end)+1;    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% ONLINE FOR DUAL PROBLEM %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[bN1,normRdu1,ct_du1]=online_parabolic_dual(mu, N_du, pr_DEF, du_DEF, du_ONLINE,...
     InsideOutsideD, Get_Theta_q, output_Get_Theta_q, nk_du, contrl_du, J);

for i=1:nt-2 
    contrl_du2(i)=(contrl_du(i+1)-contrl_du(i))/dt;
end
contrl_du2(nt-1)=(0.5*contrl_du(nt)-contrl_du(nt-1))/dt;
contrl_du2(nt)=(0-contrl_du(nt))/dt;
contrl_du2=-contrl_du2;
[bN2,normRdu2,ct_du2]=online_parabolic_dual(mu, N_du, pr_DEF, du_DEF, du_ONLINE,...
     InsideOutsideD, Get_Theta_q, output_Get_Theta_LM_q, nk_du, contrl_du2, J);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% RESIDUAL CORRECTION %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[residual1,theta_u1]=residual_correction(mu,N_pr,N_du,aN,bN1,pr_DEF,du_DEF,pr_ONLINE,du_ONLINE,...
    InsideOutsideD, Get_Theta_q, output_Get_Theta_q,nk_sol(end), ct_pr);          

[residual2,theta_u2]=residual_correction(mu, N_pr, N_du, aN, bN2, pr_DEF,du_DEF, pr_ONLINE, du_ONLINE,...
    InsideOutsideD, Get_Theta_q, output_Get_Theta_LM_q,nk_sol(end), ct_pr);          

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% OUTPUT AND ERROR BOUNDS %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ql=du_DEF.Q_affine.l;
sample=feval(output_Get_Theta_q, du_DEF, mu);
theta_l = sample(1:Ql);
LN = theta_l(1)*du_ONLINE.RB.matrix.LNq_pr{1}(1:N_pr);
for q=2:Ql
    LN=LN+theta_l(q)*du_ONLINE.RB.matrix.LNq_pr{q}(1:N_pr);
end
for k=1:nk_sol(end)
    DN_pr(k)=sqrt((dt/alpha_LB)*sum(abs(normR(1:k))));            
end
for k=nt:-1:nk_du
    DN_du(k)=sqrt((dt/alpha_LB)*sum(abs(normRdu(end:-1:k))));
end
DN_s=DN_pr.*DN_du(end:-1:nk_du);

for i=1:length(Li)
    saN=ct_du(Li(i))*aN(:,Li(i));
    for j=1:J-1
        saN=saN+ct_du(Li(i)+j)*aN(:,Li(i)+j);
    end
    if J>0
        saN=saN+0.5*ct_du(Li(i)+J)*aN(:,Li(i)+J);
    end
    sN0(i)=LN'*saN;        
    sN(i)=sN0(i)+(residual(Li(i)+J));
end
DN_s=DN_s(Li+J);
%DN_pr=DN_pr(Li+J);
%DN_du=DN_du(end:-1:nk_du);
%DN_du=DN_du(Li+J);
%DN_du=DN_du(end:-1:1);

if J==0 % point-wise output
    sN=sN0+residual(Li+J)/(0.5);
    DN_s=DN_s/(0.5*dt);
    DN_du=DN_du/(0.5*dt);    
else % integral output
    sN0=sN0*dt;
    sN=sN*dt;    
    LU0=theta_l(1)*du_ONLINE.RB.matrix.LU0{1};
    for q=2:Ql
        LU0=LU0+theta_l(q)*du_ONLINE.RB.matrix.LU0{q};
    end
    sLU0=0.5*dt*ct_du0*(theta_u*LU0);
    sN0=sN0+sLU0;    
    sN=sN+sLU0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% NESTED FUNCTION %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [residual,theta_u]=residual_correction(mu, N, Ndu, aN, bN, pr_DEF, du_DEF,...
    pr_ONLINE, du_ONLINE,InsideOutsideD, Get_Theta_q, output_Get_Theta_q, nk_sol, ct_pr)

Qa=pr_DEF.Q_affine.a;
Qm=pr_DEF.Q_affine.m;
Qf=pr_DEF.Q_affine.f;
Qu=pr_DEF.Q_affine.u;
Ql=du_DEF.Q_affine.l;
dt=pr_ONLINE.RB.timestep.dt;
nt=pr_ONLINE.RB.timestep.nt;

sample=feval(Get_Theta_q, pr_DEF, mu);
theta_a = sample(1:Qa);
theta_f = sample(Qa+1:(Qa+Qf));
theta_m = sample(Qa+Qf+1:(Qa+Qf+Qm));
theta_u = sample((1+Qa+Qf+Qm):(Qa+Qf+Qm+Qu));
sample=feval(output_Get_Theta_q, du_DEF, mu);
theta_l = sample(1:Ql);

FN_du=theta_f(1)*du_ONLINE.RB.matrix.FNq{1}(1:Ndu);
for q=2:Qf
    FN_du=FN_du+theta_f(q)*du_ONLINE.RB.matrix.FNq{q}(1:Ndu);
end
AN_pr_du=theta_a(1)*du_ONLINE.RB.matrix.ANq_pr_du{1}(1:N,1:Ndu);
for q=2:Qa
    AN_pr_du=AN_pr_du+theta_a(q)*du_ONLINE.RB.matrix.ANq_pr_du{q}(1:N,1:Ndu);
end
MN_pr_du=theta_m(1)*du_ONLINE.RB.matrix.MNq_pr_du{1}(1:N,1:Ndu);
uN0_du=theta_m(1)*du_ONLINE.RB.matrix.uN0{1}(1:Ndu,:);
for q=2:Qm
    MN_pr_du=MN_pr_du+theta_m(q)*du_ONLINE.RB.matrix.MNq_pr_du{q}(1:N,1:Ndu);
    uN0_du=uN0_du+theta_m(q)*du_ONLINE.RB.matrix.uN0{q}(1:Ndu,:);
end

RN(1,:)=ct_pr(1)*FN_du-aN(:,1)'*AN_pr_du-(1/dt)*(aN(:,1)'*MN_pr_du-(uN0_du*theta_u')');
for i=2:nk_sol    
    RN(i,:)=ct_pr(i)*FN_du-aN(:,i)'*AN_pr_du-(1/dt)*(aN(:,i)-aN(:,i-1))'*MN_pr_du;
end
residual(1)=RN(1,:)*bN(:,nt);
for k=2:nk_sol
    residual(k)=0;
    for kp=1:k
        residual(k)=residual(k)+RN(kp,:)*bN(:,nt-k+kp);  
    end
end



