function [aN,normR,ct]=online_parabolic_primal(mu, N, pr_ONLINE, pr_DEF,...
    InsideOutsideD, Get_Theta_q, nk, contrl, pr_OFFLINE)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% CHECK INPUT ARGUMENTS %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[InsideBoolean] = feval(InsideOutsideD, pr_DEF, mu);
if InsideBoolean==0
   error(['WARNING : parameter value is out of range']);
end
Nmax=pr_ONLINE.space.Nmax;
if (N > Nmax) | (N <=0)
   error(['Size of reduced basis space should be smaller than Nmax and greater than 0']);
end
Qa=pr_DEF.Q_affine.a;
Qm=pr_DEF.Q_affine.m;
Qf=pr_DEF.Q_affine.f;
Qu=pr_DEF.Q_affine.u;
dt=pr_ONLINE.RB.timestep.dt;
nt=pr_ONLINE.RB.timestep.nt;
if nk(end)<1 | nk(end) > nt
    error('nk must be such that 1<= nk <= K');
end
if contrl==1      % constant control
    ct=ones(1,nt);
elseif contrl==2  % impulse control
    ct=zeros(1,nt);
    ct(1)=1;    
else
    ct=contrl;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sample=feval(Get_Theta_q, pr_DEF, mu);
theta_a = sample(1:Qa);
theta_f = sample(Qa+1:(Qa+Qf));
theta_m = sample(Qa+Qf+1:(Qa+Qf+Qm));
theta_u = sample((1+Qa+Qf+Qm):(Qa+Qf+Qm+Qu));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% FORM RB MATRICES & VECTORS %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
AN = theta_a(1)*pr_ONLINE.RB.matrix.ANq{1}(1:N,1:N);
for q=2:Qa
    AN = AN + theta_a(q)*pr_ONLINE.RB.matrix.ANq{q}(1:N,1:N);
end
MN = theta_m(1)*pr_ONLINE.RB.matrix.MNq{1}(1:N,1:N);
uN0= theta_m(1)*(pr_ONLINE.RB.matrix.UNq{1}(1:N,:)*theta_u');
for q=2:Qm
    MN = MN + theta_m(q)*pr_ONLINE.RB.matrix.MNq{q}(1:N,1:N);
    uN0 = uN0 + theta_m(q)*(pr_ONLINE.RB.matrix.UNq{q}(1:N,:)*theta_u');
end
FN = theta_f(1)*pr_ONLINE.RB.matrix.FNq{1}(1:N);
for q=2:Qf
    FN = FN + theta_f(q)*pr_ONLINE.RB.matrix.FNq{q}(1:N);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% SOLVE THE RB SYSTEM %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
KN=MN+(dt)*AN;
[KNL,KNU]=lu(KN);
for i=1:nk
    if i==1
        aN(:,i)=KNU\(KNL\(uN0+(ct(i)*dt)*FN));
    else
        aN(:,i)=KNU\(KNL\(MN*aN(:,i-1)+(ct(i)*dt)*FN)); 
    end                
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% DUAL NORM OF THE RESIDUAL %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargout>1
dff=0;
for q=1:Qf
  for p=1:Qf
     dff=dff+theta_f(q)*theta_f(p)*pr_ONLINE.RB.matrix.cff{q,p};
  end
end
daa=0;
for q=1:Qa
  for p=1:Qa
     daa=daa+theta_a(q)*theta_a(p)*pr_ONLINE.RB.matrix.caa{q,p}(1:N,1:N);
  end
end
dmm=0;
for q=1:Qm
  for p=1:Qm
     dmm=dmm+theta_m(q)*theta_m(p)*pr_ONLINE.RB.matrix.cmm{q,p}(1:N,1:N);
  end
end
dfa=0;
for q=1:Qf
  for p=1:Qa
     dfa=dfa+theta_f(q)*theta_a(p)*pr_ONLINE.RB.matrix.cfa{q,p}(1:N);
  end
end
dfm=0;
for q=1:Qf
  for p=1:Qm
     dfm=dfm+theta_f(q)*theta_m(p)*pr_ONLINE.RB.matrix.cfm{q,p}(1:N);
  end
end
dam=0;
for q=1:Qa
  for p=1:Qm
     dam=dam+theta_a(q)*theta_m(p)*pr_ONLINE.RB.matrix.cam{q,p}(1:N,1:N);
  end
end
dfu=0;
for q=1:Qf
  for p=1:Qm
     dfu=dfu+theta_f(q)*theta_m(p)*pr_ONLINE.RB.matrix.cfu{q,p};
  end
end
dau=0;
for q=1:Qa
  for p=1:Qm
     dau=dau+theta_a(q)*theta_m(p)*pr_ONLINE.RB.matrix.cau{q,p}(1:N,:);
  end
end
dmu=0;
duu=0;
for q=1:Qm
  for p=1:Qm
     dmu=dmu+theta_m(q)*theta_m(p)*pr_ONLINE.RB.matrix.cmu{q,p}(1:N,:);
     duu=duu+theta_m(q)*theta_m(p)*pr_ONLINE.RB.matrix.cuu{q,p};
  end
end

for i=1:nk    
    if i==1        
        normR(i)=(ct(i)^2)*dff+aN(:,i)'*daa*aN(:,i)...
          +(1/(dt*dt))*(aN(:,i)'*dmm*aN(:,i))...
          -2*ct(i)*(dfa*aN(:,i))-(2*ct(i)/dt)*(dfm*aN(:,i))...
          +(2/dt)*(aN(:,i)'*dam*aN(:,i))...
          +(2*ct(i)/dt)*(dfu*theta_u')-(2/dt)*(aN(:,i)'*dau*theta_u')...
          -(2/(dt*dt))*(aN(:,i)'*dmu*theta_u')+(1/(dt*dt))*(theta_u*duu*theta_u');              
    else
        normR(i)=(ct(i)^2)*dff+aN(:,i)'*daa*aN(:,i)+...
                  (1/(dt*dt))*(aN(:,i)-aN(:,i-1))'*dmm*(aN(:,i)-aN(:,i-1))...
                  -2*ct(i)*(dfa*aN(:,i))-(2*ct(i)/dt)*(dfm*(aN(:,i)-aN(:,i-1)))+...
                  (2/dt)*(aN(:,i)'*dam*(aN(:,i)-aN(:,i-1)));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% TESTING: TO BE REMOVED ON LAUNCH %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% if N>2
% [aN2,normR2]=check_parabolic_primal(mu, N, pr_DEF, pr_OFFLINE, ...
%                                InsideOutsideD, Get_Theta_q, contrl);
% max(max(abs([aN'-aN2'])))                            
% max(max(abs([normR'-normR2'])))
% [max(normR) max(normR2)]
% pause
% end
end

