function plots(probname,outputname,mu_min,mu_max,muref,otype,filename)

% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 

% t=0:0.01:1;
% nt=length(t);
% ct=[ones(1,nt);1+sin(2*pi*t);[2*(0:0.01:0.5) 2*(0.49:-0.01:0)];[ones(1,51) zeros(1,50)]];
% figure(1);clf;
% set(axes, 'FontSize', 20, 'LineWidth', 1.0, 'GridLineStyle', '-', ...
%     'TickLength', [0.015 0.015]);
% hold on;
% plot(t,ct(1,:),'-k','MarkerSize',6,'LineWidth',2.0);
% plot(t,ct(2,:),'--k','MarkerSize',6,'LineWidth',2.0);
% plot(t,ct(3,:),'-.k','MarkerSize',6,'LineWidth',2.0);
% plot(t,ct(4,:),':k','MarkerSize',6,'LineWidth',2.0);
% xlabel('Time','FontSize',20);
% ylabel('Control  Inputs','FontSize',20);
% legend('Unit constant','Periodic','Trangular','Step');
% grid off;
% box on;
% axis tight;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% ADD PATHS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath('../rbMIT_Library')
addpath('../rbMIT_Library/parabolic')
addpath('../rbMIT_Library/symbolic')
d0=fileparts([pwd,filesep]);
addpath([d0,strcat('/',outputname)]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% LOAD DATA %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s1=strcat(probname,'_','pr_ONLINE');
s2=strcat(probname,'_','pr_PROBDEF');
s3=strcat(probname,'_','pr_OFFLINE');
s4=strcat(probname,'_','VIS');
load(s1); load(s2); load(s3); load(s4);
eval(['pr_ONLINE=' s1 ';']);
eval(['pr_DEF=' s2 ';']);
eval(['pr_OFFLINE=' s3 ';']);
eval(['VIS=' s4 ';']);
clear(s1); clear(s2); clear(s3); clear(s4);
s1=strcat(probname,'_',outputname,'_','du_ONLINE');
s2=strcat(probname,'_',outputname,'_','du_PROBDEF');
s3=strcat(probname,'_',outputname,'_','du_OFFLINE');
load(s1); load(s2); load(s3);
eval(['du_ONLINE=' s1 ';']);
eval(['du_DEF=' s2 ';']);
eval(['du_OFFLINE=' s3 ';']);
clear(s1); clear(s2); clear(s3);

nt=pr_ONLINE.RB.timestep.nt;
dt=pr_ONLINE.RB.timestep.dt;
Nmax=pr_ONLINE.space.Nmax;
Mmax=du_ONLINE.space.Nmax;
B=VIS.B;  
pmesh=VIS.p;
tmesh=VIS.t;
UD=VIS.UD;
emesh=VIS.e;

% Plot the reference domain and mesh
figure(1);clf;
pdemesh(pmesh,emesh,tmesh);
axis equal; axis tight; axis on;
print('-depsc',[filename num2str(1)]);

% Plot the greedy error bounds
maxe=pr_OFFLINE.space.eps_out_star;
figure(2);clf;
set(axes, 'FontSize', 20, 'LineWidth', 1.0, 'YScale', 'log', 'GridLineStyle', '-', ...
    'TickLength', [0.015 0.015]);
semilogy(1:length(maxe),maxe,'-ok','MarkerSize',6,'LineWidth',2.0);
xlabel('$N$','FontSize',20,'Interpreter','latex');
ylabel('Maximum  Error  Bound','FontSize',20);
grid off;
box on;
axis tight;   
print('-deps',[filename num2str(2)]);

% if strcmp(DEF.type,'parabolic')
%     figure(1);clf;
%     set(axes, 'FontSize', 20, 'LineWidth', 1.0, 'YScale', 'log', 'GridLineStyle', '-', ...
%         'TickLength', [0.015 0.015]);
%     semilogy(1:length(ONLINE.space.eps_out_star),ONLINE.space.eps_out_star,'-ok','MarkerSize',6,'LineWidth',2.0);
%     xlabel('$N$','FontSize',20,'Interpreter','latex');
%     ylabel('Maximum  Error  Bound','FontSize',20);
%     grid off;
%     box on;
%     axis tight;   
% end

maxe=du_OFFLINE.space.eps_out_star;
figure(3);clf;
set(axes, 'FontSize', 20, 'LineWidth', 1.0, 'YScale', 'log', 'GridLineStyle', '-', ...
    'TickLength', [0.015 0.015]);
semilogy(1:length(maxe),maxe,'-ok','MarkerSize',6,'LineWidth',2.0);
xlabel('$N$','FontSize',20,'Interpreter','latex');
ylabel('Maximum  Error  Bound','FontSize',20);
grid off;
box on;
axis tight;
print('-deps',[filename num2str(3)]);

% random parameter
mu=mu_min+(mu_max-mu_min).*rand(size(mu_min));
pmu=geoplot(probname,mu);
InsideOutsideD=str2func(strcat(probname,'_','InsideOutsideD'));
Get_Theta_q=str2func(strcat(probname,'_','Get_Theta_q'));

% Primal control
t=dt:dt:nt*dt;
ctrl_pr{1}='1';
ctrl_pr{2}='1+sin(2*pi*t)';
ctrl_pr{3}='t.*(t<=0.5)+(1-t).*(t>0.5)';
ctrl_pr{4}='t.*(t<=0.5)';
for i=1:length(ctrl_pr)
    if otype==1;
        ctrl_du{i}=1;
    else
        ctrl_du{i}=ctrl_pr{i};
    end
end
     
% Plot solutions
for i=1:length(ctrl_pr)
    temp=eval(ctrl_pr{i});
    [aN]=online_parabolic_primal(mu, Nmax, pr_ONLINE, pr_DEF, ...
         InsideOutsideD, Get_Theta_q, nt, temp);    
    aN=aN(:,nt); 
    uRB=pr_OFFLINE.space.Z(:,1:Nmax)*aN;
    uT=B*uRB+UD;
    figure(3+i);clf;
    PLOTSOLUTION(pmu',tmesh(1:3,:)',uT);
    colorbar;
    axis equal; axis tight; axis off;
    print('-depsc',[filename num2str(3+i)]);
end


for i=1:length(ctrl_pr)
    for N=1:Nmax
        [sN,DN_s,es]=RB_parabolic(probname,outputname,mu,N,min(N,Mmax),ctrl_pr{i},ctrl_du{i},2);
        maxes(i,N)=max(es);
        DeltaN(i,N)=max(DN_s);
    end
end

% Plot error and error bounds
for i=1:length(ctrl_pr)
    figure(7+i);clf;
    set(axes, 'FontSize', 20, 'LineWidth', 1.0, 'YScale', 'log', 'GridLineStyle', '-', ...
    'TickLength', [0.015 0.015]);
    hold on;
    semilogy(1:length(maxes(i,:)),DeltaN(i,:),'-sk','MarkerSize',6,'LineWidth',2.0);
    semilogy(1:length(maxes(i,:)),maxes(i,:),'-ok','MarkerSize',6,'LineWidth',2.0);
    xlabel('$N$','FontSize',20,'Interpreter','latex');
    ylabel('Convergence','FontSize',20);
    legend({['$\Delta_N^s$'],['$\varepsilon_N^s$']},'Location','NE','Interpreter','latex');
    grid off;
    box on;
    axis tight;
    hold off;
    print('-deps',[filename num2str(7+i)]);
end


function PLOTSOLUTION(p,t,u)
% Plot the temperature field

ss=[1 0 0; 0 1 0; 0 0 1];
tt=[1 2 3];
ns=size(ss,1);
nt=size(t,1);
dim=2;

pp=zeros(ns,dim,nt);
for comp=1:dim
  for trinode=1:dim+1
    dp=ss(:,trinode)*(p(t(:,trinode),comp))';        
    pp(:,comp,:)=pp(:,comp,:)+permute(dp,[1,3,2]);
  end
end

uu=zeros(ns,nt);
for trinode=1:dim+1    
    du=ss(:,trinode)*(u(t(:,trinode),1))';    
    uu=uu+du;
end

pvis=reshape(permute(pp,[1,3,2]),[ns*nt,2]);
tvis=kron(ones(nt,1),tt)+kron(ns*(0:nt-1)',0*tt+1);

cla,patch('vertices',pvis,'faces',tvis,'cdata',uu, ...
          'facecol','interp','edgec','none');

set(gcf,'rend','z');
drawnow



