function varargout=vistdepbs1d(varargin)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
global VIS pr_OFFLINE pr_ONLINE pr_DEF cr_probname;

if nargin<1
    error('Warning: Too few input arguments.');
end
if (nargout > 3)
   error('Warning: Too many output arguments.')
end
if strcmp(cr_probname,varargin{1})~=1
    clear global pr_ONLINE pr_DEF du_ONLINE du_DEF cr_probname;
    global pr_ONLINE pr_DEF du_ONLINE du_DEF cr_probname;
end

% Handling input arguments
[VIS,pr_OFFLINE,pr_ONLINE,pr_DEF,probname,mu,N_pr,...
          contrl_pr,nk,options] = checkInputs(varargin{:});      
cr_probname=probname;
p=VIS.p;
t=VIS.t;
UD=VIS.UD;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% THETA FUNCTIONS %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
InsideOutsideD=str2func(strcat(probname,'_','InsideOutsideD'));
Get_Theta_q=str2func(strcat(probname,'_','Get_Theta_q'));

% start timing pre-processing
cputms = zeros(1,1);
t0 = cputime; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% REDUCED BASIS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[aN,normR]=online_parabolic_primal(mu, N_pr, pr_ONLINE, pr_DEF, ...
     InsideOutsideD, Get_Theta_q, nk, contrl_pr);    
[alpha_LB, alphaUB, rat, ONLINE]=alphaLB_coer(mu, pr_DEF, pr_ONLINE, Get_Theta_q);
dt=pr_ONLINE.RB.timestep.dt;
nt=pr_ONLINE.RB.timestep.nt;
delta_u=sqrt((dt/abs(alpha_LB))*sum(abs(normR(1:nk))));            
sol_bound = delta_u;
disp(['Error bound in energy norm at timestep ' num2str(nk) ' is ' num2str(sol_bound)]);
varargout{1}=sol_bound;

if contrl_pr==1  
    ct=ones(1,nt);
elseif contrl_pr==2 
    ct=zeros(1,nt);
    ct(1)=1;    
else
    ct=contrl_pr;
end

if strcmp(options,'mv')==1    
    the=2*mu(2)-mu(1)^2-mu(3);
    uRBt=pr_OFFLINE.space.Z(:,1:N_pr)*aN(:,1:nk);
    uRB=zeros(length(p),size(uRBt,2));
    uRB(2:end-1,:)=uRBt;
    for j=1:nk
        uT(:,j)=exp(-the*(j)*dt)*uRB(:,j)+exp(-mu(2)*(j)*dt)*UD;        
    end
    [X,Y]=meshgrid(dt:dt:dt*nk,p);
    figure(1);clf;mesh(X,Y,uT);axis tight;
    %[ufem,ut] = bs1dsol(pr_OFFLINE,pr_DEF,Get_Theta_q,mu,nt,dt);
    %[X,Y]=meshgrid(dt:dt:dt*nk,p(2:end-1));
    %figure(2);clf;mesh(X,Y,ut-uRBt);axis tight;    
else
    the=2*mu(2)-mu(1)^2-mu(3);
    uRBt=pr_OFFLINE.space.Z(:,1:N_pr)*aN(:,nk);    
    uRB=zeros(size(p));
    uRB(2:end-1)=uRBt;
    uT=exp(-the*(nk)*dt)*uRB+exp(-mu(2)*(nk)*dt)*UD;        
    figure(1);clf;
    plot(p,uT);
    axis tight;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% HANDLING INPUT ARGUMENTS %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
function [VIS,pr_OFFLINE,pr_ONLINE,pr_DEF,probname,mu,N_pr,...
          contrl_pr,nk,options] = checkInputs(varargin)

global VIS pr_OFFLINE pr_ONLINE pr_DEF

probname=varargin{1};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% ADD PATHS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%addpath('../rbMIT_Library')
%addpath('../rbMIT_Library/parabolic')
%addpath('../rbMIT_Library/symbolic')
d0=fileparts([pwd,filesep]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% LOAD ONLINE DATA %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(VIS) | isempty(pr_OFFLINE) | isempty(pr_ONLINE) | isempty(pr_DEF)
    s1=strcat(probname,'_','pr_ONLINE');
    s2=strcat(probname,'_','pr_PROBDEF');
    s3=strcat(probname,'_','pr_OFFLINE');
    s4=strcat(probname,'_','VIS');
    load(s1); load(s2); load(s3); load(s4);
    eval(['pr_ONLINE=' s1 ';']);
    eval(['pr_DEF=' s2 ';']);
    eval(['pr_OFFLINE=' s3 ';']);
    eval(['VIS=' s4 ';']);
    clear(s1); clear(s2); clear(s3); clear(s4);
end

nt=pr_ONLINE.RB.timestep.nt;
dt=pr_ONLINE.RB.timestep.dt;
t=dt:dt:nt*dt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% GET INPUT ARGUMENTS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (nargin<2)
    mu=pr_DEF.mu_min+(pr_DEF.mu_max-pr_DEF.mu_min).*rand(size(pr_DEF.mu_max));
else
    mu=varargin{2};
end
if isempty(mu)
    mu=pr_DEF.mu_min+(pr_DEF.mu_max-pr_DEF.mu_min).*rand(size(pr_DEF.mu_max));
end
if nargin<3
    N_pr=pr_ONLINE.space.Nmax; % default: N=N_max
else
    N_pr=varargin{3};
end
if isempty(N_pr)
    N_pr=pr_ONLINE.space.Nmax;
end
if nargin<4    
    contrl_pr=1; % default: unity constant control
else
    gt=varargin{4};
    if isempty(gt)
        contrl_pr=1; 
    else        
        if ischar(gt)
            contrl_pr=eval(gt).*ones(1,length(t));
        else
            contrl_pr=gt;
        end
    end    
end
if isempty(contrl_pr)
    contrl_pr=1; 
end
if nargin<5
    nk=pr_ONLINE.RB.timestep.nt; % default: final timestep
else
    nk=varargin{5};
end
if isempty(nk)
    nk=pr_ONLINE.RB.timestep.nt; 
end
if nargin<6
    options=0;
else
    options=varargin{6};
end
if isempty(options)
    options=0; 
end


function [ufem,ut] = bs1dsol(pr_OFFLINE,pr_DEF,Get_Theta_q,mu,nt,dt)
Thetavectorvalue=feval(Get_Theta_q, pr_DEF, mu);
Qa=length(pr_OFFLINE.FEM.matrix.Aq);
Qf=length(pr_OFFLINE.FEM.matrix.Fq);
Qm=length(pr_OFFLINE.FEM.matrix.Mq);
Qu=length(pr_OFFLINE.FEM.matrix.Uq);
Thetaa(:,1:Qa) = Thetavectorvalue(:,1:Qa);
Thetaf(:,1:Qf) = Thetavectorvalue(:,Qa+1:Qa+Qf);
Thetam(:,1:Qm) = Thetavectorvalue(:,Qa+Qf+1:Qa+Qf+Qm);
Thetau(:,1:Qu) = Thetavectorvalue(:,Qa+Qf+Qm+1:Qa+Qf+Qm+Qu);
A = 0*pr_OFFLINE.FEM.matrix.Aq{1};
for q = 1:Qa
    A = A + Thetaa(q)*pr_OFFLINE.FEM.matrix.Aq{q};
end;
F = 0*pr_OFFLINE.FEM.matrix.Fq{1};
for q = 1:Qf
    F = F + Thetaf(q)*pr_OFFLINE.FEM.matrix.Fq{q};
end;    
M = 0*pr_OFFLINE.FEM.matrix.Mq{1};
for q = 1:Qm
    M = M + Thetam(q)*pr_OFFLINE.FEM.matrix.Mq{q};
end;    
u0 = 0*pr_OFFLINE.FEM.matrix.Uq{1};
for q = 1:Qm
    u0 = u0 + Thetau(q)*pr_OFFLINE.FEM.matrix.Uq{q};
end;    
UD=pr_OFFLINE.FEM.matrix.UD;
K=M+dt*A;
the=2*mu(2)-mu(1)^2-mu(3);
ct1=exp(the*(dt:dt:nt*dt));
ct2=exp(-mu(2)*(dt:dt:nt*dt));
ct=ct1.*ct2;
for i=1:nt    
    if i==1
        ut(:,i)=K\(M*(u0)+(ct(i)*dt)*F);        
    else
        ut(:,i)=K\(M*ut(:,i-1)+(ct(i)*dt)*F);         
    end    
    ufem(:,i)=exp(-the*i*dt)*ut(:,i)+exp(-mu(2)*(i*dt))*UD;
end

