function [Kq, Fq] = assem1(p,t,kappa,f,cflag)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
% flag correction for assema "style" - constant coeff at central point
if not(exist('cflag'))
    cflag = 0;
end
% Coefficients set-up
if (size(kappa,1)~=size(kappa,2)) || (size(kappa,1)~=size(f,1)*3)
    fprintf('assemMAT: Input is not consistent!\n');
    return
end
n = size(f,1);
ischarkappa = checkchar(kappa);
ischarf = checkchar(f);
% Gaussian points set-up
if not(cflag)
%     Gint.GPoints = [1/3,1/3]; Gint.Gweight = [1/2]; % precision 1
    Gint.GPoints = [2/3,1/6;1/6,2/3;1/6,1/6]; Gint.Gweight = [1/6;1/6;1/6]; % precision 2
else
    aa = 0.0E+00;    bb = 0.5E+00;    cc = 2.0E+00 /  3.0E+00;    dd = 1.0E+00 /  6.0E+00;    vv = 1.0E+00 / 30.0E+00;    ww = 3.0E+00 / 10.0E+00;
    Gint.GPoints = [aa,bb; bb,aa; bb,bb; cc,dd; dd,cc; dd,dd;]; Gint.Gweight = [vv; vv; vv; ww; ww; ww]/2; % precision 6
end
for i = 1:3*n
    for j = 1:3*n
        Kq{i,j} = sparse(size(p,2)*n,size(p,2)*n);
    end
end
for i = 1:n
    Fq{i} = zeros(size(p,2)*n,1);
end
for ie = 1:size(t,2)
    nodes = p(:,t(1:3,ie))';
    if not(cflag)
        [Nc, dNc] = shape([1/3,1/3],nodes);  iXc = Nc*nodes; xc = iXc(1); yc = iXc(2);
    end
    switch n
        case 1
            I = t(1:3,ie);
            for ipoint = 1:length(Gint.Gweight)
                [N, dN] = shape(Gint.GPoints(ipoint,:),nodes);
                J = dN*nodes;
                iJ = inv(J);
                iN = iJ*dN;
                iX = N*nodes; x = iX(1); y = iX(2);
                if not(cflag)
                    x = xc; y = yc;
                end
                detJw = abs(det(J))*Gint.Gweight(ipoint); % counter-clockwise
                for i = 1:3*n
                    for j = 1:3*n
                        ktmp = zeros(3*n);
                        if ischarkappa(i,j)
                            ktmp(i,j) = eval(kappa{i,j});
                        else
                            ktmp(i,j) = kappa{i,j};
                        end
                        if (ktmp(i,j)~=0)
                            Kq{i,j}(I,I) = Kq{i,j}(I,I) + [iN;N]'*ktmp*[iN;N]*detJw;
                        end
                    end
                end
                for i = 1:n
                    ftmp = zeros(1,n);
                    if ischarf(i)
                        ftmp(i) = eval(f{i});
                    else
                        ftmp(i) = f{i};
                    end
                    if (ftmp(i)~=0)
                        Fq{i}(I) = Fq{i}(I) + N'*ftmp*detJw;
                    end
                end
            end
        case 2
            %nothing yet!
            I = [t(1:3,ie);size(p,2)+t(1:3,ie)];
            for ipoint = 1:length(Gint.Gweight)
                [N, dN] = shape(Gint.GPoints(ipoint,:),nodes);
                J = dN*nodes;
                iJ = inv(J);
                B = zeros(6,6); 
                B([1:2],[1:2]) = iJ; B([3:4],[3:4]) = iJ;
                B(5,5) = 1; B(6,6) = 1;
                C = zeros(6,6);
                C([1,2],[1,2,3]) = dN; C([3,4],[4,5,6]) = dN;
                C(5,[1,2,3]) = N; C(6,[4,5,6]) = N;
                A = B*C;
                iX = N*nodes; x = iX(1); y = iX(2);
                if not(cflag)
                    x = xc; y = yc;
                end
                detJw = abs(det(J))*Gint.Gweight(ipoint); % counter-clockwise
                for i = 1:3*n
                    for j = 1:3*n
                        ktmp = zeros(3*n);
                        if ischarkappa(i,j)
                            ktmp(i,j) = eval(kappa{i,j});
                        else
                            ktmp(i,j) = kappa{i,j};
                        end
                        if (ktmp(i,j)~=0)
                            Kq{i,j}(I,I) = Kq{i,j}(I,I) + A'*ktmp*A*detJw;
                        end
                    end
                end
                D = zeros(2,6);
                D(1,[1,2,3]) = N;
                D(2,[4,5,6]) = N;
                for i = 1:n
                    ftmp = zeros(n,1);
                    if ischarf(i)
                        ftmp(i) = eval(f{i});
                    else
                        ftmp(i) = f{i};
                    end
                    if (ftmp(i)~=0)
                        Fq{i}(I) = Fq{i}(I) + D'*ftmp*detJw;
                    end
                end
            end
    end
end

function [N, dN] = shape(L, nodes)
N = [L(1), L(2), 1-L(1)-L(2)];
dN = [1,0,-1;0,1,-1];

function ischarmat = checkchar(invar)
for i = 1:size(invar,1)
    for j = 1:size(invar,2)
        ischarmat(i,j) = ischar(invar{i,j});
    end
end