function [Qq,Gq,Hq,Rq] = assem2(p,e,n,neumann,dirichlet,cflag)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
if not(exist('cflag'))
    cflag = 0;
end

if not(isempty(neumann)) && not(isempty(dirichlet))
    flag = 3; % mixed bc
elseif not(isempty(neumann))
    flag = 2; % only Neumann bc
elseif not(isempty(dirichlet))
    flag = 1; % only Dirichlet bc
else
%     fprintf('Error in assem2: no input?\n');
    % phuong - 9/25/08
    Qq = []; Gq = []; Hq = []; Rq = [];    
    return;
end

Gint.GPoints = [0.93246951420315;   0.66120938646626;   0.23861918608320;  -0.23861918608320;  -0.66120938646626;  -0.93246951420315];
Gint.Gweight = [0.17132449237917;   0.36076157304814;   0.46791393457269;   0.46791393457269;   0.36076157304814;   0.17132449237917];

% Neumann
Qq = []; Gq = [];
for ibc = 1:length(neumann)
    for i1 = 1:n
        for i2 = 1:n
            Qq{ibc}{i1,i2} = sparse(size(p,2)*n,size(p,2)*n);
        end
        Gq{ibc}{i1} = sparse(size(p,2)*n,1);
    end
    allI = find(e(5,:) == neumann{ibc}.edge);
    for i = 1:n
        for j = 1:n
            ischarq(i,j) = ischar(neumann{ibc}.q{i,j});
        end
    end
    for i = 1:n
        ischarg(i) = ischar(neumann{ibc}.g{i});
    end
    for ie = 1:length(allI)
        I = e(1:2,allI(ie));
        for j = 2:n
            I = [I;size(p,2)*(j-1)+e(1:2,allI(ie))];
        end
        nodes = p(:,e(1:2,allI(ie)))';
        dJ = 1/2*norm(nodes(2,:)-nodes(1,:),2);

        iX = [0.5,0.5]*nodes; x = iX(1); y = iX(2);
        for i = 1:n
            for j = 1:n
                if ischar(neumann{ibc}.q{i,j})
                    tmpqc(i,j) = eval(neumann{ibc}.q{i,j});
                else
                    tmpqc(i,j) = neumann{ibc}.q{i,j};
                end
            end
            if ischar(neumann{ibc}.g{i})
                tmpgc(i) = eval(neumann{ibc}.g{i});
            else
                tmpgc(i) = neumann{ibc}.g{i};
            end
        end

        for ipoint = 1:length(Gint.Gweight)
            N0 = shape(Gint.GPoints(ipoint,:),nodes);
            iX = N0*nodes; x = iX(1); y = iX(2);
            for i = 1:n
                N(i,(i-1)*length(N0)+1:i*length(N0)) = N0;
            end
            for i = 1:n
                for j = 1:n
                    tmpq = zeros(n,n);
                    if ischarq(i,j)
                        tmpq(i,j) = eval(neumann{ibc}.q{i,j});
                        if not(cflag)
                            tmpq(i,j) = tmpqc(i,j);
                        end
                    else
                        tmpq(i,j) = neumann{ibc}.q{i,j};
                    end
                    Qq{ibc}{i,j}(I,I) = Qq{ibc}{i,j}(I,I) + N'*tmpq*N*dJ*Gint.Gweight(ipoint);
                end
                tmpg = zeros(n,1);
                if ischarg(i)
                    tmpg(i) = eval(neumann{ibc}.g{i});
                    if not(cflag)
                        tmpg(i) = tmpgc(i);
                    end
                else
                    tmpg(i) = neumann{ibc}.g{i};
                end
                Gq{ibc}{i}(I) = Gq{ibc}{i}(I) + N'*tmpg*dJ*Gint.Gweight(ipoint);
            end
        end
    end
end

% Dirichlet
Hq = []; Rq = [];
for ibc = 1:length(dirichlet)
    tmpHq = sparse(size(p,2)*n,size(p,2)*n);
    tmpRq = zeros(size(p,2)*n,1);
    Rcount = ones(size(p,2)*n,1);
    count = 0;
    oldn = [];
    for ie = 1:length(dirichlet{ibc})
        if (length(dirichlet{ibc}{ie}.h) ~= length(dirichlet{ibc}{ie}.r))
            fprintf('Error in assem2: h and r inputs are not correct?\n');
            return;
        end
        numh = length(dirichlet{ibc}{ie}.h);
        for i = 1:length(dirichlet{ibc}{ie}.h)
            for j = 1:n
                ischarh(i,j) = ischar(dirichlet{ibc}{ie}.h{i}{j});
            end
        end
        for i = 1:length(dirichlet{ibc}{ie}.r)
            ischarr(i) = ischar(dirichlet{ibc}{ie}.r{i});
        end
        allI = find(e(5,:) == dirichlet{ibc}{ie}.edge);
        alln = unique(e(1:2,allI));
        numdbc = length(alln);
        for idbc = 1:numdbc
            for i = 1:numh
                for j = 1:n
                    if ischarh(i,j)
                        x = p(1,alln(idbc)); y = p(2,alln(idbc));
                        tmpHq(count + idbc+(i-1)*numdbc,alln(idbc)+(j-1)*size(p,2)) = eval(dirichlet{ibc}{ie}.h{i}{j});
                    else
                        tmpHq(count + idbc+(i-1)*numdbc,alln(idbc)+(j-1)*size(p,2)) = dirichlet{ibc}{ie}.h{i}{j};
                    end
                end
                [isbcexist,tmploc] = ismember(tmpHq(count + idbc+(i-1)*numdbc,:),tmpHq(1:count + idbc+(i-1)*numdbc-1,:),'rows');
                if ischarr(i)
                    x = p(1,alln(idbc)); y = p(2,alln(idbc));
                    tmp = eval(dirichlet{ibc}{ie}.r{i});
                else
                    tmp = dirichlet{ibc}{ie}.r{i};
                end
                if isbcexist
                    tmpRq(count + idbc+(i-1)*numdbc) = tmp+tmpRq(tmploc);
                    Rcount(count + idbc+(i-1)*numdbc) = Rcount(count + idbc+(i-1)*numdbc) + 1;
                    tmpRq(tmploc) = tmp+tmpRq(tmploc);
                    Rcount(tmploc) = Rcount(tmploc) + 1;                    
                else
                    tmpRq(count + idbc+(i-1)*numdbc) = tmp;
                end
            end
        end
        count = count + numdbc*numh;
    end
    tmpRq = tmpRq./Rcount;
    tmpHq = tmpHq(1:count,1:size(p,2)*n);
    tmpRq = tmpRq(1:count);
    [tmp,I,J] = unique(tmpHq,'rows');
    Hq{ibc} = tmp;
    Rq{ibc} = tmpRq(I,:);
end

% % Dirichlet
% Hq = []; Rq = [];
% for ibc = 1:length(dirichlet)
%     if (length(dirichlet{ibc}.h) ~= length(dirichlet{ibc}.r))
%         fprintf('Error in assem2: h and r inputs are not correct?\n');
%         return;
%     end
%     numh = length(dirichlet{ibc}.h);
%     for i = 1:length(dirichlet{ibc}.h)
%         for j = 1:n
%             ischarh(i,j) = ischar(dirichlet{ibc}.h{i}{j});
%         end
%     end
%     for i = 1:length(dirichlet{ibc}.r)
%         ischarr(i) = ischar(dirichlet{ibc}.r{i});
%     end
%     allI = find(e(5,:) == dirichlet{ibc}.edge);
%     alln = unique(e(1:2,allI));
%     numdbc = length(alln);
%     Hq{ibc} = sparse(numdbc*length(dirichlet{ibc}.h),size(p,2)*n);
%     Rq{ibc} = zeros(numdbc*length(dirichlet{ibc}.h),1);
%     for idbc = 1:numdbc
%         for i = 1:numh
%             for j = 1:n
%                 if ischarh(i,j)
%                     x = p(1,idbc); y = p(2,idbc);
%                     Hq{ibc}(idbc+(i-1)*numdbc,alln(idbc)+(j-1)*size(p,2)) = eval(dirichlet{ibc}.h{i}{j});
%                 else
%                     Hq{ibc}(idbc+(i-1)*numdbc,alln(idbc)+(j-1)*size(p,2)) = dirichlet{ibc}.h{i}{j};
%                 end
%             end
%             if ischarr(i)
%                 x = p(1,idbc); y = p(2,idbc);
%                 Rq{ibc}(idbc+(i-1)*numdbc) = eval(dirichlet{ibc}.r{i});
%             else
%                 Rq{ibc}(idbc+(i-1)*numdbc) = dirichlet{ibc}.r{i};
%             end
%         end
%     end
% end

function [N] = shape(L, nodes)
N = [(1-L)/2, (1+L)/2];