function [Ke, Fe] = pde_assem1(femOPT,mesh,subdomain,Kcoeff,Fcoeff)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 
p = mesh.p;
numsubdomain = length(unique(mesh.t(4,:)));
if not(isempty(subdomain))
    t = [];
    for i = 1:length(subdomain)
        t = [t, mesh.t(:,find(mesh.t(4,:) == subdomain(i)))];
    end
else
    subdomain = unique(mesh.t(4,:));
    t = mesh.t;
end
n = size(Fcoeff,1);
if not(iscell(Kcoeff))
    Kcoeff = num2cell(Kcoeff);
end
if not(iscell(Fcoeff))
    Fcoeff = num2cell(Fcoeff);
end
switch femOPT.codes
    case 'HB' % use "Home-brew" codes
        [Ke, Fe] = assem1(p,t,Kcoeff,Fcoeff);
    case 'PDETB' % use "PDE Toolbox" codes
        switch n
            case {1} % all 'TH' cases
                for i = 1:2
                    for j = 1:2
                        clear tmp0;
                        if ischar(Kcoeff{i,j})
                            tmp0(1) = '0';
                            for k = 2:length(Kcoeff{i,j})
                                tmp0(k) = ' ';
                            end
                        else
                            tmp0 = 0;
                        end
                        clear Ktmp;
                        for k = 1:4
                            Ktmp(k,:) = tmp0;
                        end
                        Ktmp((j-1)*2+i,:) = Kcoeff{i,j};
                        % calculate all K
                        Ke{i,j} = assema(p,t,Ktmp,0,0);
                    end
                end
                [tmp1, Ke{3,3}] = assema(p,t,[0;0;0;0],Kcoeff{3,3},0);
                D =  assem1(p,t,{0,0,Kcoeff{1,3};0,0,Kcoeff{2,3};Kcoeff{3,1},Kcoeff{3,2},0},{0});
                Ke{3,1} = D{3,1}; Ke{3,2} = D{3,2};
                Ke{1,3} = D{1,3}; Ke{2,3} = D{2,3};
                % calculate all F
                [tmp1,tmp2,Fe{1}] = assema(p,t,[0;0;0;0],0,Fcoeff{1});
            case {2} % all 'LE' cases
                Cmap = [1,3,9,11;2,4,10,12;5,7,13,15;6,8,14,16];
                % calculate all K
                for i = 1:4
                    for j = 1:4
                        clear tmp0;
                        if ischar(Kcoeff{i,j})
                            tmp0(1) = '0';
                            for k = 2:length(Kcoeff{i,j})
                                tmp0(k) = ' ';
                            end
                        else
                            tmp0 = 0;
                        end
                        clear Ctmp;
                        for k = 1:16
                            Ctmp(k,:) = tmp0;
                        end
                        Ctmp(Cmap(i,j),:) = Kcoeff{i,j};
                        Ke{i,j} = assema(p,t,Ctmp,[0;0;0;0],[0;0]);
                    end
                end
                for i = 1:2
                    for j = 1:2
                        clear tmp0;
                        if ischar(Kcoeff{4+i,4+j})
                            tmp0(1) = '0';
                            for k = 2:length(Kcoeff{4+i,4+j})
                                tmp0(k) = ' ';
                            end
                        else
                            tmp0 = 0;
                        end
                        clear Ktmp;
                        for k = 1:4
                            Ktmp(k,:) = tmp0;
                        end
                        Ktmp((j-1)*2+i,:) = Kcoeff{i+4,j+4};
                        [tmp1, Ke{i+4,j+4}] = assema(p,t,[0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0],Ktmp,[0;0]);                        
                    end
                end
                D = assem1(p,t,{0,0,0,0,Kcoeff{1,5},Kcoeff{1,6};0,0,0,0,Kcoeff{2,5},Kcoeff{2,6};...
                                0,0,0,0,Kcoeff{3,5},Kcoeff{3,6};0,0,0,0,Kcoeff{4,5},Kcoeff{4,6};...
                                Kcoeff{5,1},Kcoeff{5,2},Kcoeff{5,3},Kcoeff{5,4},0,0;...
                                Kcoeff{6,1},Kcoeff{6,2},Kcoeff{6,3},Kcoeff{6,4},0,0},{0;0}); 
                for i = 1:4
                    for j = 5:6
                        Ke{j,i} = D{j,i}; Ke{i,j} = D{i,j};
                    end
                end
                for i = 1:2
                    if ischar(Fcoeff{i})
                        clear tmp0;
                        tmp0(1) = '0';
                        for k = 2:length(Fcoeff{i})
                            tmp0(k) = ' ';
                        end
                    else
                        tmp0 = 0;
                    end
                    clear Ftmp;
                    for k = 1:2
                        Ftmp(k,:) = tmp0;
                    end
                    Ftmp(i,:) = Fcoeff{i};
                    % calculate all F
                    [tmp1,tmp2,Fe{i}] = assema(p,t,[0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0],[0;0;0;0],Ftmp);
                end
        end
    case 'COMSOL'
        switch n
            case 1
                % get all "stiff" Kq
                zeromat = sparse(size(mesh.p,2),size(mesh.p,2));
                zerovec = zeros(size(mesh.p,2),1);
                clear fem;
                fem.shape = 1;
                fem.mesh = mesh;
                fem.equ.ind = 1:numsubdomain;
                fem.sdim = {'x', 'y'};
                fem.dim = 1;
                for is = 1:numsubdomain
                    fem.equ.f{is} = 0;
                    fem.equ.c{is} = {{0,0;0,0}};
                    fem.equ.a{is} = 0;
                    fem.equ.be{is} = {{0,0}};
                end
                for i = 1:2
                    for j = 1:2
                        if (strcmp(Kcoeff{i,j},'0')) || (not(ischar(Kcoeff{i,j})) && Kcoeff{i,j} == 0)
                            Ke{i,j} = zeromat;
                        else
                            femtmp = fem;
                            for is = 1:numsubdomain
                                if ismember(is,subdomain)
                                    femtmp.equ.c{is}{1}{i,j} = Kcoeff{i,j};
                                end
                            end
                            femtmp.xmesh = meshextend(femtmp);
                            Ke{i,j} = assemble(femtmp,'Out','K');
                            tmpout = xmeshinfo(femtmp.xmesh);
                            [tmp,Cmap] = sort(tmpout.nodes);
                            Ke{i,j} = Ke{i,j}(Cmap,Cmap);
                        end
                    end
                    % get a pair of "cross" Kq
                    if strcmp(Kcoeff{3,i},'0') || (not(ischar(Kcoeff{3,i})) && Kcoeff{3,i} == 0)
                        Ke{3,i} = zeromat;
                    else
                        femtmp = fem;
                        for is = 1:numsubdomain
                            if ismember(is,subdomain)
                                femtmp.equ.be{is}{1}{i} = Kcoeff{3,i};
                            end
                        end
                        femtmp.xmesh = meshextend(femtmp);
                        Ke{3,i} = assemble(femtmp,'Out','K');
                        tmpout = xmeshinfo(femtmp.xmesh);
                        [tmp,Cmap] = sort(tmpout.nodes);
                        Ke{3,i} = Ke{3,i}(Cmap,Cmap);
                    end
                    if strcmp(Kcoeff{i,3},'0') || (not(ischar(Kcoeff{i,3})) && Kcoeff{i,3} == 0)
                        Ke{i,3} = zeromat;
                    else
                        femtmp = fem;
                        for is = 1:numsubdomain
                            if ismember(is,subdomain)
                                femtmp.equ.be{is}{1}{i} = Kcoeff{i,3};
                            end
                        end
                        femtmp.xmesh = meshextend(femtmp);
                        Ke{i,3} = assemble(femtmp,'Out','K');
                        tmpout = xmeshinfo(femtmp.xmesh);
                        [tmp,Cmap] = sort(tmpout.nodes);
                        Ke{i,3} = transpose(Ke{i,3}(Cmap,Cmap));
                    end
                end
                % get Mq
                if strcmp(Kcoeff{3,3},'0') || (not(ischar(Kcoeff{3,3})) && Kcoeff{3,3} == 0)
                    Ke{3,3} = zeromat;
                else
                    femtmp = fem;
                    for is = 1:numsubdomain
                        if ismember(is,subdomain)
                            femtmp.equ.a{is} = Kcoeff{3,3};
                        end
                    end
                    femtmp.xmesh = meshextend(femtmp);
                    Ke{3,3} = assemble(femtmp,'Out','K');
                    tmpout = xmeshinfo(femtmp.xmesh);
                    [tmp,Cmap] = sort(tmpout.nodes);
                    Ke{3,3} = Ke{3,3}(Cmap,Cmap);
                end
                % get Fq
                if strcmp(Fcoeff{1},'0') || (not(ischar(Fcoeff{1})) && Fcoeff{1} == 0)
                    Fe{1} = zerovec;
                else
                    femtmp = fem;
                    for is = 1:numsubdomain
                        if ismember(is,subdomain)
                            femtmp.equ.f{is} = Fcoeff{1};
                        end
                    end
                    femtmp.xmesh = meshextend(femtmp);
                    Fe{1} = assemble(femtmp,'Out','L');
                    tmpout = xmeshinfo(femtmp.xmesh);
                    [tmp,Cmap] = sort(tmpout.nodes);
                    Fe{1} = Fe{1}(Cmap);
                end
             case 2
                % get all "stiff" Kq
                zeromat = sparse(size(mesh.p,2)*2,size(mesh.p,2)*2);
                zerovec = zeros(size(mesh.p,2)*2,1);
                clear fem;
                fem.shape = 1;
                fem.mesh = mesh;
                fem.equ.ind = 1:numsubdomain;
                fem.sdim = {'x', 'y'};
                fem.dim = 2;
                for is = 1:numsubdomain
                    fem.equ.f{is} = {0,0};
                    fem.equ.c{is} = {{0,0;0,0},{0,0;0,0};{0,0;0,0},{0,0;0,0}};
                    fem.equ.a{is} = {0,0;0,0};
                    fem.equ.be{is} = {{0,0},{0,0};{0,0},{0,0}};
                end
                cmap = {{{1,1},{1,2};{2,1},{2,2}},{{1,3},{1,4};{2,3},{2,4}};{{3,1},{3,2};{4,1},{4,2}},{{3,3},{3,4};{4,3},{4,4}}};
                amap = {{5,5},{5,6};{6,5},{6,6}};
                bemap = {{{5,1},{5,2}},{{5,3},{5,4}};{{6,1},{6,2}},{{6,3},{6,4}}};
                for i1 = 1:2
                    for j1 = 1:2
                        for i2 = 1:2
                            for j2 = 1:2
                                if strcmp(Kcoeff{cmap{i1,j1}{i2,j2}{1},cmap{i1,j1}{i2,j2}{2}},'0')  || (not(ischar(Kcoeff{cmap{i1,j1}{i2,j2}{1},cmap{i1,j1}{i2,j2}{2}})) && Kcoeff{cmap{i1,j1}{i2,j2}{1},cmap{i1,j1}{i2,j2}{2}} == 0)
                                    Ke{cmap{i1,j1}{i2,j2}{1},cmap{i1,j1}{i2,j2}{2}} = zeromat;
                                else
                                    femtmp = fem;
                                    for is = 1:numsubdomain
                                        if ismember(is,subdomain)
                                            femtmp.equ.c{is}{i1,j1}{i2,j2} = Kcoeff{cmap{i1,j1}{i2,j2}{1},cmap{i1,j1}{i2,j2}{2}};
                                        end
                                    end
                                    femtmp.xmesh = meshextend(femtmp);
                                    Ke{cmap{i1,j1}{i2,j2}{1},cmap{i1,j1}{i2,j2}{2}} = assemble(femtmp,'Out','K');
                                    tmpout = xmeshinfo(femtmp.xmesh);
                                    [tmp,Cmap] = sort(tmpout.nodes);
                                    Ke{cmap{i1,j1}{i2,j2}{1},cmap{i1,j1}{i2,j2}{2}} = Ke{cmap{i1,j1}{i2,j2}{1},cmap{i1,j1}{i2,j2}{2}}(Cmap,Cmap);
                                end
                            end
                            % get a pair of "cross" Kq
                            if strcmp(Kcoeff{bemap{i1,j1}{i2}{1},bemap{i1,j1}{i2}{2}},'0') || (not(ischar(Kcoeff{bemap{i1,j1}{i2}{1},bemap{i1,j1}{i2}{2}})) && Kcoeff{bemap{i1,j1}{i2}{1},bemap{i1,j1}{i2}{2}} == 0)
                                Ke{bemap{i1,j1}{i2}{1},bemap{i1,j1}{i2}{2}} = zeromat;
                            else
                                femtmp = fem;
                                for is = 1:numsubdomain
                                    if ismember(is,subdomain)
                                        femtmp.equ.be{is}{i1,j1}{i2} = Kcoeff{bemap{i1,j1}{i2}{1},bemap{i1,j1}{i2}{2}};
                                    end
                                end
                                femtmp.xmesh = meshextend(femtmp);
                                Ke{bemap{i1,j1}{i2}{1},bemap{i1,j1}{i2}{2}} = assemble(femtmp,'Out','K');
                                tmpout = xmeshinfo(femtmp.xmesh);
                                [tmp,Cmap] = sort(tmpout.nodes);
                                Ke{bemap{i1,j1}{i2}{1},bemap{i1,j1}{i2}{2}} = Ke{bemap{i1,j1}{i2}{1},bemap{i1,j1}{i2}{2}}(Cmap,Cmap);
                            end
                            if strcmp(Kcoeff{bemap{i1,j1}{i2}{2},bemap{i1,j1}{i2}{1}},'0')  || (not(ischar(Kcoeff{bemap{i1,j1}{i2}{2},bemap{i1,j1}{i2}{1}})) && Kcoeff{bemap{i1,j1}{i2}{2},bemap{i1,j1}{i2}{1}} == 0)
                                Ke{bemap{i1,j1}{i2}{2},bemap{i1,j1}{i2}{1}} = zeromat;
                            else
                                femtmp = fem;
                                for is = 1:numsubdomain
                                    if ismember(is,subdomain)
                                        femtmp.equ.be{is}{i1,j1}{i2} = Kcoeff{bemap{i1,j1}{i2}{2},bemap{i1,j1}{i2}{1}};
                                    end
                                end
                                femtmp.xmesh = meshextend(femtmp);
                                Ke{bemap{i1,j1}{i2}{2},bemap{i1,j1}{i2}{1}} = assemble(femtmp,'Out','K');
                                tmpout = xmeshinfo(femtmp.xmesh);
                                [tmp,Cmap] = sort(tmpout.nodes);
                                Ke{bemap{i1,j1}{i2}{2},bemap{i1,j1}{i2}{1}} = transpose(Ke{bemap{i1,j1}{i2}{2},bemap{i1,j1}{i2}{1}}(Cmap,Cmap));
                            end
                        end
                        % get Mq
                        if strcmp(Kcoeff{amap{i1,j1}{1},amap{i1,j1}{2}},'0') || (not(ischar(Kcoeff{amap{i1,j1}{1},amap{i1,j1}{2}})) && Kcoeff{amap{i1,j1}{1},amap{i1,j1}{2}} == 0)
                            Ke{amap{i1,j1}{1},amap{i1,j1}{2}} = zeromat;
                        else
                            femtmp = fem;
                            for is = 1:numsubdomain
                                if ismember(is,subdomain)
                                    femtmp.equ.a{is}{i1,j1} = Kcoeff{amap{i1,j1}{1},amap{i1,j1}{2}};
                                end
                            end
                            femtmp.xmesh = meshextend(femtmp);
                            Ke{amap{i1,j1}{1},amap{i1,j1}{2}} = assemble(femtmp,'Out','K');
                            tmpout = xmeshinfo(femtmp.xmesh);
                            [tmp,Cmap] = sort(tmpout.nodes);
                            Ke{amap{i1,j1}{1},amap{i1,j1}{2}} = Ke{amap{i1,j1}{1},amap{i1,j1}{2}}(Cmap,Cmap);
                        end
                    end
                    % get Fq
                    if strcmp(Fcoeff{i1},'0') || (not(ischar(Fcoeff{1})) && Fcoeff{i1} == 0)
                        Fe{i1} = zerovec;
                    else
                        femtmp = fem;
                        for is = 1:numsubdomain
                            if ismember(is,subdomain)
                                femtmp.equ.f{is}{i1} = Fcoeff{i1};
                            end
                        end
                        femtmp.xmesh = meshextend(femtmp);
                        Fe{i1} = assemble(femtmp,'Out','L');
                        tmpout = xmeshinfo(femtmp.xmesh);
                        [tmp,Cmap] = sort(tmpout.nodes);
                        Fe{i1} = Fe{i1}(Cmap);
                    end
                end
                % remap!
                Cmap = [1:2:size(mesh.p,2)*2-1,2:2:size(mesh.p,2)*2];
                for i = 1:6
                    for j = 1:6
                        Ke{i,j} = Ke{i,j}(Cmap,Cmap);
                    end
                end
                for i = 1:2
                    Fe{i} = Fe{i}(Cmap);
                end                
        end
end
