function [Qe,Ge,He,Re] = pde_assem2(femOPT,mesh,n,neumann,dirichlet)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 

p = mesh.p; e = mesh.e;
if isempty(neumann)
    Qe = []; Ge = [];
end
if isempty(dirichlet)
    He{1} = []; Re{1} = []; % phuong 25-9
end
numedge = length(unique(mesh.e(5,:)));
switch femOPT.codes
    case 'HB' % use "Home-brew" codes
        [Qe,Ge,He,Re] = assem2(p,e,n,neumann,dirichlet);
    case 'PDETB' % use "PDE Toolbox" codes
        switch n
            case {1} % all 'TH' cases
                % assemble all homogeneous Neumann bc
                for ie = 1:numedge
                    tmp = [1 0 1 1 '0' '0' '0' '0' '0' '0']';
                    pdebc0(1:length(tmp),ie) = tmp;
                end
                % assemble Neumann bc
                for ibc = 1:length(neumann)
                    % q
                    tmp1 = regexprep(char(sym(neumann{ibc}.q{1})),{'*','\^'},{'.*','.^'});
                    tmp = [1 0 length(tmp1) 1 tmp1 '0' '0' '0' '0' '0']';
                    pdebctmp = pdebc0;
                    pdebctmp(1:length(tmp),neumann{ibc}.edge) = tmp;
                    Qe{ibc}{1,1} = assemb(pdebctmp,p,e);
                    % g
                    tmp1 = regexprep(char(sym(neumann{ibc}.g{1})),{'*','\^'},{'.*','.^'});
                    tmp = [1 0 1 length(tmp1) '0' tmp1 '0' '0' '0' '0']';
                    pdebctmp = pdebc0;
                    pdebctmp(1:length(tmp),neumann{ibc}.edge) = tmp;
                    [tmp,Ge{ibc}{1}] = assemb(pdebctmp,p,e);
                end
                % assemble Dirichlet bc
                for ibc = 1:length(dirichlet)
                    pdebctmp = pdebc0;
                    for ie = 1:length(dirichlet{ibc})
                        % length(dirichlet{ibc}.h) is always equal to 1
                        tmp1 = regexprep(char(sym(dirichlet{ibc}{ie}.h{1}{1})),{'*','\^'},{'.*','.^'});
                        tmp2 = regexprep(char(sym(dirichlet{ibc}{ie}.r{1})),{'*','\^'},{'.*','.^'});
                        tmp = [1 1 1 1 length(tmp1) length(tmp2) '0' '0' tmp1 tmp2]';
                        pdebctmp(1:length(tmp),dirichlet{ibc}{ie}.edge) = tmp;
                    end
                    [tmp1,tmp2,He{ibc},Re{ibc}] = assemb(pdebctmp,p,e);
                end
            case {2} % all 'LE' cases
                % assemble all homogeneous Neumann bc
                for ie = 1:numedge
                    tmp = [2 0 1 1 1 1 1 1 '0' '0' '0' '0' '0' '0']';
                    pdebc0(1:length(tmp),ie) = tmp;
                end
                 % assemble Neumann bc
                for ibc = 1:length(neumann)
                    % q
                    tmp1 = regexprep(char(sym(neumann{ibc}.q{1,1})),{'*','\^'},{'.*','.^'});
                    tmp = [2 0 length(tmp1) 1 1 1 1 1 tmp1 '0' '0' '0' '0' '0']';
                    pdebctmp = pdebc0;
                    pdebctmp(1:length(tmp),neumann{ibc}.edge) = tmp;
                    Qe{ibc}{1,1} = assemb(pdebctmp,p,e);
                    
                    tmp1 = regexprep(char(sym(neumann{ibc}.q{2,1})),{'*','\^'},{'.*','.^'});
                    tmp = [2 0 1 length(tmp1) 1 1 1 1 '0' tmp1 '0' '0' '0' '0']';
                    pdebctmp = pdebc0;
                    pdebctmp(1:length(tmp),neumann{ibc}.edge) = tmp;
                    Qe{ibc}{2,1} = assemb(pdebctmp,p,e);
                    
                    tmp1 = regexprep(char(sym(neumann{ibc}.q{1,2})),{'*','\^'},{'.*','.^'});
                    tmp = [2 0 1 1 length(tmp1) 1 1 1 '0' '0' tmp1 '0' '0' '0']';
                    pdebctmp = pdebc0;
                    pdebctmp(1:length(tmp),neumann{ibc}.edge) = tmp;
                    Qe{ibc}{1,2} = assemb(pdebctmp,p,e);
                    
                    tmp1 = regexprep(char(sym(neumann{ibc}.q{2,2})),{'*','\^'},{'.*','.^'});
                    tmp = [2 0 1 1 1 length(tmp1) 1 1 '0' '0' '0' tmp1 '0' '0']';
                    pdebctmp = pdebc0;
                    pdebctmp(1:length(tmp),neumann{ibc}.edge) = tmp;
                    Qe{ibc}{2,2} = assemb(pdebctmp,p,e);
                    
                    % g
                    tmp1 = regexprep(char(sym(neumann{ibc}.g{1})),{'*','\^'},{'.*','.^'});
                    tmp = [2 0 1 1 1 1 length(tmp1) 1 '0' '0' '0' '0' tmp1 '0']';
                    pdebctmp = pdebc0;
                    pdebctmp(1:length(tmp),neumann{ibc}.edge) = tmp;
                    [tmp,Ge{ibc}{1}] = assemb(pdebctmp,p,e);
                    
                    tmp1 = regexprep(char(sym(neumann{ibc}.g{2})),{'*','\^'},{'.*','.^'});
                    tmp = [2 0 1 1 1 1 1 length(tmp1) '0' '0' '0' '0' '0' tmp1]';
                    pdebctmp = pdebc0;
                    pdebctmp(1:length(tmp),neumann{ibc}.edge) = tmp;
                    [tmp,Ge{ibc}{2}] = assemb(pdebctmp,p,e);
                end
                % assemble Dirichlet bc
                for ibc = 1:length(dirichlet)
                    pdebctmp = pdebc0;
                    for ie = 1:length(dirichlet{ibc})
                        switch length(dirichlet{ibc}{ie}.h)
                            case 1 % 1 dirichlet bc
                                tmp1 = regexprep(char(sym(dirichlet{ibc}{ie}.h{1}{1})),{'*','\^'},{'.*','.^'});
                                tmp2 = regexprep(char(sym(dirichlet{ibc}{ie}.h{1}{2})),{'*','\^'},{'.*','.^'});
                                tmp3 = regexprep(char(sym(dirichlet{ibc}{ie}.r{1})),{'*','\^'},{'.*','.^'});
                                tmp = [2 1 1 1 1 1 1 1 length(tmp1) length(tmp2) length(tmp3) '0' '0' '0' '0' '0' '0' tmp1 tmp2 tmp3]';
                                pdebctmp(1:length(tmp),dirichlet{ibc}{ie}.edge) = tmp;
                            case 2 % 2 dirichlet bc
                                tmp1 = regexprep(char(sym(dirichlet{ibc}{ie}.h{1}{1})),{'*','\^'},{'.*','.^'});
                                tmp2 = regexprep(char(sym(dirichlet{ibc}{ie}.h{2}{1})),{'*','\^'},{'.*','.^'});
                                tmp3 = regexprep(char(sym(dirichlet{ibc}{ie}.h{1}{2})),{'*','\^'},{'.*','.^'});
                                tmp4 = regexprep(char(sym(dirichlet{ibc}{ie}.h{2}{2})),{'*','\^'},{'.*','.^'});
                                tmp5 = regexprep(char(sym(dirichlet{ibc}{ie}.r{1})),{'*','\^'},{'.*','.^'});
                                tmp6 = regexprep(char(sym(dirichlet{ibc}{ie}.r{2})),{'*','\^'},{'.*','.^'});
                                tmp = [2 2 1 1 1 1 1 1 length(tmp1) length(tmp2) length(tmp3) length(tmp4) length(tmp5) length(tmp6) '0' '0' '0' '0' '0' '0' tmp1 tmp2 tmp3 tmp4 tmp5 tmp6]';
                                pdebctmp(1:length(tmp),dirichlet{ibc}{ie}.edge) = tmp;
                        end
                    end
                    [tmp1,tmp2,He{ibc},Re{ibc}] = assemb(pdebctmp,p,e);
                end
        end
    case 'COMSOL'
        switch n
            case {1} % all 'TH' cases
                zeromat = sparse(size(mesh.p,2),size(mesh.p,2));
                zerovec = zeros(size(mesh.p,2),1);
                clear fem;
                fem.shape = 1;
                fem.mesh = mesh;
                fem.sdim = {'x', 'y'};
                fem.dim = 1;
                fem.equ.a = 0;
                fem.equ.c = 0;
                fem.equ.f = 0;
                for ie = 1:numedge
                    fem.bnd.q{ie} = {0};
                    fem.bnd.g{ie} = {0};
                    fem.bnd.h{ie} = {0};
                    fem.bnd.r{ie} = {0};
                end
                for ibc = 1:length(neumann)
                    if strcmp(char(sym(neumann{ibc}.q{1})),'0')
                        Qe{ibc}{1,1} = zeromat;
                    else
                        femtmp = fem;
                        for ie = 1:numedge
                            if ismember(ie,neumann{ibc}.edge)
                                femtmp.bnd.q{ie} = char(sym(neumann{ibc}.q{1}));
                            end
                        end
                        femtmp.xmesh = meshextend(femtmp);
                        Qe{ibc}{1,1} = assemble(femtmp,'Out','K');
                        tmpout = xmeshinfo(femtmp.xmesh);
                        [tmp,Cmap] = sort(tmpout.nodes);
                        Qe{ibc}{1,1} = Qe{ibc}{1,1}(Cmap,Cmap);
                    end
                    if strcmp(char(sym(neumann{ibc}.g{1})),'0')
                        Ge{ibc}{1} = zerovec;
                    else
                        femtmp = fem;
                        for ie = 1:numedge
                            if ismember(ie,neumann{ibc}.edge)
                                femtmp.bnd.g{ie} = char(sym(neumann{ibc}.g{1}));
                            end
                        end
                        femtmp.xmesh = meshextend(femtmp);
                        Ge{ibc}{1} = assemble(femtmp,'Out','L');
                        tmpout = xmeshinfo(femtmp.xmesh);
                        [tmp,Cmap] = sort(tmpout.nodes);
                        Ge{ibc}{1} = Ge{ibc}{1}(Cmap);
                    end
                end
                for ibc = 1:length(dirichlet)
                    femtmp = fem;
                    for ine = 1:length(dirichlet{ibc})
                        for ie = 1:numedge
                            if ismember(ie,dirichlet{ibc}{ine}.edge)
                                femtmp.bnd.h{ie} = char(sym(dirichlet{ibc}{ine}.h{1}{1}));
                                femtmp.bnd.r{ie} = char(sym(dirichlet{ibc}{ine}.r{1}));
                            end
                        end
                    end
                    femtmp.xmesh = meshextend(femtmp);
                    [He{ibc},Re{ibc}] = assemble(femtmp,'Out',{'N' 'M'});
                    tmpout = xmeshinfo(femtmp.xmesh);
                    [tmp,Cmap] = sort(tmpout.nodes);
                    He{ibc} = He{ibc}(:,Cmap);
                end
            case {2} % all 'LE' cases
                zeromat = sparse(size(mesh.p,2)*2,size(mesh.p,2)*2);
                zerovec = zeros(size(mesh.p,2)*2,1);
                clear fem;
                fem.shape = 1;
                fem.mesh = mesh;
                fem.sdim = {'x', 'y'};
                fem.dim = 2;
                fem.equ.a = 0;
                fem.equ.c = 0;
                fem.equ.f = 0;
                for ie = 1:numedge
                    fem.bnd.q{ie} = {0,0;0,0};
                    fem.bnd.g{ie} = {0,0};
                    fem.bnd.h{ie} = {0,0};
                    fem.bnd.r{ie} = {0};
                end
                for ibc = 1:length(neumann)
                    for i = 1:2
                        for j = 1:2
                            if strcmp(char(sym(neumann{ibc}.q{i,j})),'0')
                                Qe{ibc}{i,j} = zeromat;
                            else
                                femtmp = fem;
                                for ie = 1:numedge
                                    if ismember(ie,neumann{ibc}.edge)
                                        femtmp.bnd.q{ie}{i,j} = char(sym(neumann{ibc}.q{i,j}));
                                    end
                                end
                                femtmp.xmesh = meshextend(femtmp);
                                Qe{ibc}{i,j} = assemble(femtmp,'Out','K');
                                tmpout = xmeshinfo(femtmp.xmesh);
                                [tmp,Cmap] = sort(tmpout.nodes);
                                Qe{ibc}{i,j} = Qe{ibc}{i,j}(Cmap,Cmap);
                            end
                        end
                        if strcmp(char(sym(neumann{ibc}.g{i})),'0')
                            Ge{ibc}{i} = zerovec;
                        else
                            femtmp = fem;
                            for ie = 1:numedge
                                if ismember(ie,neumann{ibc}.edge)
                                    femtmp.bnd.g{ie}{i} = char(sym(neumann{ibc}.g{i}));
                                end
                            end
                            femtmp.xmesh = meshextend(femtmp);
                            Ge{ibc}{i} = assemble(femtmp,'Out','L');
                            tmpout = xmeshinfo(femtmp.xmesh);
                            [tmp,Cmap] = sort(tmpout.nodes);
                            Ge{ibc}{i} = Ge{ibc}{i}(Cmap);
                        end
                    end
                end
                for ibc = 1:length(dirichlet)
                    femtmp = fem;
                    for ine = 1:length(dirichlet{ibc})
                        for ie = 1:numedge
                            if ismember(ie,dirichlet{ibc}{ine}.edge)
                                switch length(dirichlet{ibc}{ine}.h)
                                    case 1
                                        femtmp.bnd.h{ie} = {char(sym(dirichlet{ibc}{ine}.h{1}{1})), char(sym(dirichlet{ibc}{ine}.h{1}{2}));0,0};
                                        femtmp.bnd.r{ie} = {char(sym(dirichlet{ibc}{ine}.r{1})),0};
                                    case 2
                                        femtmp.bnd.h{ie} = {char(sym(dirichlet{ibc}{ine}.h{1}{1})), char(sym(dirichlet{ibc}{ine}.h{1}{2}));...
                                                            char(sym(dirichlet{ibc}{ine}.h{2}{1})), char(sym(dirichlet{ibc}{ine}.h{2}{2}))};
                                        femtmp.bnd.r{ie} = {char(sym(dirichlet{ibc}{ine}.r{1})),char(sym(dirichlet{ibc}{ine}.r{2}))};
                                end
                            end
                        end
                    end
                    femtmp.xmesh = meshextend(femtmp);
                    [He{ibc},Re{ibc}] = assemble(femtmp,'Out',{'N' 'M'});
                    tmpout = xmeshinfo(femtmp.xmesh);
                    [tmp,Cmap] = sort(tmpout.nodes);
                    He{ibc} = He{ibc}(:,Cmap);
                end
                % remap!
                Cmap = [1:2:size(mesh.p,2)*2-1,2:2:size(mesh.p,2)*2];
                for ibc = 1:length(neumann)
                    for i = 1:2
                        for j = 1:2
                            Qe{ibc}{i,j} = Qe{ibc}{i,j}(Cmap,Cmap);
                        end
                        Ge{ibc}{i} = Ge{ibc}{i}(Cmap);
                    end
                end
                for ibc = 1:length(dirichlet)
                    He{ibc} = He{ibc}(:,Cmap);
                end
        end        
end