function [out_coeff, out_term, out_coeffc] = symvarsep(in_var)
% -----rbMIT Software Copyright MIT 2006-07----
%  DBP  Huynh,NC Nguyen, AT Patera, G Rozza
% Henceforth, Software shall refer to the rbMIT_System software package: 
% the contents of the rbMIT_Library and rbMIT_Aux folders.
% 
% This License governs use of all accompanying Software, and your download and/or use of the Software constitutes acceptance 
% of this License.
% 
% You may use this Software for any non-commercial purpose, subject to the restrictions in this license. 
% Some purposes which can be non-commercial are teaching and academic research.
% 
% You may not use or distribute this Software or any derivative works in any form for commercial purposes. 
% Examples of commercial purposes would be licensing, leasing, or selling the Software, or distributing the 
% Software for use with commercial products.
% 
% You may modify this Software and distribute the modified Software for non-commercial purposes,
%  however, you may not grant rights to the Software or derivative works that are broader than those 
% provided by this License. For example, you may not distribute modifications of the Software under terms 
% that would permit commercial use.
% 
% In return, we require that you agree:
% 
% not to remove any copyright or other notices from the Software;
% 
% that if you distribute the Software in source or object form, you will include a verbatim copy of this license;
% 
% that if you distribute derivative works of the Software in source code form you do so only under a license
% that includes all of the provisions of this License, and if you distribute derivative works of the Software solely 
% in object form you do so only under a license that complies with this License;
% 
% that if you have modified the Software or created derivative works, and distribute such modifications or derivative
% works, you will cause the modified files to carry prominent notices so that recipients know that they are not receiving the original Software;
% 
% THAT THE SOFTWARE COMES "AS IS", WITH NO WARRANTIES: THIS MEANS NO EXPRESS, IMPLIED OR STATUTORY WARRANTY, 
% INCLUDING WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR ANY WARRANTY 
% OF TITLE OR NON-INFRINGEMENT; YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% THAT NEITHER MIT NOR THE AUTHORS WILL BE LIABLE FOR ANY DAMAGES RELATED TO THE SOFTWARE OR THIS LICENSE, 
% INCLUDING DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL OR INCIDENTAL DAMAGES; YOU MUST PASS THIS LIMITATION OF LIABILITY 
% ON WHENEVER YOU DISTRIBUTE THE SOFTWARE OR DERIVATIVE WORKS;
% 
% that your rights under the License end automatically if you breach the License in any way;
% 
% that MIT reserves all rights not expressly granted to you in this License.
% 
% (Note: The wording of this license agreement is derived from a Microsoft Shared Source license agreement.)
% 
% License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
%
% rbMIT Software Part_II_and_IV
%
% ---------- 

% SYMVARSEP Separation of polynomial symbolic terms
% [out_coeff, out_term, out_coeffc] = symvarsep(in_var) returns 
% a set of out_coeff and out_term such that
%            in_var = sum(out_term*out_coeff)
% where out_term are polynomical function of "x" and "y"
% and out_coeff are function of "mus", "Cfun" and "Gfun"
% "Cfun" and "Gfun" are mapping coefficients that depends on "mus"
%        x_out = Cfun1 + x_in*Gfun11 + y_in*Gfun12
%        y_out = Cfun2 + x_in*Gfun21 + y_in*Gfun22
% out_coeffc are "char" version of out_coeff
%
% Example:
% syms mu1 x y;
% E{1} = mu1 + x + y;
% [out_coeff, out_term] = symvarsep(E)
%
% out_coeff = [1, x, y];
% out_term{1} = mu1 + Cfun1 + Cfun2
% out_term{2} = Gfun21+Gfun11
% out_term{3} = Gfun22+Gfun12

issingleoutput = 0;
if not(iscell(in_var))
    tmp = in_var; clear in_var;
    in_var{1} = tmp;
    issingleoutput = 1;
end

for i = length(in_var)
    tmp = regexprep(findsym(in_var{i}),',',' ');
    if not(strcmp(tmp,''))
        eval(['syms ',tmp,';']);
    end
end
syms x y xbar ybar Cfun1 Cfun2 Gfun11 Gfun12 Gfun21 Gfun22;

[size1, size2] = size(in_var{1});

for idom = 1:length(in_var)
    in_vartmp{idom} = sym(subs(in_var{idom},{x,y},{Cfun1+Gfun11*xbar+Gfun12*ybar,Cfun2+Gfun21*xbar+Gfun22*ybar}));
    for i = 1:size1*size2
        tmp = expand(in_vartmp{idom}(i));
        [tmp_coeff1,tmp_term1] = symcoeffs(tmp,xbar); % phuong 2/2/09
        clear tmp_term2 tmp_coeff2;
        for ii = 1:length(tmp_coeff1)
            [tmp1,tmp2] = symcoeffs(tmp_coeff1(ii),ybar); % phuong 2/2/09
            tmp_coeff2{ii} = tmp1;
            tmp_term2{ii} = tmp2;
        end
        clear tmp1 tmp2; count = 0;
        for ii = 1:length(tmp_coeff1)
            for jj = 1:length(tmp_coeff2{ii})
                count = count + 1;
                tmp2(count) = tmp_term1(ii)*tmp_term2{ii}(jj);
                tmp1(count) = tmp_coeff2{ii}(jj);
            end
        end
        tmp_coeff{idom}{i} = tmp2;
        tmp = zeros(size1,size2); tmp(i) = 1;
        for ii = 1:length(tmp2)
            tmp_term{idom}{i}{ii} = tmp1(ii)*tmp;
        end
    end
    count = 0;
    for i = 1:size1*size2
        for ii = 1:length(tmp_coeff{idom}{i})
            count = count + 1;
            tmp_coeffc{idom}{count} = char(sym(tmp_coeff{idom}{i}(ii))); % phuong 2/2/09
        end
    end
end
tmp = tmp_coeffc;
out_coeffc = tmp{1};
for idom = 2:length(in_var)
    out_coeffc = [out_coeffc, tmp{idom}];
end

out_coeffc = unique(out_coeffc);
for idom = 1:length(in_var)
    for i = 1:length(out_coeffc)
        out_term{idom}{i} = zeros(size1,size2);
    end
    for i = 1:size1*size2
        for ii = 1:length(tmp_coeff{idom}{i})
            [tmp,tmploc] = ismember(char(sym(tmp_coeff{idom}{i}(ii))),out_coeffc); % phuong 2/2/09
            if tmp
                out_term{idom}{tmploc} = out_term{idom}{tmploc} + tmp_term{idom}{i}{ii};
            end
        end
    end
end

clear out_coeff;
for i = 1:length(out_coeffc)
    out_coeff(i) = sym(out_coeffc{i});
end
if not(isempty(out_coeff)) % phuong 2/2/09
    out_coeff = simplify(sym(subs(out_coeff,{xbar,ybar},{x,y}))); % hotfix!
end

if issingleoutput
    tmp = out_term; clear out_term;
    out_term = tmp{1};
end