
clear all;
% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%
probname = 'cantilever2'
% Problem: This problem corresponds to a curved cantilever beam clamped at one side 
% under a uniform pressure at the free end.
% mu1: thickness of the beam
probtype = 'LE2';
points = '[1-mu1,0;0,1-mu1;0,1+mu1;1+mu1,0]';
edge = [1,2;2,3;4,3;4,1];
geometry{1} = [1,2,3,4];
gflag = [1];
% material properties, Young modulus and Poisson ratio
matprop{1} = '[1, 3/10]';
muref = [0.8];
mu_min = [0.7];
mu_max = [0.9];
mu_bar = [0.8];
% nload: edge fn ft
% ft is tangential force (aligned with the en, direction is from p1 to p2)
% fn is normal force (cross(en,et)>0))
nload = '[2,1,0]';
% dirichlet: edge lab val
% lab can be "x" and "y"
dirichlet = '[4,x,0;4,y,0]';
curvedat = '[0,0,1-mu1,1-mu1,0,cos(t),sin(t);0,0,1+mu1,1+mu1,0,cos(t),sin(t)]';
tarclist = '[1,1,0,pi/2;3,2,0,pi/2]';

%Output: The output is the integral of the horizontal displacement along the side of 
% the free end. 
% The output results can be compared with the thick curved beam solution in Roark's book.
outputname = 'cantilever2_y'
oload = '[2,0,1]';
 
%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))