clear;

% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT ? MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%

% Consider time-dependent advection-diffusion in a rectangular channel characterized by the
% length of the channel and Peclet number. Velocity field U = y in x direction and 
% V = 0 in y direction. Heat flux is added in at the lower side. The left and top sides 
% of the channel are subject to zero temperature, while the right side is subject 
% to homogeneous Neumman condition. It simulates a simple scalar flow. 
% See rbU_advdiff.m for the steady version of this problem.

probname = 'tdepadvdiff'
probtype = 'TH';
femOPT.maxsize=500;
plotdemo = 0;

points = '[0,0;mu1,0;mu1,0.5;0,0.5]';
%mu1 is the length of channel
edge = [1,2;2,3;3,4;4,1];
geometry{1} = [1,2,3,4];
gflag = [1];
kappa{1} = '[1/mu2, 0, 0; 0, 1/mu2, 0; y, 0, 0]';
%mu2 is the Peclet number
muref = [1,1];
mu_min = [1,0.1];
mu_max = [10,10];
mu_bar = [1,1];
dirichlet = '[3,0;4,0]';
%zero Dirichlet BC on side 3 and 4
nload = '[1, 0, 0, 1; 2, 0, 0, 0]';
% Neumann on other side (non homogeneous on side 1 and zero on side 2)

outputname = 'layer'
oareaload = '[1,1]';

initsol = '[0]'; 
theta_u = '[1]';
% The temperature is initially set 0 degrees.
    
nt = 50;          % Number of timesteps  
dt = 0.05;        % Timestep size
% Finally, to complete the problem formulation, we specify that the starting time is 0 
% and that we want to study the heat distribution during 2.5 seconds.
 

%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))
%%%%%%%%%% End of the offline execution. 

% Now the user can perform the online stage to rapidly simulate the problem 
% for different parameter values. See the software documentation for details. 


