clear all;

% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%

% This example illustrates heat conduction through an elliptical buried object with
% high conductivity.

% We consider a square domain with an embedded elliptical object of minor axis a
% and major axis b. The conductivity of the embedded object is high compared to that of 
% the square region. The top surface of the square region is subject to a heat flux. 
% The bottom is kept at zero temperature. The rest boundaries are subject to homogeneous Neumman. 


probname = 'tdepbobject'
femOPT.maxsize = 500;
femOPT.refine = 'subdivision';
plotdemo=0;

points = '[0,0;0,-mu2;mu1,0;0,mu2;0,2;1,2;5,2;5,-5;0,-5]';
% mu1 and mu2 represent the two axes of a buried object
edge = [1,2;2,3;3,4;4,1;4,5;5,6;6,7;7,8;8,9;9,2];
geometry{1} = [1,2,3,4]; 
% A buried object with elliptical shape 
geometry{2} = [2,3,5,6,7,8,9,10]; 
% Square region
kappa{1} = '[mu3 0 0; 0 mu3 0; 0 0 0]'; 
% mu3 is conductivity of the buried object
kappa{2} = '[1 0 0; 0 1 0; 0 0 0]';
gflag = [1,1];
muref = [0.75,0.75,10];
mu_min = [0.5,0.5,5];
mu_max = [1.0,1.0,15];
mu_bar = [0.75,0.75,10];
curvedat = '[0,0,mu1,mu2,0,cos(t),sin(t)]';
tarclist = '[2,1,3*pi/2,2*pi;3,1,0,pi/2]';
% curvedat and tarclist describe the elliptical shape
nload = '[6,0,0,1;7,0,0,1]';
% Heat flux added in at the top surface
dirichlet='[9,0]';
% Temperature is kept zero at the bottom

outputname = 'strip'
oload = '[6,1]';
% Output is the average temperature over a strip [0,2] of the top surface

% zero initial condition
initsol='[0]';
theta_u='[1]';
% The temperature field is 0 degrees at the time we start applying heat flux.

nt = 200;         % Number of timesteps  
dt = 0.05;        % Timestep size
% Finally, to complete the problem formulation, we specify that the starting time is 0 
% and that we want to study the heat distribution during the first 10 seconds.
 

%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))
%%%%%%%%%% End of the offline execution. 

% Now the user can perform the online stage to rapidly simulate the problem 
% for different parameter values. See the software documentation for details. 

