
clear;
% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%

% This example illustates the convection and diffusion of the contaminant
% concentration in a short channel, where the velocity flows from left to right.
% Homogeneous Dirichlet BC is applied on the inflow boundary (the left of the channel), 
% while homogeneous Dirichlet BCs are applied on the remaining boundaries.
% The initial distribution of the contaminant concentration is assumed to
% be Gaussian and depends on a parameter mu2. The Peclet number is mu1.

probname = 'tdepchannel'
probtype = 'TH';
plotdemo = 0;
femOPT.maxsize=500;
femOPT.refine = 'subdivision';

points = '[0,0;2,0;2,1;0,1]';
% A rectangular channel of size 2 by 1
edge = [1,2;2,3;3,4;4,1];
geometry{1} = [1,2,3,4];
gflag = [1];
% operator is 1/mu1*du/dx*du/dx+ 1/mu1*du/dy*du/dy + Udu/dxv + Vdu/dyv
kappa{1} = '[1/mu1, 0, 0; 0, 1/mu1, 0; 1, 0, 0]';
% mu1 is the Peclet number
muref = [1,1];
mu_min = [0.1,0.5];
mu_max = [20,2];
mu_bar = [1,1];
dirichlet = '[4,0]';
% Zero Dirichlet BC on the inflow boundary

outputname = 'average'
oareaload = '[1,1]';
% Output is the average concentration over the whole domain

initsol = '[exp(-15*(x1-0.5).^2-30*(x2-0.5).^2)]'; 
theta_u = '[mu2]';
% The initial concentration is gaussian and dependent on mu2.

nt = 50;           % Number of timesteps  
dt = 0.02;         % Timestep size
% Finally, to complete the problem formulation, we specify that the starting time is 0 
% and that we want to study the heat distribution during the first second.

%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))
%%%%%%%%%% End of the offline execution. 

% Now the user can perform the online stage to rapidly simulate the problem 
% for different parameter values. See the software documentation for details. 

