clear all;
% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%

% This example illustrates heat conduction through a rectangular plate containing a very 
% thin crack. The crack is characterized by mu1 (the length) and mu2 (the location). 
% Heat flux is added in at the top surface, while the bottom is subject to zero temperature. 

probname = 'tdepcrack'
femOPT.maxsize = 500;
femOPT.refine = 'subdivision';
plotdemo=0;

points = '[0,.0001; mu1,0; 5,0; 5,mu2; 2,mu2; 0,mu2; 0,-.0001; 5,-5; 0,-5]';
% mu1 is the length of the crack 
% mu2 is the location of the crack from the top surface
edge = [1,2;2,3;3,4;4,5;5,6;6,1;7,2;3,8;8,9;9,7];
% edges 1 and 7 correspond to the two sides of the plate
geometry{1} = [1,2,3,4,5,6]; % the top part of the rectangular plate
geometry{2} = [7,2,8,9,10];  % the bottom part of the rectangular plate
gflag = [1,1];
kappa{1} = '[1 0 0; 0 1 0; 0 0 0]'; %
kappa{2} = '[1 0 0; 0 1 0; 0 0 0]'; % 
muref = [1,1];
mu_min = [0.5,0.5];
mu_max = [2.0,2.0];
mu_bar = [1,1];
nload = '[4,0,0,1;5,0,0,1]';
% Heat flux is added in at the top
dirichlet = '[9,0]';
% Ambient temperature at the bottom

outputname = 'heatstrip'
oload = '[5,1]';  
% Corresponding to the average temperature over a strip [0 2] of the top surface. 

initsol='[0]';
theta_u='[1]';
% zero initial data

nt = 50;            % Number of timesteps  
dt = 0.05;          % Timestep size
% Finally, to complete the problem formulation, we specify that the starting time is 0 
% and that we want to study the heat conduction in 2.5 seconds.


%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))
%%%%%%%%%% End of the offline execution. 

% Now the user can perform the online stage to rapidly simulate the problem 
% for different parameter values. See the software documentation for details. 

