clear all;

% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%

% In the following example, a heat transfer problem with differing material 
% parameters is solved.

% The 2-D domain consists of a square with an embedded curvy region. The square region 
% consists of a material with coefficient of heat conduction of 1 and a density of 1. 
% The curvy region contains a uniform heat source of 4, and it has a coefficient of heat 
% conduction of mu2 and a density of 1. Both regions have a heat capacity of 1.
% The curvy region has radius mu1.

probname = 'tdepcurvy'
plotdemo = 0;
% femOPT.codes = 'HB';
% femOPT.maxsize = 1000;
% femOPT.refine = 'subdivision';

points = '[0,0;mu1,0;5,0;5,5;0,5;0,mu1]';
% mu1 is the radius of the curvy region.
edge = [1,2;2,3;3,4;4,5;5,6;6,1;2,6];
geometry{1} = [2,3,4,5,7]; 
geometry{2} = [1,6,7];     % curvy region
gflag = [1,1];
kappa{1} = '[1 0 0; 0, 1, 0; 0 0 0]';
kappa{2} = '[mu2 0 0; 0, mu2, 0; 0 0 0]'; 
% mu2 is the conductivity of the curvy region
muref = [1,0.3];
mu_min = [0.5,0.1];
mu_max = [2.0,0.5];
mu_bar = [1,0.3];
dirichlet = '[3,0;4,0]'; 
% T=0 on the far field boundary
fareaload='[2,4]';   
% Radioactive heat source f=4 is applied on the curvy region
curvedat = '[0,0,mu1,mu1,0,cos(pi/2*t),t]';
tarclist = '[7,1,0,1]';
% Curvy line 

outputname = 'average'
oareaload = '[2,1]';
% Output is the average temparature of the curvy region

initsol = '[0]'; 
theta_u = '[1]';
% The temperature is 0 degrees at the time we start applying heat.
    
nt = 50;         % Number of timesteps  
dt = .1;         % Timestep size
% Finally, to complete the problem formulation, we specify that the starting time is 0 
% and that we want to study the heat distribution during the first five seconds.

%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))
%%%%%%%%%% End of the offline execution. 

% Now the user can perform the online stage to rapidly simulate the problem 
% for different parameter values. See the software documentation for details. 


