clear;

% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%

% We study time-dependent advection-diffusion in a rectangular channel characterized by the
% length of the channel and Peclet number. Velocity field U = 10*y*(1.0-y) (parabolic profile)
% in x direction and V = 0 in y direction. It simulates a simple scalar Graezt flow (see Arpaci). 
% See rbU_graetz.m for the steady version of this problem.

probname = 'tdepgraetz'
probtype = 'TH';
plotdemo = 0;
femOPT.maxsize=600;

points = '[0,0;1,0;1+mu1,0;1+mu1,1.0;1,1.0;0,1.0]';
% mu1 is the length of the channel. Note there is a free zone (a square region) before the
% fluid enters the channel
edge = [1,2;2,3;3,4;4,5;5,6;6,1];
geometry{1} = [1,2,3,4,5,6];
gflag = [1];
kappa{1} = '[1/mu2, 0, 0; 0, 1/mu2, 0; 10*y*(1.0-y), 0, 0]';
% mu2 is the Peclet number
muref = [1,1];
mu_min = [1,0.1];
mu_max = [10,10];
mu_bar = [1,1];
dirichlet = '[1,0;2,1;4,1;5,0;6,0]';
% Zero Dirichlet BCs on all sides of the free zone. Temperature is set to 1 
% on the top and bottom of the channel. Homogeneous Neumann BC on other sides.

outputname = 'average'
oareaload = '[1,1]';
% Output is the average temperature

initsol = '[0]'; 
theta_u = '[1]';
% The temperature is initially set 0 degrees.
    
nt = 50;          % Number of timesteps  
dt = 0.05;        % Timestep size
% Finally, to complete the problem formulation, we specify that the starting time is 0 
% and that we want to study the heat distribution during the first 2.5 seconds.

%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))
%%%%%%%%%% End of the offline execution. 

% Now the user can perform the online stage to rapidly simulate the problem 
% for different parameter values. See the software documentation for details. 


