clear all;
% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%

% This example studies a heated metal block with a rectangular crack or cavity. 
% The left side of the block is heated to 100 degrees centigrade. At the right side
% of the metal block, heat is flowing from the block to the surrounding air at 
% a constant rate. All the other block boundaries are isolated. 

probname = 'tdepmetal'
plotdemo = 0;
femOPT.refine = 'subdivision';
femOPT.maxsize = 500;

points = '[-0.5,-1.0;0,-1;0.5,-1.0;0.5,0;0.5,1.0;0,1.0;-0.5,1.0;-0.5,0;-mu1/2,-mu2/2;mu1/2,-mu2/2;mu1/2,mu2/2;-mu1/2,mu2/2]';
% The rectangular cavity is parametrized by mu1 and mu2
% mu1 is the width and mu2 is the height, respectively.
edge = [1,2;2,3;3,4;4,5;5,6;6,7;7,8;8,1;9,10;10,11;11,12;12,9];
geometry{1} = [1,2,3,4,5,6,7,8]; % Metal block
geometry{2} = [9,10,11,12]; % Rectangular cavity
gflag = [1,0];
kappa{1} = '[1 0 0; 0, 1, 0; 0 0 0]';
kappa{2} = '[1 0 0; 0, 1, 0; 0 0 0]';
muref = [0.1,0.5];
mu_min = [0.05,0.2];
mu_max = [0.2,0.8];
mu_bar = [0.1,0.5];
dirichlet = '[7,10;8,10]'; 
% T=10 on the left side
nload = '[3,0,0,-1;4,0,0,-1]'; 
% At the right side of the metal block, heat is flowing from the block 
% to the surrounding air at a constant rate.

outputname = 'average'
oareaload = '[1,1]';
% Output is the average temparature of the metal block

initsol = '[0]'; 
theta_u = '[1]';
% The temperature of the block is 0 degrees at the time we start applying heat.
    
nt = 50;         % Number of timesteps  
dt = .1;         % Timestep size
% Finally, to complete the problem formulation, we specify that the starting time is 0 
% and that we want to study the heat distribution during the first five seconds.

%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))
%%%%%%%%%% End of the offline execution. 

% Now the user can perform the online stage to rapidly simulate the problem 
% for different parameter values. See the software documentation for details. 

 