
clear all;
% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%
probname = 'crack3'
% Problem: This problem corresponds to a plate contains two opening edge cracks
% under tension,  assuming isotropic plane stress. The model is modelled as a 
% quarter of the original domain, symmetric boundary conditions are considered.
% mu1: half-length of the crack
% mu2: half-length of the opening crack
probtype = 'LE2';
points = '[0,mu2;mu1,0;1,0;1,2;0,2]';
edge = [1,2;2,3;3,4;4,5;5,1;2,4];
geometry{1} = [1,6,4,5];
geometry{2} = [2,3,6];
gflag = [1,1];
% material properties, Young modulus and Poisson ratio
matprop{1} = '[1, 3/10]';
matprop{2} = '[1, 3/10]';
muref = [0.5,0.6];
mu_min = [0.2,0.0];
mu_max = [0.8,1.2];
mu_bar = [0.5,0.6];
% nload: edge fn ft
% ft is tangential force (aligned with the en, direction is from p1 to p2)
% fn is normal force (cross(en,et)>0))
nload = '[4,1,0]';
% dirichlet: edge lab val
% lab can be "x" and "y"
dirichlet = '[2,y,0;3,x,0]';

% Output: The output is the compliant output of the system. This compliant output can then 
% be used to calculate the energy release  rate and stress intensity factor of the model. 
% For the case there are no opening, analytical stress intensity factors are also 
% available for comparison in Murakami's Stress Intensity Factos's Handbook
outputname = 'compliance'
oload = '[4,1,0]';

 
%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))