clear;
% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%
probname = 'plateaddedmass'
% Added mass of an infinitely thin plate: illustrates in the scalar context
% the proper way to input a crack (with inhomogeneous boundary conditions);
% also represents case in which user should comment out standard mesh
% refines and instead use adaptmesh (go to primal_pdesolution.m, near the
% top).

points = '[0,.0001; 1,0; mu1,0; mu1,mu1; 0,mu1; 0,-.0001; mu1,-mu1; 0,-mu1]';
% points = '[0,1/10000; 1,0; mu1,0; mu1,mu1; 0,mu1; 0,-1/10000; mu1,-mu1; 0,-mu1]';
% Note we exploit symmetry in x_2 and consider only half the domain;
% mu1 is the half--size of the "basin" in which the plate (of half--length
% unity) oscillates.
% To be able to input boundary conditions on the crack, we slightly
% separate the (in fact coincident) two sides of the plate in order to
% create two edges: at x_1 = 0, the two edges are separated by .0001; at
% x_1 = 1 the two edges come together.
% plotdemo = 1;
edge = [1,2;2,3;3,4;4,5;5,1;6,2;3,7;7,8;8,6];
% edges 1 and 6 correspond to the two sides of the plate

% femOPT.codes = 'HB';
% femOPT.maxsize = 1000;
% femOPT.pref = [2,4]; % refine point 2 to "level 4"
% femOPT.lref = [1,2;6,2]; % refine edge 1 and 6 to "level 2"

geometry{1} = [1,2,3,4,5]; % the top part of the basin
geometry{2} = [6,2,7,8,9]; % the bottom part of the basin
gflag = [1,1];
kappa{1} = '[1 0 0; 0 1 0; 0 0 0]';
kappa{2} = '[1 0 0; 0 1 0; 0 0 0]';
muref = [3];
mu_min = [2];
mu_max = [5];
mu_bar = [3];
nload = '[1,0,0,1;6,0,0,-1]';
% corresponds to dpressure/dn = + and - unity (which corresponds to a
% normal velocity of +1)
dirichlet = '[4,0]';
% a free surface condition (pressure = 0)

outputname = 'pforce'

oload = '[1,1;6,-1]';
% corresponds to integral of pressure over plate and hence force exerted by
% fluid on plate

% in limit of mu1 tends to infinity, output should be pi/2 (analytical
% result for added mass)


%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))