clear all;
% License material
%
% -----rbMIT  Software Copyright MIT 2006-07-----
% -----------DBP Huynh, NC Nguyen, AT Patera, G Rozza-------
% 
% The user agrees to rbMIT  MIT usage, distribution, and citation terms and conditions 
% described in the License at http://augustine.mit.edu/rbMIT_SystemLicense.htm.
% rbMIT Software Part_II_and_IV
%

% This heat distribution problem is an example of a 3-D heat problem that 
% is reduced to a 2-D problem by exploiting axis-symmetrical coordinates.

% Consider a cylindrical radioactive rod with radius r and length L. At the left end, 
% heat is continuously added. The right end is kept at a constant temperature. 
% At the outer boundary, heat is exchanged with the surroundings by transfer. 
% At the same time, heat is uniformly produced in the whole rod due to radioactive processes.
% Assume that the initial temperature is zero.

probname = 'tdeprod'
plotdemo = 0;
probtype = 'TH2'; % axis-symmetric flag
femOPT.refine = 'subdivision';
femOPT.maxsize = 500;

points = '[0,0;mu1,0;mu1,mu2;0,mu2]';
% mu1 is the length of the rod
% mu2 is the radius of the rod
edge = [1,2;2,3;3,4;4,1];
geometry{1} = [1,2,3,4];
gflag = [1];
kappa{1} = '[1 0 0; 0, 1, 0; 0 0 0]';
muref = [3,0.4];
mu_min = [2,0.2];
mu_max = [4,0.6];
mu_bar = [3,0.4];
fareaload='[1,5]';   
% Radioactive heat source f=5
nload = '[4,0,0,1;3,1,0,1]'; 
% Heat flux at the left end and Robin condition at the outer boundary
% Homogenuous Neumann condition at the bottom due to symmetry
dirichlet = '[2,1]'; 
% Dirichlet T=1 on the right end

outputname = 'bottom'
oload = '[1,1]';
% Output is the average temparature at the bottom of the rod

initsol = '[0]'; 
theta_u = '[1]';
% The temperature of the rod is 0 degrees at the time we start applying heat.

nt = 50;         % Number of timesteps  
dt = 0.1;        % Timestep size
% Finally, to complete the problem formulation, we specify that the starting time is 0 
% and that we want to study the heat distribution during the first five seconds.

%%%%%%%%%%%% no user input required beyond this point
if ~exist('plotdemo')
    plotdemo = 0;
end

save rbU ;
addpath('../rbMIT_Aux')
copyfile('../rbMIT_Aux/Step1_coer_noncompliant.m',...
    strcat(probname,'_','Step1_coer_noncompliant.m'))
eval(strcat(probname,'_','Step1_coer_noncompliant'))
%%%%%%%%%% End of the offline execution. 

% Now the user can perform the online stage to rapidly simulate the problem 
% for different parameter values. See the software documentation for details. 

